//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Basic dynamic script: Triangle renderer
//

function getDynamicScriptParameters() {

    // Information about the hosts
    local hostList = stringToObject(Application.getInstance().getHostListForSquirrel());
    local displayHost = ENVIRONMENTVARIABLES.rawin("commanderHost") ? ENVIRONMENTVARIABLES.commanderHost : (hostList.len() ? hostList[0] : null);
    local renderHostCountList = [];
    local renderHostCount = displayHost ? hostList.len()-1 : hostList.len();
    for(local i=1; i<=renderHostCount; i++) renderHostCountList.append(i);

    // Screen properties
    local screenWidth = ENVIRONMENTVARIABLES.rawin("screenWidth") ? ENVIRONMENTVARIABLES.screenWidth.tointeger() : 640;
    local screenHeight = ENVIRONMENTVARIABLES.rawin("screenHeight") ? ENVIRONMENTVARIABLES.screenHeight.tointeger() : 480;
    
    // Consider max framesize
    screenWidth = screenWidth * 600 / screenHeight;
    screenHeight = 600;

    return [
        {name="renderHostCount" type="integer" description="Number of render hosts" possibleValues=renderHostCountList defaultValue=(renderHostCount>0 ? renderHostCount : null)}
        {name="displayHost" type="string" description="Name of display host" possibleValues=hostList defaultValue=displayHost}
        {name="frameCount" type="integer" description="Number of frames to render" defaultValue=100}
        {name="frameWidth" type="integer" description="Width of the frame in pixels" defaultValue=screenWidth}
        {name="frameHeight" type="integer" description="Height of the frame in pixels" defaultValue=screenHeight}
        //{name="fullScreen" type="boolean" description="Full-screen mode" defaultValue=true}

        {name="triangleCount" type="integer" description="Number of triangles to render per frame" defaultValue=1000}
        {name="minTriangleCount" type="integer" description="Minimum number of triangles to render per frame" defaultValue=null}
        {name="renderMode" type="string" description="Rendering mode: immediate mode (im), vertex arrays (va), or display lists (dl)" possibleValues=["im", "va", "dl"] defaultValue="im"}
        {name="compositeMode" type="string" description="Compositing mode: depth compositing (de) or alpha compositing (al) without soring" possibleValues=["de", "al"] defaultValue="de"}
        {name="lightCount" type="integer" description="Number of point lights" defaultValue=1}

        {name="glFrameletEXT" type="boolean" description="Use OpenGL framelet extension" defaultValue=false}

        {name="exportFrameStep" type="integer" description="Export every (exportFrameStep)th frame to a png file" defaultValue="0"}
    ];
}

function createLowLevelScript(clusterStr) {

    // Check for enough render hosts
    if(!DYNAMICSCRIPTPARAMETERS.rawin("renderHostCount")) throw "There are no rendering hosts. Please add more hosts to the framework.";

    // Check parameter compatibility
    if(DYNAMICSCRIPTPARAMETERS.rawin("minTriangleCount") && DYNAMICSCRIPTPARAMETERS.renderMode == "dl") throw "Display lists are not compatible with incremental triganle rendering.";

    // Convert parameter to table
    local cluster = stringToObject(clusterStr);
     
    // Create empty structure
    local lowLevelScript = createEmptyLowLevelScript("Triangle renderer", cluster);

    // Counts for id generation
    local nodeCount = 0;
    local processCount = 0;
    local bufferCount = 0;

    // Benchmark duration (0 means endless rendering)
    local numberOfFrames = DYNAMICSCRIPTPARAMETERS.frameCount;
	
    // Setup context
    lowLevelScript.contexts.append({
	glFrameletEXT = DYNAMICSCRIPTPARAMETERS.glFrameletEXT
	
	frameWidth = DYNAMICSCRIPTPARAMETERS.frameWidth
	frameHeight = DYNAMICSCRIPTPARAMETERS.frameHeight

        colourFormat = PC_PF_BGRA8
        depthFormat = PC_PF_Z32I
				
        compositeType = (DYNAMICSCRIPTPARAMETERS.compositeMode == "de" ? PC_COMP_DEPTH : PC_COMP_ALPHA_SORT)
        compressionHint = PC_COMPRESSION_NONE
								
        retainOutputCount = 1
        volatileFrameletCount = 0
        outputDepth = false

        networkID = PC_ID_DEFAULT

	// No processes by default
        processes = []
    });

    // Get context
    local context = lowLevelScript.contexts[0];

    local processIndex = 0;

    // Create rendering process with buffer on each host
    foreach(host in lowLevelScript.hosts) {

	// Skip display hosts
	if(DYNAMICSCRIPTPARAMETERS.rawin("displayHost") && host.name==DYNAMICSCRIPTPARAMETERS.displayHost)
	    continue;

	// Skip the hosts above the specified render host count
	if(nodeCount >= DYNAMICSCRIPTPARAMETERS.renderHostCount)
	    break;

	// Create buffer
	local buffer = {
    	    name = "Buffer #" + bufferCount
	    left = 0
	    top = 0
	    width = context.frameWidth
	    height = context.frameHeight
	    depth = PC_PF_Z32I
	}
	bufferCount++

	// Assemble parameter dependent code fragments
	local initCodeFragment = ""
	local renderCodeFragment = ""

	local thisTriangleCount = DYNAMICSCRIPTPARAMETERS.triangleCount/DYNAMICSCRIPTPARAMETERS.renderHostCount
	local initJob = @"
	    // Generate model
	    triangles<-RE.generateRandomTriangles(3, "+thisTriangleCount+@");

	    // Setup lighting
	    local lightCount = "+DYNAMICSCRIPTPARAMETERS.lightCount+@"
	    for(local i=0; i<lightCount; i++) {
		RE.setAmbientLight(0.1, 0.1, 0.1, 1.0);

		local light = RE.addLightSource();
		local color = getUniqueInverseColor(i, lightCount)
		local position = getUniqueColor(i, lightCount)

		RE.setLightSourcePosition(light, 10.0*position[0], 10.0*position[1], 10.0*position[2]);
		RE.setLightSourceDiffuse(light, color[0], color[1], color[2], 1.0);
	    }
	"
	local setColorJob = @"
	    local color = getUniqueColor("+processIndex+@", "+DYNAMICSCRIPTPARAMETERS.renderHostCount+@")
	    
	    RE.setAmbientMaterial(0.1, 0.1, 0.1, 0.0)
	    RE.setColor(color[0], color[1], color[2], 0.01)
	    RE.setDiffuseMaterial(color[0], color[1], color[2], 0.01)
	    RE.setSpecularMaterial(1.0, 1.0, 1.0, 0.01, 100)
	    "+(DYNAMICSCRIPTPARAMETERS.compositeMode == "al" ? "RE.setBlending(true);" : "")+@"
	"

	// Rendering jobs
	local renderJobImmediate = "";
	local renderJobVertexArray = "";

	// Incremental rendering mode handling
	if(DYNAMICSCRIPTPARAMETERS.rawin("minTriangleCount")) {
	    local minTri = (DYNAMICSCRIPTPARAMETERS.minTriangleCount/DYNAMICSCRIPTPARAMETERS.renderHostCount).tofloat();
	    local maxTri = (thisTriangleCount).tofloat();
	    local frameCount = DYNAMICSCRIPTPARAMETERS.frameCount.tofloat();
	    local triInc = (maxTri-minTri)/frameCount;

	    renderJobImmediate = @"
		"+setColorJob+@"
		RE.renderObjectTriangles(triangles, false, ("+minTri+" + frame * "+triInc+@").tointeger());
	    "
	    renderJobVertexArray = @"
		"+setColorJob+@"
		RE.renderObjectTriangles(triangles, true, ("+minTri+" + frame * "+triInc+@").tointeger());
	    "
	}

	// Constant triangle count
	else {
	    renderJobImmediate = @"
		"+setColorJob+@"
		RE.renderObject(triangles, false);
	    "
	    renderJobVertexArray = @"
		"+setColorJob+@"
		RE.renderObject(triangles, true);
	    "
	}

    	switch(DYNAMICSCRIPTPARAMETERS.renderMode) {
	    case "im":
		initCodeFragment =@"function initProc() { 
		    "+initJob+@"
		}"
		renderCodeFragment = renderJobImmediate;
	    break;
	    case "va":
		initCodeFragment =@"function initProc() { 
		    "+initJob+@"
		}"
		renderCodeFragment = renderJobVertexArray;
	    break;
	    case "dl":
		initCodeFragment =@"function initProc() { 
		    "+initJob+@"

		    displayList<-RE.createDisplayList();
		    "+renderJobImmediate+@"
		    RE.finishDisplayList();
		}"
		renderCodeFragment = "RE.executeDisplayList(displayList);"
	    break;
	}

	// Create process
	local process = {
            name = "Process #" + processCount++
            type = Process.RENDER
            buffer = buffer.name

            initProc = initCodeFragment

            runningProc = @"function runningProc(time, frame) {
        	time = time.tofloat();
        	frame = frame.tointeger();
		local size = 16.0;

        	RE.perspective(90.0, 0.01, 100.0);

        	RE.setCameraPosition(10.0, 10.0, 10.0);
        	RE.setCameraTarget(0.0, 0.0, 0.0);
        	RE.setCameraUpVector(0.0, 0.0, 1.0);

        	RE.rotate(-time * 15.0, 0.0, 0.0, 1.0);
		RE.translate(-0.5*size, -0.5*size, -0.5*size);
		RE.scale(size, size, size);

		// Here comes the rendering job
		"+renderCodeFragment+@"
            }"
	    
	    sortOrder = processCount

            // Set stop frame ID
	    stopID = numberOfFrames
	}
	processIndex++;
	context.processes.append(process.name);
	
	// Create node
	local node = {
	    name = "Node #" + nodeCount++
	    buffers = [ buffer ]
	    processes = [ process ]
	};
	host.nodes.append(node);
    }

    // Create compositing process
    {
	// Compositing host by default on the first host
	local host = lowLevelScript.hosts[0];
	
	if(DYNAMICSCRIPTPARAMETERS.rawin("displayHost")) {
	    // If it is possible, try to place it one the commander
	    host = findItemByName(lowLevelScript.hosts, DYNAMICSCRIPTPARAMETERS.displayHost);
	    if(host == null) throw "Host `" + name +"\' not found."
	}
    
	// Create buffer
	local buffer = {
    	    name = "Buffer #" + bufferCount++
	    left = 0
	    top = 0
	    width = context.frameWidth
	    height = context.frameHeight
	    depth = PC_PF_Z32I
	}
    
	// Create process
	local process = {
            name = lowLevelScript.name
            type = Process.COMPOSITE
            buffer = buffer.name
	    display = true

            // Set stop frame ID
	    stopID = numberOfFrames
	    
	    // Exporting frames
	    exportFrameStep = DYNAMICSCRIPTPARAMETERS.exportFrameStep
	    frameFilenamePattern = "datascal-%06d.png"
	}
	processCount++
	context.processes.append(process.name);

	// Create node
	local node = {
	    name = "Node #" + nodeCount++
	    buffers = [ buffer ]
	    processes = [ process ]
	};
	host.nodes.append(node);
    }

    return objectToString(lowLevelScript);
}
