//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Host initialization
//

function getHostScript(hostName, lowLevelScript) {
    foreach(hostScript in lowLevelScript.hosts)
	if(hostScript.name == hostName) return hostScript;
    
    throw "Host `" + hostName + "\' not found in low level script.";
}

function findRelatedContexts(hostScript, lowLevelScript) {

    // Get list of processes on this host
    local processList = [];
    foreach(nodeScript in hostScript.nodes)
	foreach(processScript in nodeScript.processes)
	    processList.append(processScript.name);

    // Get contexts containing one of the processes in processList
    local contextScripts = [];
    foreach(contextScript in lowLevelScript.contexts)
	foreach(processName in contextScript.processes)
	    if(hasElement(processList, processName)) contextScripts.append(contextScript);
	
    return contextScripts;
}

function findRelatedContext(processName, contextScripts) {
    foreach(contextScript in contextScripts)
	if(hasElement(contextScript.processes, processName)) return contextScript;

    throw "Cannot find corresponding context to process `" + processName + "\'.";
}

function calculateLocalIDName(processName, lowLevelScript) {
    foreach(hostScript in lowLevelScript.hosts) {
    
	local processNumber = 0;
    
	foreach(nodeScript in hostScript.nodes)
	    foreach(processScript in nodeScript.processes) {
		if(processScript.name == processName) return hostScript.name + ":" + processNumber;
		processNumber++;
	    }
    }
    
    throw "Cannot resolve localid name of process `" + processName + "\'.";
}

function calculateLocalIDNameLists(lowLevelScript) {
    foreach(contextScript in lowLevelScript.contexts) {
	contextScript.processesLocalID <- [];
	foreach(processName in contextScript.processes)
	    contextScript.processesLocalID.append(calculateLocalIDName(processName, lowLevelScript));
    }
}

function createNode(host, contextScripts, nodeScript) {
    local node = host.createNode(nodeScript.name);
    
    // Create buffers
    foreach(bufferScript in nodeScript.buffers)
	createBuffer(node, bufferScript);
	
    // Create processes
    foreach(processScript in nodeScript.processes) {
	local contextScript = findRelatedContext(processScript.name, contextScripts);
    	createProcess(node, contextScript, processScript);
    }
}

function createBuffer(node, bufferScript) {
    node.createBuffer(bufferScript.name, bufferScript.left, bufferScript.top, bufferScript.width, bufferScript.height, bufferScript.depth);
}

function createProcess(node, contextScript, processScript) {

    local process = node.createProcess(processScript.name);

    process.setProcessTypeSq(processScript.type);
    process.setBufferByName(processScript.buffer);

    // Set process code if specified
    if(processScript.rawin("initProc"))
	process.setInitProcCode(processScript.initProc);
    if(processScript.rawin("runningProc"))
	process.setRunningProcCode(processScript.runningProc);

    // Set sort order if specified
    if(processScript.rawin("sortOrder"))
	process.setSortOrder(processScript.sortOrder);

    // Set stop frame-id if specified
    if(processScript.rawin("stopID"))
	process.setStopID(processScript.stopID);

    // Set display output flag, if specified
    if(processScript.rawin("display"))
	process.setDisplayOutput(processScript.display);

    // Set operation flag, if specified
    if(processScript.rawin("operate"))
	process.setOperate(processScript.operate);

    // Set statistics gathering flag, if specified
    if(processScript.rawin("statistics"))
	process.setGatherStatistics(processScript.statistics);

    // Set export frame count if specified
    if(processScript.rawin("exportFrameStep"))
	process.setExportFrameStep(processScript.exportFrameStep.tointeger());

    // Set export frame filename pattern if specified
    if(processScript.rawin("frameFilenamePattern"))
	process.setFrameFilenamePattern(processScript.frameFilenamePattern);

    // Setup context
    local context = process.addContext();
    setupContext(context, contextScript, processScript);
}

function setupContext(context, contextScript, processScript) {

    // Whether use OpenGL framlet extension
    context.setUseGLFrameletEXT(contextScript.glFrameletEXT);
	
    // Process is master in the context
    if(processScript.name == contextScript.processes[0]) {
	context.setContextTypeSq(Context.MASTER);
	context.setFrameWidth(contextScript.frameWidth);
	context.setFrameHeight(contextScript.frameHeight);
	context.setColourFormat(contextScript.colourFormat);
	context.setDepthFormat(contextScript.depthFormat);
	context.setCompositeType(contextScript.compositeType);
	context.setCompressionHint(contextScript.compressionHint);
	context.setRetainOutputCount(contextScript.retainOutputCount);
	context.setVolatileFrameletCount(contextScript.volatileFrameletCount);
	context.setOutputDepth(contextScript.outputDepth);
	context.setNetworkID(contextScript.networkID);
    }
	
    // Process is slave in the context
    else
	context.setContextTypeSq(Context.SLAVE);

    // Create host:id list separated by semicolons
    local processList = "";
    foreach(processName in contextScript.processesLocalID)
	processList += ";" + processName;
    processList = processList.slice(1);

    context.setProcesses(processList);
    context.setProcessIndex(getIndex(contextScript.processes, processScript.name));
}

function main(hostName, lowLevelScriptStr) {
    // Convert parameter to table
    local lowLevelScript = compilestring("return " + lowLevelScriptStr)();

    // Calculate host:id name list for contexts
    calculateLocalIDNameLists(lowLevelScript);

    // Get host and host script
    local host = Host.getInstance();
    local hostScript = getHostScript(hostName, lowLevelScript);

    // Find contexts related to this host
    local contextScripts = findRelatedContexts(hostScript, lowLevelScript);

    // Create nodes
    foreach(nodeScript in hostScript.nodes)
	createNode(host, contextScripts, nodeScript);
}
