
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMApplication.h
/// Contains declaration of class Application

#ifndef __PCM_APPLICATION_H__
#  define __PCM_APPLICATION_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMSingleton.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMPointer.h"

#  include "PCMNetServer.h"

#  include "PCMNetClient.h"

#  include "PCMStringConverter.h"

#  include "PCMCluster.h"

#  include "PCMHost.h"

#  include "PCMHostInfo.h"

#  include "PCMConfigOptions.h"

#  include "PCMPluginManager.h"

//
// Outer includes
//

#  include <readline/readline.h>

#  include <readline/history.h>

//
// Scripting bind includes (for Squirrel)
//

#  include <squirrel/squirrel.h>
#  include <squirrel/sqstdio.h>
#  include <squirrel/sqstdaux.h>
#  include <squirrel/sqplus.h>

namespace ParCompMark
{

  /**

	 This singleton class handles the application initializing, command line parsing, starting tasks etc.

   */

  class Application:public Singleton < Application >
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestApplication;
#  endif

	//
	// Scripting binding (for Squirrel)
	//

	/// @name Scripting binding
	//@{  

  /**
   Static glue code method for Squirrel-C++ binding. 
   This method should be call from each Squirrel virtual machines 
   to generate the proper Squirrel glue code.
   To content of this method is fully autogenerated in the header implementation file, 
   you should not modify it.
   */
   public:static void squirrelGlue()
	{
	 ::SqPlus::SQClassDef < Application > (_T("Application")).staticFunc(&Application::getInstance,
										_T("getInstance")).
		func(&Application::loadLowLevelScript, _T("loadLowLevelScript")).func(&Application::loadDynamicScript,
										  _T("loadDynamicScript")).
		func(&Application::loadScenarioScript, _T("loadScenarioScript")).func(&Application::pushSquirrelCommand,
										  _T("pushSquirrelCommand")).
		func(&Application::quit, _T("quit")).func(&Application::compileDynamic,
							_T("compileDynamic")).func(&Application::start,
										  _T("start")).
		func(&Application::stop, _T("stop")).func(&Application::autoDetection,
							_T("autoDetection")).func(&Application::cleanup,
										 _T("cleanup")).
		func(&Application::getHostListForSquirrel,
		  _T("getHostListForSquirrel")).func(&Application::getLowLevelScriptForSquirrel,
							_T("getLowLevelScriptForSquirrel")).
		func(&Application::getDynamicScriptForSquirrel,
		  _T("getDynamicScriptForSquirrel")).func(&Application::hasDynamicScript,
							  _T("hasDynamicScript")).
		func(&Application::hasScenarioScript,
		  _T("hasScenarioScript")).func(&Application::getDynamicScriptParametersForSquirrel,
						 _T("getDynamicScriptParametersForSquirrel")).
		func(&Application::getProgramInfo, _T("getProgramInfo")).staticFunc(&Application::getExpectedHostCount,
										 _T("getExpectedHostCount")).
		func(&Application::getDynamicScriptCompiled,
		  _T("getDynamicScriptCompiled")).func(&Application::setDynamicScriptCompiled,
							_T("setDynamicScriptCompiled")).func(&Application::getCompRun,
											   _T("getCompRun"));
	}

	//@}  

	//
	// Enumerations
	//

   protected:

  /** Definitions of for post command tasks. Post command task is a job to execute after stopping the user interface Squirrel VM. */
	enum PostCommand
	{

	/** Nothing to do */
	 NOTHING,

	/** Load scenario script */
	 LOADSCENARIO,

	/** Load dynamic script */
	 LOADDYNAMIC,

	/** Compile dynamic script */
	 COMPILE,

	/** Start compositing */
	 START,

	/** Compile dynamic script and start compositing */
	 COMPILE_START,

	/** Stop compositing */
	 STOP,

	/** Quit from application */
	 QUIT,

	/** Stop compositing and quit from application */
	 STOP_QUIT
	};

  /** Definitions of for post command tasks. Post command task is a job to execute after stopping the user interface Squirrel VM. */
	enum DynamicScriptParameterType
	{

	/** String type */
	 STRING = 0,

	/** Integer type */
	 INTEGER = 1,

	/** Floating point type */
	 REAL = 2,

	/** Boolean type */
	 BOOL = 3
	};

  /** Definitions of for user interface types. */
	enum UserInterface
	{

	/** Do not provide user interface. This is used for batch jobs */
	 NONE,

	/** Textual user interface using GNU readline() or gets() */
	 CONSOLE
	};

	//
	// Typedefs
	//

   protected:

  /** Structure for command line options. */
	typedef struct

	{		/** Short name */

	 s8 shortName;					  /** Long name */

	  std::string longName;										   /** Description */

	  std::string description;																	  /** The option has an argument */

	 bool hasArgument;																											/** Argument handler method */
	 void (*handler) (const std::string &);
	} CommandLineOption;

  /** Structure for dynamic script parameters. */
	typedef struct

	{		/** Name of the parameter */

	 std::string name;							 /** Parameter type */

	 DynamicScriptParameterType type;													  /** Description of the parameter */

	  std::string description;																							  /** Textual representation of the possible values (can be any value if the set is empty) */

	  std::set < std::string > possibleValues;																																								 /** Textual representation of the default value of the parameter */
	  std::string defaultValue;
	} DynamicScriptParameter;

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** PC API Squirrel script file (relative to the data directory).

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string PCAPINUT;

	/** ParCompMark API Squirrel script file (relative to the data directory).

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string PARCOMPMARKNUT;

	/** Common utilities Squirrel script file (relative to the data directory).

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string UTILSNUT;

	/** User command abbreviations resolver Squirrel script file (relative to the data directory).

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string ABBREVIATIONSNUT;

	/** Dynamic script initialization Squirrel script file (relative to the data directory).

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string DYNAMICINITNUT;

	/** User interface initialization Squirrel script file (relative to the data directory).

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string UIINITNUT;

	/** Directory that contatins user commands in separated nut files (relative to the data directory).

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string COMMANDDIRECTORY;

	/** Path of help.nut file in user commands directory (relative to the data directory).

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string HELPNUT;

	/** List of supported PC implementation libraries.

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string PCIMPLEMENTATIONS[];

   protected:

	/** Command line options for ParCompMark.

	 @remarks
	 This is own attribute of this class.
	*/
	static const Application::CommandLineOption mCommandLineOptions[];

	/** Number of command line options.

	 @remarks
	 This is own attribute of this class.
	*/
	static const u32 mCommandLineOptionCount;

	/** Application usage string.

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string mUsageString;

	//@}  

	//
	// Class variables
	//

	/// @name Class variables
	//@{  

   protected:

	/** PC dynamic library search path (default /usr/lib).

	 @remarks
	 This is own attribute of this class.
	*/
	static std::string mLibrarySearchPath;

	/** Indicates commander mode (default false).

	 @remarks
	 This is own attribute of this class.
	*/
	static bool mCommanderMode;

	/** Indicates GUI mode (default false).

	 @remarks
	 This is own attribute of this class.
	*/
	static bool mGUIMode;

	/** Indicates manual cluster description (default false).

	 @remarks
	 This is own attribute of this class.
	*/
	static bool mManualClusterDescription;

	/** Cluster description file name.

	 @remarks
	 This is own attribute of this class.
	*/
	static std::string mClusterDescription;

	/** Indicates low-level scripting mode (default false).

	 @remarks
	 This is own attribute of this class.
	*/
	static bool mLowLevelMode;

	/** Indicates interactive parameter settings (default true).

	 @remarks
	 This is own attribute of this class.
	*/
	static bool mInteractiveParameters;

	/** Parameters description file name.

	 @remarks
	 This is own attribute of this class.
	*/
	static std::string mParameters;

	/** Input script file name.

	 @remarks
	 This is own attribute of this class.
	*/
	static std::string mInput;

	/** Output file name.

	 @remarks
	 This is own attribute of this class.
	*/
	static std::string mOutput;

	/** Expected number of rendering hosts.

	 @remarks
	 This is own attribute of this class.
	*/
	static int mExpectedHostCount;

	/** Type of user interface.

	 @remarks
	 This is own attribute of this class.
	*/
	static UserInterface mUserInterface;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Information about the host. Soldire opertaion fills it.

	 @remarks
	 This is own attribute of this class.
	*/
	HostInfo * mHostInfo;

	/** The application is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** Current post command task.

	 @remarks
	 This is own attribute of this class.
	*/
	PostCommand mPostCommand;

	/** Flag indicating that the application is quitting now (to avoid multiple quit requests in quit() method).

	 @remarks
	 This is own attribute of this class.
	*/
	bool mQuiting;

	/** Flag indicating that the output document has been written now and it is safe to quit.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mOutputIsDone;

	/** Root of the output document.

	 @remarks
	 This is own attribute of this class.
	*/
	OutputNode::Pointer mOutputDocument;

	/** The output document has separate 'execution' children for each benchmark execution. This pointer addresses the actual execution.

	 @remarks
	 This is own attribute of this class.
	*/
	OutputNode::Pointer mCurrentExecutionOutputDocument;

	/** Index of the actual execution.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mExecutionIndex;

	/** Root of the output document.

	 @remarks
	 This is own attribute of this class.
	*/
	OutputNode::Pointer mCollectClusterDescription;

	/** Program configuration options.

	 @remarks
	 This is own attribute of this class.
	*/
	ConfigOptions::Pointer mConfigOptions;

	/** Squirrel low-level script.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mLowLevelScript;

	/** Squirrel cluster dynamic script.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mDynamicScript;

	/** Squirrel scenario script.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mScenarioScript;

	/** List of Squirrel commands to execute. If this list is not empty, the user interface take and 
			executes these commands instead of putting command prompt. Moreover, you can put commands to this list from Squirrel scripts, too.
			This is useful for example scenario scripts.

	 @remarks
	 This is own attribute of this class.
	*/
	std::list < std::string > mSquirrelCommandList;

	/** Indicates that the dynamic script is compiled.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mDynamicScriptCompiled;

	/** Server network.

	 @remarks
	 This is own attribute of this class.
	*/
	NetServer *mServer;

	/** Client network.

	 @remarks
	 This is own attribute of this class.
	*/
	NetClient *mClient;

	/** Output file name.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mCompRun;

	/** Squirrel virtual machine for dynamic script execution.

	 @remarks
	 This is own attribute of this class.
	*/
	SqVM::Pointer mDynamicScriptSqVM;

	/** Squirrel virtual machine to handle user interaction.

	 @remarks
	 This is own attribute of this class.
	*/
	SqVM::Pointer mUserInterfaceSqVM;

	/** List of dynamic script parameters.

	 @remarks
	 This is own attribute of this class.
	*/
	std::list < DynamicScriptParameter * >mDynamicScriptParameters;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor. 
	 */
	Application();

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Application();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mLibrarySearchPath. Returns value of mLibrarySearchPath.

	 @return The value of mLibrarySearchPath
	*/
	static const std::string & getLibrarySearchPath();

	/**
	 Setter of mLibrarySearchPath. Sets value of mLibrarySearchPath.

	 @param[in] librarysearchpath The value of mLibrarySearchPath
	*/
	static void setLibrarySearchPath(const std::string & librarysearchpath);

	/**
	 Getter of mHostInfo. Returns value of mHostInfo.

	 @return The value of mHostInfo
	*/
	HostInfo *getHostInfo() const;

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mPostCommand. Returns value of mPostCommand.

	 @return The value of mPostCommand
	*/
	const Application::PostCommand & getPostCommand() const;

	/**
	 Getter of mQuiting. Returns value of mQuiting.

	 @return The value of mQuiting
	*/
	const bool & getQuiting() const;

	/**
	 Getter of mOutputIsDone. Returns value of mOutputIsDone.

	 @return The value of mOutputIsDone
	*/
	const bool & getOutputIsDone() const;

	/**
	 Getter of mOutputDocument. Returns value of mOutputDocument.

	 @return The value of mOutputDocument
	*/
	OutputNode::Pointer & getOutputDocument();

	/**
	 Getter of mCurrentExecutionOutputDocument. Returns value of mCurrentExecutionOutputDocument.

	 @return The value of mCurrentExecutionOutputDocument
	*/
	OutputNode::Pointer & getCurrentExecutionOutputDocument();

	/**
	 Getter of mExecutionIndex. Returns value of mExecutionIndex.

	 @return The value of mExecutionIndex
	*/
	u32 & getExecutionIndex();

	/**
	 Getter of mCollectClusterDescription. Returns value of mCollectClusterDescription.

	 @return The value of mCollectClusterDescription
	*/
	OutputNode::Pointer & getCollectClusterDescription();

	/**
	 Getter of mConfigOptions. Returns value of mConfigOptions.

	 @return The value of mConfigOptions
	*/
	ConfigOptions::Pointer & getConfigOptions();

	/**
	 Getter of mLowLevelScript. Returns value of mLowLevelScript.

	 @return The value of mLowLevelScript
	*/
	const std::string & getLowLevelScript() const;

	/**
	 Getter of mDynamicScript. Returns value of mDynamicScript.

	 @return The value of mDynamicScript
	*/
	const std::string & getDynamicScript() const;

	/**
	 Getter of mScenarioScript. Returns value of mScenarioScript.

	 @return The value of mScenarioScript
	*/
	const std::string & getScenarioScript() const;

	/**
	 Getter of mSquirrelCommandList. Returns value of mSquirrelCommandList.

	 @return The value of mSquirrelCommandList
	*/
	const std::list < std::string > &getSquirrelCommandList() const;

	/**
	 Getter of mDynamicScriptCompiled. Returns value of mDynamicScriptCompiled.

	 @return The value of mDynamicScriptCompiled
	*/
	bool & getDynamicScriptCompiled();

	/**
	 Setter of mDynamicScriptCompiled. Sets value of mDynamicScriptCompiled.

	 @param[in] dynamicscriptcompiled The value of mDynamicScriptCompiled
	*/
	void setDynamicScriptCompiled(const bool dynamicscriptcompiled);

	/**
	 Getter of mServer. Returns value of mServer.

	 @return The value of mServer
	*/
	NetServer *getServer() const;

	/**
	 Getter of mClient. Returns value of mClient.

	 @return The value of mClient
	*/
	NetClient *getClient() const;

	/**
	 Getter of mUsageString. Returns value of mUsageString.

	 @return The value of mUsageString
	*/
	static const std::string & getUsageString();

	/**
	 Getter of mCommanderMode. Returns value of mCommanderMode.

	 @return The value of mCommanderMode
	*/
	static const bool & getCommanderMode();

	/**
	 Getter of mGUIMode. Returns value of mGUIMode.

	 @return The value of mGUIMode
	*/
	static const bool & getGUIMode();

	/**
	 Getter of mManualClusterDescription. Returns value of mManualClusterDescription.

	 @return The value of mManualClusterDescription
	*/
	static const bool & getManualClusterDescription();

	/**
	 Getter of mClusterDescription. Returns value of mClusterDescription.

	 @return The value of mClusterDescription
	*/
	static const std::string & getClusterDescription();

	/**
	 Getter of mLowLevelMode. Returns value of mLowLevelMode.

	 @return The value of mLowLevelMode
	*/
	static const bool & getLowLevelMode();

	/**
	 Getter of mInteractiveParameters. Returns value of mInteractiveParameters.

	 @return The value of mInteractiveParameters
	*/
	static const bool & getInteractiveParameters();

	/**
	 Getter of mParameters. Returns value of mParameters.

	 @return The value of mParameters
	*/
	static const std::string & getParameters();

	/**
	 Getter of mInput. Returns value of mInput.

	 @return The value of mInput
	*/
	static const std::string & getInput();

	/**
	 Getter of mOutput. Returns value of mOutput.

	 @return The value of mOutput
	*/
	static const std::string & getOutput();

	/**
	 Getter of mExpectedHostCount. Returns value of mExpectedHostCount.

	 @return The value of mExpectedHostCount
	*/
	static int &getExpectedHostCount();

	/**
	 Getter of mUserInterface. Returns value of mUserInterface.

	 @return The value of mUserInterface
	*/
	static const Application::UserInterface & getUserInterface();

	/**
	 Getter of mCompRun. Returns value of mCompRun.

	 @return The value of mCompRun
	*/
	bool & getCompRun();

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Complement command line commands.
	 @param[in] first Need for correct readline function type.
	 @param[in] second Need for correct readline function type.
	 @return Need for correct readline function type.
	 */
	static int complement(int first, int second);

	/** 
	 Parse ANSI C command line.
	 @param[in] argc Number of command line arguments.
	 @param[in] argv Array of command line arguments.
	 */
	static void parseCommandLine(const u32 & argc, const char **&argv);

	/** 
	 Write help to std out.
	 @param[in] strarg String argument (dummy).
	 */
	static void showHelp(const std::string & strarg);

	/** 
	 Write version to std out.
	 @param[in] strarg String argument (dummy).
	 */
	static void showVersion(const std::string & strarg);

	/** 
	 Set commander mode.
	 @param[in] strarg String argument (dummy).
	 */
	static void setCommanderOn(const std::string & strarg);

	/** 
	 Set GUI mode.
	 @param[in] strarg String argument (dummy).
	 */
	static void setGUIOn(const std::string & strarg);

	/** 
	 Set low-level scripting mode.
	 @param[in] strarg String argument (dummy).
	 */
	static void setLowLevelOn(const std::string & strarg);

	/** 
	 Set low-level script file.
	 @param[in] strarg String argument.
	 */
	static void setLowLevelScriptFile(const std::string & strarg);

	/** 
	 Set dynamic script file.
	 @param[in] strarg String argument.
	 */
	static void setDynamicScriptFile(const std::string & strarg);

	/** 
	 Set scenario script file.
	 @param[in] strarg String argument.
	 */
	static void setScenarioScriptFile(const std::string & strarg);

	/** 
	 Set cluster description file.
	 @param[in] strarg String argument.
	 */
	static void setCluster(const std::string & strarg);

	/** 
	 Set parameters description file.
	 @param[in] strarg String argument.
	 */
	static void setParameters(const std::string & strarg);

	/** 
	 Set input file.
	 @param[in] strarg String argument.
	 */
	static void setInput(const std::string & strarg);

	/** 
	 Set output file.
	 @param[in] strarg String argument.
	 */
	static void setOutput(const std::string & strarg);

	/** 
	 Set expected number of rendering hosts.
	 @param[in] hostCount String argument.
	 */
	static void setExpectedHostCount(const std::string & hostCount);

	/** 
	 Set user the type of interface.
	 @param[in] strUI String argument.
	 */
	static void setUserInterface(const std::string & strUI);

	/** 
	 Returns true if specified environment variable exists.
	 @param[in] name Name of the variable.
	 @return Indicates that the specified environment variable exists.
	 */
	static bool hasEnvironmentVariable(const std::string & name);

	/** 
	 Get environment variable.
	 @param[in] name Name of the variable.
	 @return Value of the specified variable.
	 */
	static std::string getEnvironmentVariable(const std::string & name);

	/** 
	 Abnormal termination handler.
	 */
	static void terminateHandler();

	/** 
	 No operation termination handler. The terminateHandler() method sets this method as Abnormal termination handler during its run.
	 */
	static void terminateHandlerNOP();

	/** 
	 Unexpected exception handler.
	 */
	static void unexpectedHandler();

	/** 
	 Segmentation fault handler.
	 @param[in] value Signal parameter.
	 */
	static void segfaultHandler(int value);

	/** 
	 Interrupt signal handler.
	 @param[in] value Signal parameter.
	 */
	static void interruptHandler(int value);

	/** 
	 Gets the instance of the singleton class. Reimplemented for squirrel usage.
	 @return Instance of the class
	 */
	static Application *getInstance();

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Setup special event handlers.
	 */
	virtual void setupHandlers() const;

	/** 
	 Initialize the PCM application.
	 */
	virtual void initialize();

	/** 
	 Finalize the PCM application.
	 */
	virtual void finalize();

	/** 
	 Addintional finalization code for commander mode. This method is executed at the beginning of the common finalize() method.
	 */
	virtual void finalizeCommander();

	/** 
	 Addintional finalization code for soldier mode. This method is executed at the beginning of the common finalize() method.
	 */
	virtual void finalizeSoldier();

	/** 
	 Call application-wide setters on config options.
	 */
	virtual void setFromConfigOptions();

	/** 
	 Create and initialize virtual machines for dynamic script execution and for user interaction handling.
	 */
	virtual void createVirtualMachines();

	/** 
	 The application starts its operation. The operation depends on the
			commander mode flag.
	 @return Return true on error.
	 */
	virtual bool startOperation();

	/** 
	 Write collected output.
	 */
	virtual void writeOutput();

	/** 
	 Wait until the output is fully written.
	 */
	virtual void waitForOutput();

	/** 
	 Quit from application.
	 */
	virtual void quit();

	/** 
	 Compile dynamic script.
	 */
	virtual void compileDynamic();

	/** 
	 Start compositing.
	 */
	virtual void start();

	/** 
	 Stop compositing.
	 */
	virtual void stop();

	/** 
	 Auto detection of the cluster.
	 */
	virtual void autoDetection();

	/** 
	 Do cleanup. Empty output document etc.
	 */
	virtual void cleanup();

	/** 
	 Return the list of hosts in cluster as a Squirrel string array.
	 @return Host list.
	 */
	virtual char *getHostListForSquirrel();

	/** 
	 Return the low-level script as a string.
	 @return Low-level script.
	 */
	virtual char *getLowLevelScriptForSquirrel();

	/** 
	 Return the dynamic script as a string.
	 @return Dynamic script.
	 */
	virtual char *getDynamicScriptForSquirrel();

	/** 
	 Return true if a dynamic script is loaded.
	 @return Flag indicates that the dynamic script is loaded.
	 */
	virtual bool hasDynamicScript();

	/** 
	 Return true if a scenario script is loaded.
	 @return Flag indicates that the scenario script is loaded.
	 */
	virtual bool hasScenarioScript();

	/** 
	 Return the list of dynamic script parameters as a Squirrel string array.
	 @return Dynamic parameter list.
	 */
	virtual char *getDynamicScriptParametersForSquirrel();

	/** 
	 Return one line program info as a string.
	 @return Program info.
	 */
	virtual char *getProgramInfo();

	/** 
	 Calcualtes the message sendign time.
	 */
	virtual void calculateMessageSendTime();

	/** 
	 Handle pseudoSTOP post command.
	 */
	virtual void _pseudoStop();

   protected:

	/** 
	 Operation of a commander mode application.
	 @return Return true on error.
	 */
	virtual bool commanderOperation();

	/** 
	 Operation of a soldier mode (not commander mode) application.
	 @return Return true on error.
	 */
	virtual bool soldierOperation();

	/** 
	 Start textual user interface.
	 @return Return true on error.
	 */
	virtual bool textUserInterface();

	/** 
	 Start "no" user interface.
	 @return Return true on error.
	 */
	virtual bool noUserInterface();

	/** 
	 Executes a single line user command. User command can be a Squirrel code fragment or an abrevation.
	 @param[in] command User command.
	 @return Return true on error.
	 */
	virtual bool executeUserCommand(const std::string & command);

	/** 
	 Executes the commands in the squirrel command list.
	 @return Return true on error.
	 */
	virtual bool processSquirrelCommandList();

	/** 
	 Executes current post command task.
	 */
	virtual void doPostCommand();

	/** 
	 Load low-level script from the specified filename.
	 @param[in] filename File containing low level script (in application data directory).
	 */
	virtual void loadLowLevelScript(const char *filename);

	/** 
	 Load dynamic script from the specified filename.
	 @param[in] filename File containing dynamic script (in application data directory).
	 */
	virtual void loadDynamicScript(const char *filename);

	/** 
	 Load scenario script from the specified filename.
	 @param[in] filename File containing scenario script (in application data directory).
	 */
	virtual void loadScenarioScript(const char *filename);

	/** 
	 Push a command into the Squirrel command list.
	 @param[in] command Command line to push.
	 */
	virtual void pushSquirrelCommand(const char *command);

	/** 
	 Create low-level script from dynamic script and cluster description.
	 @param[in] clusterDescription Squirrel cluster description.
	 */
	virtual void createLowLevelScript(const std::string & clusterDescription);

	/** 
	 Process scenario script file.
	 */
	virtual void processScenarioScript();

	/** 
	 Set environment variables to the Squirrel.
	 */
	virtual void setEnvironmentVariablesToSquirrel();

	/** 
	 Compile dynamic script and initialize dynamic script parameters in reset or complement mode.
	 @param[in] reset Reset dynamic script parameters.
	 */
	virtual void initializeDynamicScriptParameters(const bool & reset);

	/** 
	 Retrieve dynamic script parameters from the loaded dynamic script.
	 */
	virtual void retrieveDynamicScriptParameters();

	/** 
	 Handle LOADSCENARIO post command.
	 */
	virtual void _loadScenario();

	/** 
	 Handle LOADDYNAMIC post command.
	 */
	virtual void _loadDynamic();

	/** 
	 Handle COMPILEDYNAMIC post command.
	 */
	virtual void _compileDynamic();

	/** 
	 Handle START post command.
	 */
	virtual void _start();

	/** 
	 Handle STOP post command.
	 */
	virtual void _stop();

	/** 
	 Handle QUIT post command.
	 */
	virtual void _quit();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMApplication_impl.h"

}

// Declare Application class (and its enums) for SqPlus binding
/// \skip DECLARE_INSTANCE_TYPE_NAME
DECLARE_INSTANCE_TYPE_NAME(ParCompMark::Application, Application);

#endif
