
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMBuffer.h
/// Contains declaration of class Buffer

#ifndef __PCM_BUFFER_H__
#  define __PCM_BUFFER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMPointer.h"

//
// Outer includes
//

#  include <pcapi.h>

//
// Scripting bind includes (for Squirrel)
//

#  include <squirrel/squirrel.h>
#  include <squirrel/sqstdio.h>
#  include <squirrel/sqstdaux.h>
#  include <squirrel/sqplus.h>

namespace ParCompMark
{

  /**
	Grapics memory buffer. Buffer object can store either colour and depth information of an 
		image (PC result) or a fragment of an image (PC framelet).

   */

  class Buffer
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestBuffer;
#  endif

	//
	// Scripting binding (for Squirrel)
	//

	/// @name Scripting binding
	//@{  

  /**
   Static glue code method for Squirrel-C++ binding. 
   This method should be call from each Squirrel virtual machines 
   to generate the proper Squirrel glue code.
   To content of this method is fully autogenerated in the header implementation file, 
   you should not modify it.
   */
   public:static void squirrelGlue()
	{
	 ::SqPlus::SQClassDef < Buffer > (_T("Buffer")).func(&Buffer::getLeft, _T("getLeft")).func(&Buffer::setLeft,
												  _T("setLeft")).
		func(&Buffer::getTop, _T("getTop")).func(&Buffer::setTop, _T("setTop")).func(&Buffer::getWidth,
											  _T("getWidth")).
		func(&Buffer::setWidth, _T("setWidth")).func(&Buffer::getHeight,
							  _T("getHeight")).func(&Buffer::setHeight,
										 _T("setHeight")).
		func(&Buffer::getDepthFormat, _T("getDepthFormat")).func(&Buffer::setDepthFormat, _T("setDepthFormat"));
	}

	//@}  

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < Buffer,
	Mutex > Pointer;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Parent node of the buffer. (Parent reference is standard pointer to avoid circular reference)

	 @remarks
	 This attribute references an attribute.
	*/
	Node * mParent;

	/** 

	 @remarks
	 This is own attribute of this class.
	*/
	PCuint mLeft;

	/** 

	 @remarks
	 This is own attribute of this class.
	*/
	PCuint mTop;

	/** 

	 @remarks
	 This is own attribute of this class.
	*/
	PCuint mWidth;

	/** 

	 @remarks
	 This is own attribute of this class.
	*/
	PCuint mHeight;

	/** The Buffer is responsible for deallocation of mColour and mDepth.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mOwnPointers;

	/** Colour information buffer (RGBA32 or BGRA32 coded).

	 @remarks
	 This is own attribute of this class.
	*/
	PCuint *mColour;

	/** Depth information buffer (the coding is unknown).

	 @remarks
	 This is own attribute of this class.
	*/
	void *mDepth;

	/** Depth format. Needed for deallocating the buffer.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mDepthFormat;

	/** If the output is not the whole frame, this is the remainde pixels in a row.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mOutputRowPixel;

	/** The buffer is initialized. None of its attribute can be modified after the buffer is initialized except for mOutputRowPixel, mColour, and mDepth.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor for Squirrel compatibility. Calling this constructor always raises an exception.
	 */
	Buffer();

	/** 
	 Create a buffer by initializing its parameters.
	 @param[in] left X offset of the image (it is 0 for whole images).
	 @param[in] top Y offset of the image (it is 0 for whole images).
	 @param[in] width Width of the image.
	 @param[in] height Height of the image.
	 @param[in] depthFormat Depth format used by this image.
	 @param[in] parent Parent node
	 */
	Buffer(const PCuint & left, const PCuint & top, const PCuint & width,
		 const PCuint & height, const PCuint & depthFormat, Node * parent);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Buffer();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mParent. Returns value of mParent.

	 @return The value of mParent
	*/
	Node * getParent() const;

	/**
	 Getter of mLeft. Returns value of mLeft.

	 @return The value of mLeft
	*/
	const PCuint & getLeft() const;

	/**
	 Setter of mLeft. Sets value of mLeft.

	 @param[in] left The value of mLeft
	*/
	void setLeft(const PCuint left);

	/**
	 Getter of mTop. Returns value of mTop.

	 @return The value of mTop
	*/
	const PCuint & getTop() const;

	/**
	 Setter of mTop. Sets value of mTop.

	 @param[in] top The value of mTop
	*/
	void setTop(const PCuint top);

	/**
	 Getter of mWidth. Returns value of mWidth.

	 @return The value of mWidth
	*/
	const PCuint & getWidth() const;

	/**
	 Setter of mWidth. Sets value of mWidth.

	 @param[in] width The value of mWidth
	*/
	void setWidth(const PCuint width);

	/**
	 Getter of mHeight. Returns value of mHeight.

	 @return The value of mHeight
	*/
	const PCuint & getHeight() const;

	/**
	 Setter of mHeight. Sets value of mHeight.

	 @param[in] height The value of mHeight
	*/
	void setHeight(const PCuint height);

	/**
	 Getter of mOwnPointers. Returns value of mOwnPointers.

	 @return The value of mOwnPointers
	*/
	const bool & getOwnPointers() const;

	/**
	 Getter of mColour. Returns value of mColour.

	 @return The value of mColour
	*/
	PCuint *getColour();

	/**
	 Setter of mColour. Sets value of mColour.

	 @param[in] colour The value of mColour
	*/
	void setColour(const PCuint * colour);

	/**
	 Getter of mDepth. Returns value of mDepth.

	 @return The value of mDepth
	*/
	void *getDepth();

	/**
	 Setter of mDepth. Sets value of mDepth.

	 @param[in] depth The value of mDepth
	*/
	void setDepth(const void *depth);

	/**
	 Getter of mDepthFormat. Returns value of mDepthFormat.

	 @return The value of mDepthFormat
	*/
	const PCint & getDepthFormat() const;

	/**
	 Setter of mDepthFormat. Sets value of mDepthFormat.

	 @param[in] depthformat The value of mDepthFormat
	*/
	void setDepthFormat(const PCint depthformat);

	/**
	 Getter of mOutputRowPixel. Returns value of mOutputRowPixel.

	 @return The value of mOutputRowPixel
	*/
	const PCint & getOutputRowPixel() const;

	/**
	 Setter of mOutputRowPixel. Sets value of mOutputRowPixel.

	 @param[in] outputrowpixel The value of mOutputRowPixel
	*/
	void setOutputRowPixel(const PCint & outputrowpixel);

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initialize the buffer.
	 */
	virtual void initialize();

	/** 
	 Finalize the buffer.
	 */
	virtual void finalize();

	/** 
	 Save the image in the buffer to the specified file.
	 @param[in] filename Name of the file.
	 */
	virtual void saveToFile(const std::string & filename);

   protected:

	/** 
	 Deallocate mColour and mDepth buffers.
	 */
	virtual void freeBuffers();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMBuffer_impl.h"

}

// Declare Buffer class (and its enums) for SqPlus binding
/// \skip DECLARE_INSTANCE_TYPE_NAME
DECLARE_INSTANCE_TYPE_NAME(ParCompMark::Buffer, Buffer);

#endif
