
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMCPU.h
/// Contains declaration of class CPU

#ifndef __PCM_CPU_H__
#  define __PCM_CPU_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMOutputNode.h"

#  include "PCMPointer.h"

#  include "PCMOutputNode.h"

#  include "PCMFileSystemManager.h"

//
// Outer includes
//

#  include <string>

#  include <tinyxml/tinyxml.h>

namespace ParCompMark
{

  /**
	Class contain CPU information.
   */

  class CPU:public OutputNode
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestCPU;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef ParCompMark::Pointer < CPU,
	Mutex > Pointer;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Name of the vendor of the CPU.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mVendor;

	/** Name of the CPU model.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mModel;

	/** CPU clock in MHz.

	 @remarks
	 This is own attribute of this class.
	*/
	Real mClock;

	/** CPU cache in KB.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mCache;

	/** BogoMIPS value for the CPU. MIPS is short for Millions of Instructions Per Second. 
			It is a measure for the computation speed of a program.

	 @remarks
	 This is own attribute of this class.
	*/
	Real mBogomips;

	/** CPU flags.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mFlags;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Creates CPU node.
	 @param[in] name Name of the node.
	 @param[in] type Type of the node.
	 */
	CPU(const std::string & name, const OutputNode::NodeType & type);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ CPU();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mVendor. Returns value of mVendor.

	 @return The value of mVendor
	*/
	const std::string & getVendor() const;

	/**
	 Setter of mVendor. Sets value of mVendor.

	 @param[in] vendor The value of mVendor
	*/
	void setVendor(const std::string & vendor);

	/**
	 Getter of mModel. Returns value of mModel.

	 @return The value of mModel
	*/
	const std::string & getModel() const;

	/**
	 Setter of mModel. Sets value of mModel.

	 @param[in] model The value of mModel
	*/
	void setModel(const std::string & model);

	/**
	 Getter of mClock. Returns value of mClock.

	 @return The value of mClock
	*/
	const Real & getClock() const;

	/**
	 Setter of mClock. Sets value of mClock.

	 @param[in] clock The value of mClock
	*/
	void setClock(const Real & clock);

	/**
	 Getter of mCache. Returns value of mCache.

	 @return The value of mCache
	*/
	const u32 & getCache() const;

	/**
	 Setter of mCache. Sets value of mCache.

	 @param[in] cache The value of mCache
	*/
	void setCache(const u32 & cache);

	/**
	 Getter of mBogomips. Returns value of mBogomips.

	 @return The value of mBogomips
	*/
	const Real & getBogomips() const;

	/**
	 Setter of mBogomips. Sets value of mBogomips.

	 @param[in] bogomips The value of mBogomips
	*/
	void setBogomips(const Real & bogomips);

	/**
	 Getter of mFlags. Returns value of mFlags.

	 @return The value of mFlags
	*/
	const std::string & getFlags() const;

	/**
	 Setter of mFlags. Sets value of mFlags.

	 @param[in] flags The value of mFlags
	*/
	void setFlags(const std::string & flags);

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Creates a CPU class instance based on xml
	 @param[out] tiCPU Description of CPU.
	 @return Returns the pointer of the CPU class instance created based on the xml
	 */
	static CPU::Pointer parseXML(TiXmlElement * &tiCPU);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Refresh CPU datas.
	 */
	virtual void refreshData();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMCPU_impl.h"

}

#endif
