
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMConfigOptions.h
/// Contains declaration of class ConfigOptions

#ifndef __PCM_CONFIGOPTIONS_H__
#  define __PCM_CONFIGOPTIONS_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMPointer.h"

#  include "PCMContainer.h"

//
// Outer includes
//

#  include <string>

#  include <map>

namespace ParCompMark
{

  /**
	Program configuration options. Stored in application ini file.
   */

  class ConfigOptions
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestConfigOptions;
#  endif

	//
	// Enumerations
	//

   protected:

  /** Option type definitions. */
	enum OptionType
	{

	/** STL string type */
	 STRING,

	/** Bool type */
	 BOOL,

	/** s32 integer type */
	 INTEGER,

	/** Real type */
	 REAL
	};

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < ConfigOptions,
	Mutex > Pointer;

   protected:

  /** Struct for option. */
	typedef struct Option

	{		  /** Type of the option */

	 OptionType type;						  /** Value of the option */

	 void *value;												 /** Description of the option (optional) */

	  std::string description;																					 /** Smart pointer on this struct */
	 typedef ParCompMark::Pointer < Option,

	  DummyLock > Pointer;																																		/** Destructor of the option */
	public: ~Option()
	 {
		if(type != STRING)
		 delete[](u8 *) value;
		else
		 delete static_cast < std::string * >(value);
	 }
	}
	Option;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Container for application options.

	 @remarks
	 This is own attribute of this class.
	*/
	Container < ConfigOptions::Option, Mutex >::Pointer mOptions;

	/** The option container is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor. 
	 */
	ConfigOptions();

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ ConfigOptions();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initialize option container.
	 */
	virtual void initialize();

	/** 
	 Finalize option container.
	 */
	virtual void finalize();

	/** 
	 Check for option.
	 @param[in] name Name of the option
	 @return True if the container has the specified option.
	 */
	virtual bool hasOption(const std::string & name);

	/** 
	 Get option.
	 @param[in] name Name of the option
	 @return STL string value
	 */
	virtual std::string getString(const std::string & name);

	/** 
	 Get option.
	 @param[in] name Name of the option
	 @return Bool value
	 */
	virtual bool getBool(const std::string & name);

	/** 
	 Get option.
	 @param[in] name Name of the option
	 @return Integer value
	 */
	virtual s32 getInteger(const std::string & name);

	/** 
	 Get option.
	 @param[in] name Name of the option
	 @return Real value
	 */
	virtual Real getReal(const std::string & name);

	/** 
	 Return true if the option exists with the given name and equals to the given value.
	 @param[in] name Name of the option
	 @param[in] value Value to test
	 @return True if the option exists with the given name and equals to the given value
	 */
	virtual bool testString(const std::string & name, const std::string & value);

	/** 
	 Set option.
	 @param[in] name Name of the option
	 @param[in] value Value of the option
	 @param[in] description Description of the option
	 */
	virtual void setString(const std::string & name, const std::string & value,
			   const std::string & description = "");

	/** 
	 Set option.
	 @param[in] name Name of the option
	 @param[in] value Value of the option
	 @param[in] description Description of the option
	 */
	virtual void setBool(const std::string & name, const bool & value,
			  const std::string & description = "");

	/** 
	 Set option.
	 @param[in] name Name of the option
	 @param[in] value Value of the option
	 @param[in] description Description of the option
	 */
	virtual void setInteger(const std::string & name, const s32 & value,
				const std::string & description = "");

	/** 
	 Set option.
	 @param[in] name Name of the option
	 @param[in] value Value of the option
	 @param[in] description Description of the option
	 */
	virtual void setReal(const std::string & name, const Real & value,
			  const std::string & description = "");

	/** 
	 Remove option.
	 @param[in] name Name of the option
	 */
	virtual void removeOption(const std::string & name);

	/** 
	 Create user-level ini file in the application directory (existing file will be overwritten).
	 @param[in] filename Name of the config file
	 */
	virtual void saveToIniFile(const std::string & filename);

	/** 
	 Load user-level ini file.
	 @param[in] filename Name of the config file
	 */
	virtual void loadFromIniFile(const std::string & filename);

   protected:

	/** 
	 Get option value for the specified type (internal method).
	 @param[in] name Name of the option
	 @param[in] type Type of the option
	 @return Pointer to the specified option
	 */
	virtual ConfigOptions::Option::Pointer getValue(const std::string & name,
							const ConfigOptions::OptionType & type);

	/** 
	 Set option value for the specified type (internal method).
	 @param[in] name Name of the option
	 @param[in] value Address of value
	 @param[in] type Type of the option
	 @param[in] description Description of the option
	 */
	virtual void setValue(const std::string & name, const void *value,
			  const ConfigOptions::OptionType & type, const std::string & description);

	/** 
	 Get size of option type
	 @param[in] type Type of the option
	 @return Size of the specified type
	 */
	virtual u32 getOptionTypeSize(const ConfigOptions::OptionType & type);

	/** 
	 Give string representation of the option value
	 @param[out] option Option
	 @return String representation
	 */
	virtual std::string optionToString(ConfigOptions::Option::Pointer & option);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMConfigOptions_impl.h"

}

#endif
