
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMContainer.h
/// Contains declaration of class Container

#ifndef __PCM_CONTAINER_H__
#  define __PCM_CONTAINER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMLogger.h"

#  include "PCMException.h"

//
// Outer includes
//

#  include <string>

#  include <map>

namespace ParCompMark
{

  /**
	String addressed map of typed smart pointers. 
   */
  template < class ElementType, class LockType > class Container
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestContainer;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < Container < ElementType,
	LockType >,
	LockType > Pointer;

  /** Type definition for pointer on elements. */
	typedef typename ElementType::Pointer ElementPointer;

  /** Type definition for map of elements. */
	typedef std::map < std::string,
	ElementPointer > ElementsMap;

  /** Type definition for iterator on map of elements. */
	typedef typename ElementsMap::iterator Iterator;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Map of elements.

	 @remarks
	 This is own attribute of this class.
	*/
	ElementsMap mElements;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor. 
	 */
	Container();

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Container();

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Add an element.
	 @param[in] name Name of the element
	 @param[in] element Element
	 */
	virtual void add(std::string name, ElementPointer element);

	/** 
	 Get an element by name.
	 @param[in] name Name of the element
	 @return Pointer to the element
	 */
	virtual ElementPointer get(const std::string & name);

	/** 
	 Search for an element by name.
	 @param[in] name Name of the element
	 @return The container has the element.
	 */
	virtual bool has(const std::string & name);

	/** 
	 Remove an element by name.
	 @param[in] name Name of the element
	 */
	virtual void remove(const std::string & name);

	/** 
	 Return the number of elements.
	 @return Pointer to the element
	 */
	virtual u32 getSize();

	/** 
	 Return true if the container is empty.
	 @return True if the container is empty
	 */
	virtual bool isEmpty();

	/** 
	 Begin iterator on the container.
	 @return Iterator on the first element.
	 */
	virtual Iterator begin();

	/** 
	 End iterator on the container.
	 @return Iterator on the element after the last element.
	 */
	virtual Iterator end();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMContainer_impl.h"

}

#endif
