
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMException.h
/// Contains declaration of class Exception

#ifndef __PCM_EXCEPTION_H__
#  define __PCM_EXCEPTION_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

//
// Macro definitions
//

///> Macro for throwing exception.
#  define Except(type, func, desc) throw(ParCompMark::Exception(Exception::type, desc, __FILE__, func, __LINE__ ))
///> Macro for asserting.
#  define Assert(cond, type, func) if(!(cond)) Except(Exception::type, func, "Assertion error: " #cond)

namespace ParCompMark
{

  /**

	 @par
	Provides information about an internal error.

	 @remarks
	An application using PCM that the exceptions are caught, 
			so all PCM functions should occur within a try{} 
			catch(PCM::Exception& e) {} block.

   */

  class Exception
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestException;
#  endif

	//
	// Enumerations
	//

   public:

  /** Definitions of error codes. */
	enum ExceptionType
	{

	/** Unknown internal error (mostly occured by another library). */
	 INTERNAL_ERROR,

	/** Nullpointer error. */
	 NULL_POINTER_ERROR,

	/** Invalid name error. */
	 INVALID_NAME_ERROR,

	/** Invalid enumerated value error. */
	 INVALID_ENUM_ERROR,

	/** Invalid value error. */
	 INVALID_VALUE_ERROR,

	/** Invalid object error. */
	 INVALID_OBJECT_ERROR,

	/** Invalid class error (not proper derived class). */
	 INVALID_CLASS_ERROR,

	/** Invalid operation error. */
	 INVALID_OPERATION_ERROR,

	/** Operation is not supported on this platform. */
	 OPERATION_NOT_SUPPORTED_ERROR,

	/** Out of memory error. */
	 OUT_OF_MEMORY_ERROR,

	/** File I/O error. */
	 FILE_IO_ERROR,

	/** Invalid file format error. */
	 FILE_FORMAT_ERROR,

	/** The user stopped the application. */
	 USER_BREAK_ERROR,

	/** Error in a script file. */
	 SCRIPT_ERROR,

	/** Xlib error. */
	 XLIB_ERROR
	};

	//
	// Class variables
	//

	/// @name Class variables
	//@{  

   protected:

	/** Pointer to the last raised exception.

	 @remarks
	 This attribute references an attribute.
	*/
	static Exception *mLastException;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Type of the exception.

	 @remarks
	 This is own attribute of this class.
	*/
	ExceptionType mType;

	/** Textual description of the exception.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mDescription;

	/** Name of the file where the exception was thrown.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mFileName;

	/** Name of the function where the exception was thrown.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mFunctionName;

	/** Number of the line where the exception was thrown.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mLineNumber;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor. 
	 @param[in] type Type of exception.
	 @param[in] description Textual description of the exception.
	 @param[in] fileName Name of the file where the exception was thrown.
	 @param[in] functionName Name of the function where the exception was thrown.
	 @param[in] lineNumber Number of the line where the exception was thrown.
	 */
	Exception(const ExceptionType & type = INTERNAL_ERROR, const std::string & description =
		 "unknown", const std::string & fileName = "unknown", const std::string & functionName =
		 "unknown", const u32 & lineNumber = 0);

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mType. Returns value of mType.

	 @return The value of mType
	*/
	const Exception::ExceptionType & getType() const;

	/**
	 Getter of mDescription. Returns value of mDescription.

	 @return The value of mDescription
	*/
	const std::string & getDescription() const;

	/**
	 Getter of mFileName. Returns value of mFileName.

	 @return The value of mFileName
	*/
	const std::string & getFileName() const;

	/**
	 Getter of mFunctionName. Returns value of mFunctionName.

	 @return The value of mFunctionName
	*/
	const std::string & getFunctionName() const;

	/**
	 Getter of mLineNumber. Returns value of mLineNumber.

	 @return The value of mLineNumber
	*/
	const u32 & getLineNumber() const;

	/**
	 Getter of mLastException. Returns value of mLastException.

	 @return The value of mLastException
	*/
	static Exception *getLastException();

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Translate type to human readable format.
	 @param[in] type Enum exception value.
	 @return 
	 */
	static std::string translateType(const Exception::ExceptionType & type);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMException_impl.h"

}

#endif
