
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMFileSystemManager.h
/// Contains declaration of class FileSystemManager

#ifndef __PCM_FILESYSTEMMANAGER_H__
#  define __PCM_FILESYSTEMMANAGER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMSingleton.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMContainer.h"

#  include "PCMPointer.h"

//
// Outer includes
//

#  include <string>

#  include <list>

#  include <queue>

#  include <stack>

//
// Scripting bind includes (for Squirrel)
//

#  include <squirrel/squirrel.h>
#  include <squirrel/sqstdio.h>
#  include <squirrel/sqstdaux.h>
#  include <squirrel/sqplus.h>

namespace ParCompMark
{

  /**
	Singleton class managing file system. This class handles
		file opening, config file handling.
   */

  class FileSystemManager:public Singleton < FileSystemManager >
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestFileSystemManager;
#  endif

	//
	// Scripting binding (for Squirrel)
	//

	/// @name Scripting binding
	//@{  

  /**
   Static glue code method for Squirrel-C++ binding. 
   This method should be call from each Squirrel virtual machines 
   to generate the proper Squirrel glue code.
   To content of this method is fully autogenerated in the header implementation file, 
   you should not modify it.
   */
   public:static void squirrelGlue()
	{
	 ::SqPlus::SQClassDef < FileSystemManager >
		(_T("FileSystemManager")).staticFunc(&FileSystemManager::getInstance,
						  _T("getInstance")).func(&FileSystemManager::existsDataFile,
									  _T("existsDataFile"));
	}

	//@}  

	//
	// Enumerations
	//

   public:

  /** Type of file operation. */
	enum FileOperation
	{

	/** Read. */
	 READ,

	/** Write. */
	 WRITE,

	/** Append. */
	 APPEND
	};

	//
	// Typedefs
	//

   public:

  /** Type for pointer on a C-style file pointer. */
	typedef Pointer < FILE,
	Mutex > CFilePointer;

  /** Type for pointer on a std::fstream object. */
	typedef Pointer < std::fstream,
	Mutex > CppFilePointer;

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** Maximal number of chars in paths.

	 @remarks
	 This is own attribute of this class.
	*/
	static const u32 MAX_PATH;

	/** Path separator on the host OS.

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string PATH_SEPARATOR;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** The application is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** Path of user-level application directory.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mAppDirectory;

	/** Path of application data directory.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mDataDirectory;

	/** Home directory of the user.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mHomeDirectory;

	/** Path of current directory.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mCurrentDirectory;

	/** Name of user-level ini file.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mIniFile;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Creates file system manager.
	 @param[in] appDirectory Path of application directory.
	 @param[in] iniFile Name of user-level ini file.
	 */
	FileSystemManager(const std::string & appDirectory =
			 "~/.ParCompMark/", const std::string & iniFile = "parcompmark.ini");

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ FileSystemManager();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mAppDirectory. Returns value of mAppDirectory.

	 @return The value of mAppDirectory
	*/
	const std::string & getAppDirectory() const;

	/**
	 Setter of mAppDirectory. Sets value of mAppDirectory.

	 @param[in] appdirectory The value of mAppDirectory
	*/
	void setAppDirectory(const std::string & appdirectory);

	/**
	 Getter of mDataDirectory. Returns value of mDataDirectory.

	 @return The value of mDataDirectory
	*/
	const std::string & getDataDirectory() const;

	/**
	 Setter of mDataDirectory. Sets value of mDataDirectory.

	 @param[in] datadirectory The value of mDataDirectory
	*/
	void setDataDirectory(const std::string & datadirectory);

	/**
	 Getter of mHomeDirectory. Returns value of mHomeDirectory.

	 @return The value of mHomeDirectory
	*/
	const std::string & getHomeDirectory() const;

	/**
	 Getter of mCurrentDirectory. Returns value of mCurrentDirectory.

	 @return The value of mCurrentDirectory
	*/
	const std::string & getCurrentDirectory() const;

	/**
	 Getter of mIniFile. Returns value of mIniFile.

	 @return The value of mIniFile
	*/
	const std::string & getIniFile() const;

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Gets the instance of the singleton class. Reimplemented for squirrel usage.
	 @return Instance of the class
	 */
	static FileSystemManager *getInstance();

	/** 
	 Close file from fstream object. This method is static to be able to close the logfile.
	 @param[out] fp Smart pointer on a fstream object.
	 */
	static void closeFile(FileSystemManager::CppFilePointer & fp);

	/** 
	 Close file from C-style pointer. This method is static to be able to close the logfile.
	 @param[out] fp Smart pointer on a C-style file pointer.
	 */
	static void closeFile(FileSystemManager::CFilePointer & fp);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initialize the file system manager.
	 */
	virtual void initialize();

	/** 
	 Finalize the file system manager.
	 */
	virtual void finalize();

	/** 
	 Return true if the specified directory exists.
	 @param[in] name Name of the directory.
	 @return True if the specified directory exists.
	 */
	virtual bool existsDirectory(const std::string & name) const;

	/** 
	 Create directory.
	 @param[in] name Name of the directory.
	 @return True if the directory did not exist, and had to be created.
	 */
	virtual bool createDirectory(const std::string & name) const;

	/** 
	 Find data directory path. Throw exception if not found.
	 @return Path of data directory.
	 */
	virtual std::string findDataDirectory() const;

	/** 
	 Returns true if the library can be found.
	 @param[in] library Name of the library file.
	 @param[in] additionalPaths Array if additional paths to seach
	 @return The library can be found.
	 */
	virtual bool existsLibrary(const std::string & library,
				 const std::vector < std::string > &additionalPaths =
				 std::vector < std::string > ())const;

	/** 
	 Find path for library. Throw exception if not found.
	 @param[in] library Name of the library file.
	 @param[in] additionalPaths Array if additional paths to seach
	 @return Path of the library.
	 */
	virtual std::string findLibraryPath(const std::string & library,
					 const std::vector < std::string > &additionalPaths =
					 std::vector < std::string > ())const;

	/** 
	 List files in the specified directory. Throw exception if not found.
	 @param[in] directory Name of the directory to scan.
	 @return List of files in the specified directory.
	 */
	virtual std::list < std::string > listDirectory(const std::string & directory) const;

	/** 
	 List files in the specified directory relative to the data directory. Throw exception if not found.
	 @param[in] directory Name of the directory to scan.
	 @return List of files in the specified directory (relative to the data directory).
	 */
	virtual std::list < std::string > listDataDirectory(const std::string & directory) const;

	/** 
	 Return true if the specified file exists.
	 @param[in] name Name of the file.
	 @return True if the specified file exists.
	 */
	virtual bool existsFile(const std::string & name) const;

	/** 
	 Return true if the specified file exists in the application directory.
	 @param[in] name Name of the file.
	 @return True if the specified file exists.
	 */
	virtual bool existsAppFile(const std::string & name) const;

	/** 
	 Return true if the specified file exists in the data directory.
	 @param[in] name Name of the file.
	 @return True if the specified file exists.
	 */
	virtual bool existsDataFile(const char *name) const;

	/** 
	 Open file for reading or writing and return a C-style file pointer.
	 @param[in] name Name of the file.
	 @param[in] operation File operation.
	 @return Smart pointer on a C-style file pointer.
	 */
	virtual FileSystemManager::CFilePointer openFileC(const std::string & name,
							const FileOperation & operation =
							FileSystemManager::READ) const;

	/** 
	 Open file for reading or writing in the application directory and return a C-style file pointer.
	 @param[in] name Name of the file.
	 @param[in] operation File operation.
	 @return Smart pointer on a C-style file pointer.
	 */
	virtual FileSystemManager::CFilePointer openAppFileC(const std::string & name,
							  const FileOperation & operation =
							  FileSystemManager::READ) const;

	/** 
	 Open file for reading or writing in the application data directory and return a C-style file pointer.
	 @param[in] name Name of the file.
	 @param[in] operation File operation.
	 @return Smart pointer on a C-style file pointer.
	 */
	virtual FileSystemManager::CFilePointer openDataFileC(const std::string & name,
							  const FileOperation & operation =
							  FileSystemManager::READ) const;

	/** 
	 Open file for reading or writing and return a fstream object.
	 @param[in] name Name of the file.
	 @param[in] operation File operation.
	 @return Smart pointer on a fstream object.
	 */
	virtual FileSystemManager::CppFilePointer openFileCpp(const std::string & name,
							  const FileOperation & operation =
							  FileSystemManager::READ) const;

	/** 
	 Open file for reading or writing in the application directory and return a fstream object.
	 @param[in] name Name of the file.
	 @param[in] operation File operation.
	 @return Smart pointer on a fstream object.
	 */
	virtual FileSystemManager::CppFilePointer openAppFileCpp(const std::string & name,
								 const FileOperation & operation =
								 FileSystemManager::READ) const;

	/** 
	 Open file for reading or writing in the application data directory and return a fstream object.
	 @param[in] name Name of the file.
	 @param[in] operation File operation.
	 @return Smart pointer on a fstream object.
	 */
	virtual FileSystemManager::CppFilePointer openDataFileCpp(const std::string & name,
								const FileOperation & operation =
								FileSystemManager::READ) const;

	/** 
	 Get full path of the specified data file.
	 @param[in] name Name of the file.
	 @return Full path for the data file.
	 */
	virtual std::string getPathDataFile(const std::string & name) const;

	/** 
	 Read contents of a text file.
	 @param[in] name Name of the file.
	 @return Content of the file.
	 */
	virtual std::string readTextFile(const std::string & name) const;

	/** 
	 Read contents of a text file in the application directory.
	 @param[in] name Name of the file.
	 @return Content of the file.
	 */
	virtual std::string readAppTextFile(const std::string & name) const;

	/** 
	 Read contents of a text file in the application data directory.
	 @param[in] name Name of the file.
	 @return Content of the file.
	 */
	virtual std::string readDataTextFile(const std::string & name) const;

   protected:

	/** 
	 Find the home directory of the user on the filesystem.
	 */
	virtual void _findHomeDirectory();

	/** 
	 Replace '~' char in the specified path string.
	 @param[in] path Path string.
	 @return Replaced path string.
	 */
	virtual std::string _replaceHomeChar(const std::string & path) const;

	/** 

			Translate relative path to absolute (also replace '~' char in the specified path string).
			Original code: http://www.codeproject.com/useritems/path_conversion.asp

	 @param[in] path Path string.
	 @return Replaced path string.
	 */
	virtual std::string _translateToAbsolutePath(const std::string & path) const;

	/** 
	 Create user-level application directory. Do nothing if already exists.
	 */
	virtual void _createAppDirectory();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMFileSystemManager_impl.h"

}

// Declare FileSystemManager class (and its enums) for SqPlus binding
/// \skip DECLARE_INSTANCE_TYPE_NAME
DECLARE_INSTANCE_TYPE_NAME(ParCompMark::FileSystemManager, FileSystemManager);

#endif
