
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMGLXGLContext.h
/// Contains declaration of class GLXGLContext

#ifndef __PCM_GLXGLCONTEXT_H__
#  define __PCM_GLXGLCONTEXT_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMPointer.h"

#  include "PCMXDisplay.h"

//
// Outer includes
//

#  include <X11/Xlib.h>

#  include <GL/glew.h>

#  include <GL/glx.h>

#  include <GL/glxext.h>

namespace ParCompMark
{

  /**
	Class that encapsulates a GLX context. Original source can be found in Ogre3D sources (http://ogre3d.org).
   */

  class GLXGLContext
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestGLXGLContext;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < GLXGLContext,
	Mutex > Pointer;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** The context is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** Corresponding X Display.

	 @remarks
	 This is own attribute of this class.
	*/
	XDisplay::Pointer mDisplay;

	/** Corresponding GLX rendering window.

	 @remarks
	 This attribute references an attribute.
	*/
	GLXRenderWindow *mGLXWindow;

	/** Visual info for the context.

	 @remarks
	 This is own attribute of this class.
	*/
	::XVisualInfo * mVisualInfo;

	/** Wrapped GLX context.

	 @remarks
	 This is own attribute of this class.
	*/
	::GLXContext mGLXContext;

	/** The GLXContext attribute is maintaned by another code (In our case the PC library).

	 @remarks
	 This is own attribute of this class.
	*/
	bool mTainted;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create GLX context.
	 @param[out] display X display
	 @param[in] glxWindow Corresponding GLX rendering window.
	 @param[in] visualInfo Visualinfo for context creation
	 */
	GLXGLContext(XDisplay::Pointer & display, GLXRenderWindow * glxWindow,::XVisualInfo * visualInfo);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ GLXGLContext();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mDisplay. Returns value of mDisplay.

	 @return The value of mDisplay
	*/
	const XDisplay::Pointer & getDisplay() const;

	/**
	 Getter of mGLXWindow. Returns value of mGLXWindow.

	 @return The value of mGLXWindow
	*/
	GLXRenderWindow *getGLXWindow() const;

	/**
	 Getter of mVisualInfo. Returns value of mVisualInfo.

	 @return The value of mVisualInfo
	*/
	::XVisualInfo * getVisualInfo() const;

	/**
	 Getter of mGLXContext. Returns value of mGLXContext.

	 @return The value of mGLXContext
	*/
	const::GLXContext & getGLXContext() const;

	/**
	 Getter of mTainted. Returns value of mTainted.

	 @return The value of mTainted
	*/
	const bool & getTainted() const;

	/**
	 Setter of mTainted. Sets value of mTainted.

	 @param[in] tainted The value of mTainted
	*/
	void setTainted(const bool & tainted);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initialize the GL context.
	 */
	virtual void initialize();

	/** 
	 Finalize the GL context.
	 */
	virtual void finalize();

	/** 
	 Enable the context. All subsequent rendering commands will go here.
	 */
	virtual void setCurrent();

	/** 
	 Release the current context without assigning a new one.
	 */
	virtual void releaseCurrent();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMGLXGLContext_impl.h"

}

#endif
