
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMGLXRenderWindow.h
/// Contains declaration of class GLXRenderWindow

#ifndef __PCM_GLXRENDERWINDOW_H__
#  define __PCM_GLXRENDERWINDOW_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMPointer.h"

#  include "PCMLogger.h"

#  include "PCMGLXGLContext.h"

#  include "PCMXDisplay.h"

#  include "PCMTimer.h"

//
// Outer includes
//

#  include <X11/Xlib.h>

#  include <X11/keysym.h>

#  include <GL/glew.h>

#  include <GL/glx.h>

#  include <string>

#  include <sstream>

namespace ParCompMark
{

  /**
	Manages the target rendering window in GLX environment. 
		Original source can be found in Ogre3D sources (http://ogre3d.org).
   */

  class GLXRenderWindow
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestGLXRenderWindow;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < GLXRenderWindow,
	Mutex > Pointer;

  /** Struct for window statistics (FPS values, frame times). */
	typedef struct

	{		 /** Number of rendered frames */

	 u32 frameCount;							  /** Time elapsed the start of the rendering */

	 Real renderingTime;																	 /** Frame rate for the last frame */

	 Real lastFPS;																								 /** Average frame rate */

	 Real avgFPS;																														 /** Frame rate for the frame with the best frame time */

	 Real bestFPS;																																							   /** Frame rate for the frame with the worst frame time */

	 Real worstFPS;																																																	   /** Time at the beginning of the frame */

	 Real frameBeginTime;																																																									   /** Time at the end of the frame */

	 Real frameEndTime;																																																																		  /** Last frame time */

	 Real lastFrameTime;																																																																									 /** Best frame time */

	 Real bestFrameTime;																																																																															 /** Worst frame time */

	 Real worstFrameTime;																																																																																				   /** Triangle count for the last frame */

	 Real lastTriangleCount;																																																																																													   /** Average triangle count */

	 Real avgTriangleCount;																																																																																																					 /** Minimal triangle count */

	 Real minTriangleCount;																																																																																																												   /** Maximal triangle count */
	 Real maxTriangleCount;
	} WindowStatistics;

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** Constant for centered position.

	 @remarks
	 This is own attribute of this class.
	*/
	static const s32 CENTERED;

	/** Constant for maximal size.

	 @remarks
	 This is own attribute of this class.
	*/
	static const u32 MAXIMALSIZE;

	/** Undefined statistics value.

	 @remarks
	 This is own attribute of this class.
	*/
	static const Real UNDEFINEDSTATISTICS;

	/** Constant for undefined XRR configuration.

	 @remarks
	 This is own attribute of this class.
	*/
	static const s32 UNDEFINEDXRRCONFIGURATION;

	/** Default window width.

	 @remarks
	 This is own attribute of this class.
	*/
	static const u32 DEFAULTWINDOWWIDTH;

	/** Default window height.

	 @remarks
	 This is own attribute of this class.
	*/
	static const u32 DEFAULTWINDOWHEIGHT;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Corresponding X Display.

	 @remarks
	 This is own attribute of this class.
	*/
	XDisplay::Pointer mDisplay;

	/** GLX context of the render window.

	 @remarks
	 This is own attribute of this class.
	*/
	GLXGLContext::Pointer mGLXGLContext;

	/** Corresponding process.

	 @remarks
	 This attribute references an attribute.
	*/
	Process *mProcess;

	/** Wrapped GLX Window.

	 @remarks
	 This is own attribute of this class.
	*/
	::Window mWindow;

	/** The window is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** The window is visible.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mVisible;

	/** The window appears in full screen mode.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mFullScreen;

	/** Window caption.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mCaption;

	/** Horizontal position of the window.

	 @remarks
	 This is own attribute of this class.
	*/
	s32 mLeft;

	/** Vertical position of the window.

	 @remarks
	 This is own attribute of this class.
	*/
	s32 mTop;

	/** Horizontal size of the window.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mWidth;

	/** Vertical size of the window.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mHeight;

	/** Colour depth of the window.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mColourDepth;

	/** Number of fullscreen antialasing samples.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mFSAASamples;

	/** For storing original XRR configuration mode.

	 @remarks
	 This is own attribute of this class.
	*/
	s32 mOriginalXRRConfiguation;

	/** Atom to recognize window close events.

	 @remarks
	 This is own attribute of this class.
	*/
	::Atom mAtomDeleteWindow;

	/** Current window statistics.

	 @remarks
	 This is own attribute of this class.
	*/
	WindowStatistics mWindowStatistics;

	/** Time of the start of the rendering.

	 @remarks
	 This is own attribute of this class.
	*/
	Real mRenderingBeginTime;

	/** Time of stating measuring a second for FPS calculation.

	 @remarks
	 This is own attribute of this class.
	*/
	Real mFPSMeasuringBeginTime;

	/** Frame counter for storing number of frames during a second.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mFPSMeasuringFrameCount;

	/** Summarized triangle count for all frames for calculating avgTriangleCount statistics.

	 @remarks
	 This is own attribute of this class.
	*/
	Real mSumTriangleCount;

	/** Summarized frame rates for all frames for calculating avgFPS statistics.

	 @remarks
	 This is own attribute of this class.
	*/
	Real mSumFPS;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create GLX render window. Normally called by XDisplay::createRenderWindow.
	 @param[out] display X display
	 @param[in] process Corresponding process.
	 @param[in] caption Window caption
	 @param[in] fullScreen The window appears in full screen mode
	 @param[in] colourDepth Colour depth of the window
	 @param[in] width Horizontal size
	 @param[in] height Vertical size
	 @param[in] left Horizontal position
	 @param[in] top Vertical position
	 @param[in] fsaaSamples Number of fullscreen antialasing samples (Do not use FSAA samples other than 0 now with nVidia cards!)
	 */
	GLXRenderWindow(XDisplay::Pointer & display, Process * process, const std::string caption =
			"PCM Framework", const bool & fullScreen = true, const u32 & colourDepth =
			0, const u32 & width = GLXRenderWindow::MAXIMALSIZE, const u32 & height =
			GLXRenderWindow::MAXIMALSIZE, const s32 & left =
			GLXRenderWindow::CENTERED, const s32 & top =
			GLXRenderWindow::CENTERED, const u32 & fsaaSamples = 0);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ GLXRenderWindow();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mDisplay. Returns value of mDisplay.

	 @return The value of mDisplay
	*/
	XDisplay::Pointer & getDisplay();

	/**
	 Getter of mGLXGLContext. Returns value of mGLXGLContext.

	 @return The value of mGLXGLContext
	*/
	GLXGLContext::Pointer & getGLXGLContext();

	/**
	 Getter of mProcess. Returns value of mProcess.

	 @return The value of mProcess
	*/
	Process *getProcess() const;

	/**
	 Getter of mWindow. Returns value of mWindow.

	 @return The value of mWindow
	*/
	const::Window & getWindow() const;

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mVisible. Returns value of mVisible.

	 @return The value of mVisible
	*/
	const bool & getVisible() const;

	/**
	 Setter of mVisible. Sets value of mVisible.

	 @param[in] visible The value of mVisible
	*/
	void setVisible(const bool & visible);

	/**
	 Getter of mFullScreen. Returns value of mFullScreen.

	 @return The value of mFullScreen
	*/
	const bool & getFullScreen() const;

	/**
	 Setter of mFullScreen. Sets value of mFullScreen.

	 @param[in] fullscreen The value of mFullScreen
	*/
	void setFullScreen(const bool & fullscreen);

	/**
	 Getter of mCaption. Returns value of mCaption.

	 @return The value of mCaption
	*/
	const std::string & getCaption() const;

	/**
	 Setter of mCaption. Sets value of mCaption.

	 @param[in] caption The value of mCaption
	*/
	void setCaption(const std::string & caption);

	/**
	 Getter of mLeft. Returns value of mLeft.

	 @return The value of mLeft
	*/
	const s32 & getLeft() const;

	/**
	 Setter of mLeft. Sets value of mLeft.

	 @param[in] left The value of mLeft
	*/
	void setLeft(const s32 & left);

	/**
	 Getter of mTop. Returns value of mTop.

	 @return The value of mTop
	*/
	const s32 & getTop() const;

	/**
	 Setter of mTop. Sets value of mTop.

	 @param[in] top The value of mTop
	*/
	void setTop(const s32 & top);

	/**
	 Getter of mWidth. Returns value of mWidth.

	 @return The value of mWidth
	*/
	const u32 & getWidth() const;

	/**
	 Setter of mWidth. Sets value of mWidth.

	 @param[in] width The value of mWidth
	*/
	void setWidth(const u32 & width);

	/**
	 Getter of mHeight. Returns value of mHeight.

	 @return The value of mHeight
	*/
	const u32 & getHeight() const;

	/**
	 Setter of mHeight. Sets value of mHeight.

	 @param[in] height The value of mHeight
	*/
	void setHeight(const u32 & height);

	/**
	 Getter of mColourDepth. Returns value of mColourDepth.

	 @return The value of mColourDepth
	*/
	const u32 & getColourDepth() const;

	/**
	 Setter of mColourDepth. Sets value of mColourDepth.

	 @param[in] colourdepth The value of mColourDepth
	*/
	void setColourDepth(const u32 & colourdepth);

	/**
	 Getter of mFSAASamples. Returns value of mFSAASamples.

	 @return The value of mFSAASamples
	*/
	const u32 & getFSAASamples() const;

	/**
	 Setter of mFSAASamples. Sets value of mFSAASamples.

	 @param[in] fsaasamples The value of mFSAASamples
	*/
	void setFSAASamples(const u32 & fsaasamples);

	/**
	 Getter of mWindowStatistics. Returns value of mWindowStatistics.

	 @return The value of mWindowStatistics
	*/
	const GLXRenderWindow::WindowStatistics & getWindowStatistics() const;

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Set new window position.
	 @param[in] left Horizontal position
	 @param[in] top Vertical position
	 */
	virtual void reposition(const s32 & left, const s32 & top);

	/** 
	 Set new window sizes.
	 @param[in] width Horizontal size
	 @param[in] height Vertical size
	 */
	virtual void resize(const u32 & width, const u32 & height);

	/** 
	 Start a frame.
	 */
	virtual void startFrame();

	/** 
	 Finish current frame.
	 */
	virtual void finishFrame();

	/** 
	 Enable the context of the window. 
			All subsequent rendering commands will go on this window.
			The side effect of calling this method is reseting the window statistics.

	 */
	virtual void setCurrent();

	/** 
	 Disable the context of the window. 
			Release the current context without assigning a new one.

	 */
	virtual void releaseCurrent();

	/** 
	 Initialize the GLX RenderWindow. Protected method for internal use.
	 */
	virtual void initialize();

	/** 
	 Finalize the GLX RenderWindow. Protected method for internal use.
	 */
	virtual void finalize();

	/** 
	 Reset statistics. It is called internally when initializing the window. But it can be called by some other process to 
			get the most accurate window statistics.

	 */
	virtual void resetStatistics();

	/** 
	 Report certain number of triangles being rendered to the render window. 
	 @param[in] triangleCount Number of rendered triangles.
	 */
	virtual void reportTriangles(const u32 & triangleCount);

   protected:

	/** 
	 Create GLX Window. Protected method for internal use.
	 */
	virtual void createWindow();

	/** 
	 Destroy GLX Window. Protected method for internal use.
	 */
	virtual void destroyWindow();

	/** 
	 Update statistics of the window. Protected method for internal use.
	 */
	virtual void updateStatistics();

	/** 
	 Efficiently set window position (for internal use).
	 */
	virtual void _reposition();

	/** 
	 Efficiently set window sizes (for internal use).
	 */
	virtual void _resize();

	/** 
	 Efficiently set window caption (for internal use).
	 */
	virtual void _setCaption();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMGLXRenderWindow_impl.h"

}

#endif
