
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMHostInfo.h
/// Contains declaration of class HostInfo

#ifndef __PCM_HOSTINFO_H__
#  define __PCM_HOSTINFO_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMOutputNode.h"

#  include "PCMLogger.h"

#  include "PCMException.h"

#  include "PCMContainer.h"

#  include "PCMCPU.h"

#  include "PCMGPU.h"

#  include "PCMPointer.h"

#  include "PCMOutputNode.h"

//
// Outer includes
//

#  include <string>

#  include <pcapi.h>

#  include <tinyxml/tinyxml.h>

namespace ParCompMark
{

  /**
	Class for contain host information.
   */

  class HostInfo:public OutputNode
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestHostInfo;
#  endif

	//
	// Typedefs
	//

   public:

  /** Struct contain network layer and ID what PC can control. */
	typedef struct NetIDName

	{			 /** Network layer name */

	 std::string name;							/** ID of the given layer */

	 PCint ID;														/** Smart pointer on this struct */
	 typedef ParCompMark::Pointer < NetIDName,
	  DummyLock > Pointer;
	} NetIDName;

  /** Type for pointer on this class. */
	typedef ParCompMark::Pointer < HostInfo,
	Mutex > Pointer;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** CPUs on this host.

	 @remarks
	 This is own attribute of this class.
	*/
	Container < CPU, Mutex >::Pointer mCPUs;

	/** GPUs on this host.

	 @remarks
	 This is own attribute of this class.
	*/
	Container < GPU, Mutex >::Pointer mGPUs;

	/** Network layers and IDs what PC can control.

	 @remarks
	 This is own attribute of this class.
	*/
	Container < HostInfo::NetIDName, DummyLock >::Pointer mNetIDNames;

	/** PC library version.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mPCLibVersion;

	/** Number of network transprot layers available for the PC API.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mPCNumNetworks;

	/** PC vendor.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mPCVendor;

	/** Loaded PC extensions.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mPCExtensions;

	/** The maximum value of volatile framelet supported by implementation.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mVolatileFrameletLimit;

	/** The maximum value of retain output supported by implementation.

	 @remarks
	 This is own attribute of this class.
	*/
	PCint mRetainOutputLimit;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Creates a host with a specified name.
	 @param[in] name Name of the host.
	 @param[in] type Type of the host node.
	 */
	HostInfo(const std::string & name, const OutputNode::NodeType & type);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ HostInfo();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mCPUs. Returns value of mCPUs.

	 @return The value of mCPUs
	*/
	Container < CPU, Mutex >::Pointer & getCPUs();

	/**
	 Getter of mGPUs. Returns value of mGPUs.

	 @return The value of mGPUs
	*/
	Container < GPU, Mutex >::Pointer & getGPUs();

	/**
	 Getter of mNetIDNames. Returns value of mNetIDNames.

	 @return The value of mNetIDNames
	*/
	Container < HostInfo::NetIDName, DummyLock >::Pointer & getNetIDNames();

	/**
	 Getter of mPCLibVersion. Returns value of mPCLibVersion.

	 @return The value of mPCLibVersion
	*/
	const std::string & getPCLibVersion() const;

	/**
	 Getter of mPCNumNetworks. Returns value of mPCNumNetworks.

	 @return The value of mPCNumNetworks
	*/
	const PCint & getPCNumNetworks() const;

	/**
	 Getter of mPCVendor. Returns value of mPCVendor.

	 @return The value of mPCVendor
	*/
	const std::string & getPCVendor() const;

	/**
	 Getter of mPCExtensions. Returns value of mPCExtensions.

	 @return The value of mPCExtensions
	*/
	const std::string & getPCExtensions() const;

	/**
	 Getter of mVolatileFrameletLimit. Returns value of mVolatileFrameletLimit.

	 @return The value of mVolatileFrameletLimit
	*/
	const PCint & getVolatileFrameletLimit() const;

	/**
	 Getter of mRetainOutputLimit. Returns value of mRetainOutputLimit.

	 @return The value of mRetainOutputLimit
	*/
	const PCint & getRetainOutputLimit() const;

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Creates a HostInfo class instance based on xml
	 @param[out] tiHostinfo Description of GPU.
	 @return Returns the pointer of the HostInfo class instance created based on the xml
	 */
	static HostInfo::Pointer parseXML(TiXmlElement * &tiHostinfo);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Refresh output node data.
	 */
	virtual void refreshData();

	/** 
	 Refresh CPUs data on the hosts.
	 @param[out] cpus CPUs root.
	 */
	virtual void refreshCPUs(OutputNode::Pointer & cpus);

	/** 
	 Refresh GPUs data on the hosts.
	 @param[out] gpus GPUs root.
	 */
	virtual void refreshGPUs(OutputNode::Pointer & gpus);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMHostInfo_impl.h"

}

#endif
