
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMLogger.h
/// Contains declaration of class Logger

#ifndef __PCM_LOGGER_H__
#  define __PCM_LOGGER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMSingleton.h"

#  include "PCMPointer.h"

#  include "PCMException.h"

#  include "PCMTimer.h"

//
// Outer includes
//

#  include <string>

#  include <sstream>

namespace ParCompMark
{

  /**
	Class for very simple logging mechanism.
   */

  class Logger:public Singleton < Logger >
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestLogger;
#  endif

	//
	// Enumerations
	//

   public:

  /** Definitions logging levels. */
	enum LogLevel
	{

	/** Fatal error. */
	 FATAL,

	/** Error. */
	 ERROR,

	/** Warning. */
	 WARNING,

	/** Notice. */
	 NOTICE,

	/** Debug. */
	 DEBUG
	};

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** Indicates that the logger logs to console.

	 @remarks
	 This is own attribute of this class.
	*/
	static const u8 LOGTOCONSOLE;

	/** Indicates that the logger logs to a textfile.

	 @remarks
	 This is own attribute of this class.
	*/
	static const u8 LOGTOFILE;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Logging mode (console and/or textfile).

	 @remarks
	 This is own attribute of this class.
	*/
	u8 mLogMode;

	/** Logging level for console (from FATAL to NOTICE).

	 @remarks
	 This is own attribute of this class.
	*/
	LogLevel mConsoleLogLevel;

	/** Logging level for file (from FATAL to NOTICE).

	 @remarks
	 This is own attribute of this class.
	*/
	LogLevel mFileLogLevel;

	/** File pointer.

	 @remarks
	 This is own attribute of this class.
	*/
	Pointer < FILE, Mutex > mFp;

	/** Indicates that the logger is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** Name of the log file.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mLogFileName;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create logger.
	 @param[in] logFileName Name of the log file In the application directory.
	 */
	Logger(const std::string & logFileName = "parcompmark.log");

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Logger();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mLogMode. Returns value of mLogMode.

	 @return The value of mLogMode
	*/
	const u8 & getLogMode() const;

	/**
	 Getter of mConsoleLogLevel. Returns value of mConsoleLogLevel.

	 @return The value of mConsoleLogLevel
	*/
	const Logger::LogLevel & getConsoleLogLevel() const;

	/**
	 Setter of mConsoleLogLevel. Sets value of mConsoleLogLevel.

	 @param[in] consoleloglevel The value of mConsoleLogLevel
	*/
	void setConsoleLogLevel(const Logger::LogLevel & consoleloglevel);

	/**
	 Getter of mFileLogLevel. Returns value of mFileLogLevel.

	 @return The value of mFileLogLevel
	*/
	const Logger::LogLevel & getFileLogLevel() const;

	/**
	 Setter of mFileLogLevel. Sets value of mFileLogLevel.

	 @param[in] fileloglevel The value of mFileLogLevel
	*/
	void setFileLogLevel(const Logger::LogLevel & fileloglevel);

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mLogFileName. Returns value of mLogFileName.

	 @return The value of mLogFileName
	*/
	const std::string & getLogFileName() const;

	/**
	 Setter of mLogFileName. Sets value of mLogFileName.

	 @param[in] logfilename The value of mLogFileName
	*/
	void setLogFileName(const std::string & logfilename);

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Translate log level to human readable.
	 @param[in] loglevel Logging level.
	 @return 
	 */
	static std::string translateLogLevel(const LogLevel & loglevel);

	/** 
	 Translate an exception to human readable.
	 @param[in] exception Exception.
	 @return 
	 */
	static std::string translateException(const Exception & exception);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initializes the logger.
	 */
	virtual void initialize();

	/** 
	 Logs a message. (Does not care about multiline messages, because of performance).
	 @param[in] loglevel Message level.
	 @param[in] message Message string.
	 */
	virtual void log(const LogLevel & loglevel, const std::string & message);

	/** 
	 Logs a message. Handles multiline messages too. 
	 @param[in] loglevel Message level.
	 @param[in] message Message string.
	 */
	virtual void logMultiLine(const LogLevel & loglevel, const std::string & message);

	/** 
	 Logs an exception.
	 @param[in] exception Exception.
	 */
	virtual void log(const Exception & exception);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMLogger_impl.h"

}

#endif
