
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMNetwork.h
/// Contains declaration of class Network

#ifndef __PCM_NETWORK_H__
#  define __PCM_NETWORK_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMThread.h"

#  include "PCMPointer.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMContainer.h"

#  include "PCMFileSystemManager.h"

//
// Outer includes
//

#  include <sys/socket.h>

#  include <arpa/inet.h>

#  include <unistd.h>

#  include <linux/if.h>

#  include <sys/ioctl.h>

namespace ParCompMark
{

  /**
	Class for network routines.
   */

  class Network:public Thread
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestNetwork;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer to this class. */
	typedef Pointer < Network,
	DummyLock > Pointer;

  /** Struct for network interfaces IP information. */
	typedef struct IfConf

	{		  /** IP address of the network card.*/

	 std::string IP;								 /** Netmask of the network card.*/

	 std::string Netmask;														  /** Broacast IP of the network card. */

	 std::string BroadcastIP;																								/** Smart pointer on this struct */
	 typedef ParCompMark::Pointer < IfConf,
	  DummyLock > Pointer;
	} IfConf;

	//
	// Class variables
	//

	/// @name Class variables
	//@{  

   protected:

	/** Configurations of network interfaces of a host.

	 @remarks
	 This is own attribute of this class.
	*/
	static Container < Network::IfConf,
	DummyLock >::Pointer mIfConfs;

	/** The communication port number.

	 @remarks
	 This is own attribute of this class.
	*/
	static unsigned short mCommunicationPort;

	/** The broadcast port number.

	 @remarks
	 This is own attribute of this class.
	*/
	static unsigned short mBroadcastPort;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Indicates initialized the class.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** The client socket number.

	 @remarks
	 This is own attribute of this class.
	*/
	int mStreamSocket;

	/** The broadcast socket number.

	 @remarks
	 This is own attribute of this class.
	*/
	int mBroadcastSocket;

	/** IP address of the master node.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mOwnIP;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create network class.
	 @param[in] name Name of the network.
	 */
	Network(const std::string & name);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Network();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mIfConfs. Returns value of mIfConfs.

	 @return The value of mIfConfs
	*/
	static Container < Network::IfConf,
	DummyLock >::Pointer & getIfConfs();

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mCommunicationPort. Returns value of mCommunicationPort.

	 @return The value of mCommunicationPort
	*/
	static const unsigned short &getCommunicationPort();

	/**
	 Setter of mCommunicationPort. Sets value of mCommunicationPort.

	 @param[in] communicationport The value of mCommunicationPort
	*/
	static void setCommunicationPort(const unsigned short &communicationport);

	/**
	 Getter of mBroadcastPort. Returns value of mBroadcastPort.

	 @return The value of mBroadcastPort
	*/
	static const unsigned short &getBroadcastPort();

	/**
	 Setter of mBroadcastPort. Sets value of mBroadcastPort.

	 @param[in] broadcastport The value of mBroadcastPort
	*/
	static void setBroadcastPort(const unsigned short &broadcastport);

	/**
	 Getter of mStreamSocket. Returns value of mStreamSocket.

	 @return The value of mStreamSocket
	*/
	const int &getStreamSocket() const;

	/**
	 Setter of mStreamSocket. Sets value of mStreamSocket.

	 @param[in] streamsocket The value of mStreamSocket
	*/
	void setStreamSocket(const int &streamsocket);

	/**
	 Getter of mBroadcastSocket. Returns value of mBroadcastSocket.

	 @return The value of mBroadcastSocket
	*/
	const int &getBroadcastSocket() const;

	/**
	 Getter of mOwnIP. Returns value of mOwnIP.

	 @return The value of mOwnIP
	*/
	const std::string & getOwnIP() const;

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Get local machine IP address-es.
	 */
	static void getIP();

	/** 
	 Get local machine name.
	 @return name of host
	 */
	static std::string getHostName();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMNetwork_impl.h"

}

#endif
