
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMNode.h
/// Contains declaration of class Node

#ifndef __PCM_NODE_H__
#  define __PCM_NODE_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMName.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMPointer.h"

#  include "PCMProcess.h"

#  include "PCMOutputNode.h"

#  include "PCMContainer.h"

#  include "PCMBuffer.h"

//
// Outer includes
//

#  include <pcapi.h>

//
// Scripting bind includes (for Squirrel)
//

#  include <squirrel/squirrel.h>
#  include <squirrel/sqstdio.h>
#  include <squirrel/sqstdaux.h>
#  include <squirrel/sqplus.h>

namespace ParCompMark
{

  /**
	Node class. Entity for composite and/or render a framelet.
   */

  class Node:public Name
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestNode;
#  endif

	//
	// Scripting binding (for Squirrel)
	//

	/// @name Scripting binding
	//@{  

  /**
   Static glue code method for Squirrel-C++ binding. 
   This method should be call from each Squirrel virtual machines 
   to generate the proper Squirrel glue code.
   To content of this method is fully autogenerated in the header implementation file, 
   you should not modify it.
   */
   public:static void squirrelGlue()
	{
	 ::SqPlus::SQClassDef < Node > (_T("Node")).func(&Node::createProcess,
							 _T("createProcess")).func(&Node::createBuffer,
										  _T("createBuffer"));
	}

	//@}  

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < Node,
	Mutex > Pointer;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Parent host of the node. (Parent reference is standard pointer to avoid circular reference)

	 @remarks
	 This attribute references an attribute.
	*/
	Host * mParent;

	/** Search path of PC library.

	 @remarks
	 This is own attribute of this class.
	*/
	PCstring mSearchPath;

	/** Processes on this node.

	 @remarks
	 This is own attribute of this class.
	*/
	Container < Process, Mutex >::Pointer mProcesses;

	/** Node output document.

	 @remarks
	 This is own attribute of this class.
	*/
	OutputNode::Pointer mOutputDocument;

	/** Grapics memory buffers on this node.

	 @remarks
	 This is own attribute of this class.
	*/
	Container < Buffer, Mutex >::Pointer mBuffers;

	/** The node is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor for Squirrel compatibility. Calling this constructor always raises an exception.
	 */
	Node();

	/** 
	 Create node. Normally Host calls this constructor.
	 @param[in] name Name of the node.
	 @param[in] parent Parent host
	 */
	Node(const std::string & name, Host * parent);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Node();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mParent. Returns value of mParent.

	 @return The value of mParent
	*/
	Host * getParent() const;

	/**
	 Getter of mSearchPath. Returns value of mSearchPath.

	 @return The value of mSearchPath
	*/
	const PCstring & getSearchPath() const;

	/**
	 Getter of mProcesses. Returns value of mProcesses.

	 @return The value of mProcesses
	*/
	Container < Process, Mutex >::Pointer & getProcesses();

	/**
	 Getter of mOutputDocument. Returns value of mOutputDocument.

	 @return The value of mOutputDocument
	*/
	OutputNode::Pointer & getOutputDocument();

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initialize the node.
	 */
	virtual void initialize();

	/** 
	 Finalize the node.
	 */
	virtual void finalize();

	/** 
	 Create process on this node.
	 @param[in] processName Name of the process (C string for squirrel compatibility).
	 @return Pointer to the created process.
	 */
	virtual Process *createProcess(const char *processName);

	/** 
	 Create buffer on this node.
	 @param[in] name Name of the buffer
	 @param[in] left X offset of the image (it is 0 for whole images).
	 @param[in] top Y offset of the image (it is 0 for whole images).
	 @param[in] width Width of the image.
	 @param[in] height Height of the image.
	 @param[in] depthFormat Depth format used by this image.
	 @return Pointer to the created buffer.
	 */
	virtual Buffer *createBuffer(const char *name, const PCuint left, const PCuint top,
				  const PCuint width, const PCuint height,
				  const PCuint depthFormat);

	/** 
	 Get buffer by name.
	 @param[in] name Name of the buffer
	 @return Found buffer.
	 */
	virtual Buffer::Pointer getBuffer(const std::string name);

	/** 
	 Start the processes.
	 */
	virtual void start();

	/** 
	 Stop the processes.
	 @return frameID
	 */
	virtual u32 stop();

	/** 
	 Collect data from nodes.
	 */
	virtual void collectData();

	/** 
	 Set the ending frameID of processes.
	 @param[in] frameID StopID.
	 */
	virtual void setFrameID(const u32 & frameID);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMNode_impl.h"

}

// Declare Node class (and its enums) for SqPlus binding
/// \skip DECLARE_INSTANCE_TYPE_NAME
DECLARE_INSTANCE_TYPE_NAME(ParCompMark::Node, Node);

#endif
