
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMOpenGLExtensionLoader.h
/// Contains declaration of class OpenGLExtensionLoader

#ifndef __PCM_OPENGLEXTENSIONLOADER_H__
#  define __PCM_OPENGLEXTENSIONLOADER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

//
// Outer includes
//

#  include <list>

namespace ParCompMark
{

  /**
	The OpenGLExtensionLoader class functions load and check OpenGL extensions.
		The loader has immediate and delayed modes. Delayed mode is usefull for multidiplay applications and delayed GLX context creation.

   */

  class OpenGLExtensionLoader
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestOpenGLExtensionLoader;
#  endif

	//
	// Enumerations
	//

   public:

  /** Extension loading mode types. */
	enum LoadingMode
	{

	/** The externsions are loaded immedatelly when the load() method is called. */
	 IMMEDIATE,

	/** The load() method only registers the extensions, the loadDelayed() method loads them effectively. */
	 DELAYED
	};

	//
	// Class variables
	//

	/// @name Class variables
	//@{  

   protected:

	/** Extension loading mode.

	 @remarks
	 This is own attribute of this class.
	*/
	static LoadingMode mLoadingMode;

	/** Stores extensions to be loaded when loadDelayed() is called.

	 @remarks
	 This is own attribute of this class.
	*/
	static std::list < std::string > mExtensionsToLoad;

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mLoadingMode. Returns value of mLoadingMode.

	 @return The value of mLoadingMode
	*/
	static const OpenGLExtensionLoader::LoadingMode & getLoadingMode();

	/**
	 Setter of mLoadingMode. Sets value of mLoadingMode.

	 @param[in] loadingmode The value of mLoadingMode
	*/
	static void setLoadingMode(const OpenGLExtensionLoader::LoadingMode & loadingmode);

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 

			Load the specified extensions (immediatelly or later depending on mLoadingMode).

	 @param[in] extensions List of OpenGL extensions to load.
	 */
	static void load(const std::list < std::string > &extensions);

	/** 
	 Load queued extensions now.
	 */
	static void loadDelayed();

   protected:

	/** 

			Load OpenGL extensions in the specified list. One extensions is loaded only once.
			This function was separated from the rest of the rendering functions due to glew inclusion problems.

	 @param[in] extensions List of OpenGL extensions to load.
	 */
	static void _load(const std::list < std::string > &extensions);

	/** 

			Initialize the extensions.

	 */
	static void _init();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMOpenGLExtensionLoader_impl.h"

}

#endif
