
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMOutputNode.h
/// Contains declaration of class OutputNode

#ifndef __PCM_OUTPUTNODE_H__
#  define __PCM_OUTPUTNODE_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMName.h"

#  include "PCMStringConverter.h"

//
// Outer includes
//

#  include <string>

#  include <sstream>

#  include <list>

#  include <map>

#  include <algorithm>

namespace ParCompMark
{

  /**
	Node of the output document.
   */

  class OutputNode:public Name
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestOutputNode;
#  endif

	//
	// Enumerations
	//

   public:

  /** Node type definitions. */
	enum NodeType
	{

	/** Define something (cluster, host, node, etc.), anything that can be described in the script or in the GUI. */
	 DEFINITION,

	/** Information about the hardware or software element. The user cannot redefine its value. */
	 INFORMATION,

	/** A reference, new element cannot be defined, only referencing an existing element is possible. */
	 REFERENCE,

	/** Statistics, it cannot be defined, referenced, or determined from the hardware directly. These values are measured during the benchmark process. */
	 STATISTICS,

	/** The node is a text field. */
	 TEXT,

	/** The node is a CDATA text field. */
	 CDATA
	};

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < OutputNode,
	Mutex > Pointer;

  /** Type definition for list of child nodes. */
	typedef std::list < OutputNode::Pointer > ChildNodeList;

  /** Type definition for iterator on list of child nodes. */
	typedef ChildNodeList::iterator ChildNodeListIterator;

  /** Type definition for map of attributes. */
	typedef std::map < std::string,
	std::string > AttributeMap;

  /** Type definition for iterator on map of attributes. */
	typedef AttributeMap::const_iterator AttributeMapIterator;

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   protected:

	/** String constant for node name for text data nodes.

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string TEXTNODENAME;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Type of the node.

	 @remarks
	 This is own attribute of this class.
	*/
	NodeType mType;

	/** Attributes of the node.

	 @remarks
	 This is own attribute of this class.
	*/
	AttributeMap mAttributes;

	/** Child nodes.

	 @remarks
	 This is own attribute of this class.
	*/
	ChildNodeList mChildren;

	/** Text data.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mText;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Creates output node.
	 @param[in] name Name of the node.
	 @param[in] type Type of the node.
	 */
	OutputNode(const std::string & name, const NodeType & type);

	/** 
	 Creates text field.
	 @param[in] text Text data.
	 */
	OutputNode(const std::string & text);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ OutputNode();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of TEXTNODENAME. Returns value of TEXTNODENAME.

	 @return The value of TEXTNODENAME
	*/
	static const std::string & getEXTNODENAME();

	/**
	 Getter of mType. Returns value of mType.

	 @return The value of mType
	*/
	const OutputNode::NodeType & getType() const;

	/**
	 Getter of mText. Returns value of mText.

	 @return The value of mText
	*/
	const std::string & getText() const;

	/**
	 Setter of mText. Sets value of mText.

	 @param[in] text The value of mText
	*/
	void setText(const std::string & text);

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Parse XML document and construct its OutputNode equivalent.
	 @param[out] xmlDocument XML document or fragment to parse.
	 @return Root node of the parsed document
	 */
	static OutputNode::Pointer parseFromXML(std::string & xmlDocument);

   protected:

	/** 
	 Test name if its a correct xml name.
	 @param[in] name Name to test.
	 @return True if the specified name is a correct XML name
	 */
	static bool _testXMLName(const std::string & name);

	/** 
	 Convert special characters to XML entites.
	 @param[in] string String to convert.
	 @return Converted string
	 */
	static std::string _convertSpecialChars(const std::string & string);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Create nontextual child node.
	 @param[in] name Name of the child node.
	 @param[in] type Type of the child node.
	 @return Created child node.
	 */
	virtual OutputNode::Pointer createChildNode(const std::string & name, const NodeType & type);

	/** 
	 Create textual child node.
	 @param[in] text Text data.
	 @return Created child node.
	 */
	virtual OutputNode::Pointer createChildNode(const std::string & text);

	/** 
	 Add child node.
	 @param[in] child Child node.
	 */
	virtual void addChildNode(OutputNode::Pointer child);

	/** 
	 Return the number of child nodes.
	 @return The number of child nodes.
	 */
	virtual u32 getChildCount();

	/** 
	 Return first child node.
	 @return First child node.
	 */
	virtual OutputNode::Pointer getFirstChildNode();

	/** 
	 Return true if the node has attribute with the specified name.
	 @param[in] attribute Name of the attribute.
	 @return The node has the specified attribute.
	 */
	virtual bool hasAttribute(const std::string & attribute) const;

	/** 
	 Get an attribute by name.
	 @param[in] attribute Name of the attribute.
	 @return Attribute value.
	 */
	virtual const std::string & getAttribute(const std::string & attribute) const;

	/** 
	 Set attribute, also create if does not exists, overwrite otherwise.
	 @param[in] attribute Name of the attribute.
	 @param[in] value Value of the attribute.
	 */
	virtual void setAttribute(const std::string & attribute, const std::string & value);

	/** 
	 Serialize the node to XML (this is a recursive method, calls the same methods of the children too).
	 @param[out] osstr Output string stream.
	 @return Serialized node (added to osstr).
	 */
	virtual std::ostringstream & serialize2XML(std::ostringstream & osstr);

	/** 
	 Serialize the node to XML (this is a recursive method, calls the same methods of the children too). Use the other version of this method with parameter std::ostringstream if it is possible for memory saving.
	 @return Serialized node.
	 */
	virtual std::string serialize2XML();

	/** 
	 Refresh datas.
	 */
	virtual void refreshData();

	/** 
	 Clean outputnode.
	 */
	virtual void clean();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMOutputNode_impl.h"

}

#endif
