
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMPlugin.h
/// Contains declaration of class Plugin

#ifndef __PCM_PLUGIN_H__
#  define __PCM_PLUGIN_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMDynLoad.h"

#  include "PCMName.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMPointer.h"

//
// Outer includes
//

#  include <list>

namespace ParCompMark
{

  /**
	Application plugin.
   */

  class Plugin:public DynLoad,
   public Name
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestPlugin;
#  endif

	//
	// Enumerations
	//

   public:

  /** Definitions of plugin types. */
	enum PluginType
	{

	/** Type for Plugin class */
	 BASIC_PLUGIN,

	/** Type for Renderer class */
	 RENDERER_PLUGIN
	};

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef ParCompMark::Pointer < Plugin,
	Mutex > Pointer;

   protected:

  /** Function pointer type for */
	typedef const char **(*pcmGetNeededLibsType) ();

  /** Function pointer type for the argument of */
	typedef void (*loggerFunctionType) (const void *, const char *);

  /** Function pointer type for */
	typedef int (*pcmSetPluginHandleType) (void *);

  /** Function pointer type for */
	typedef int (*pcmSetLoggerFunctionType) (loggerFunctionType);

  /** Function pointer type for */
	typedef int (*pcmOnLoadType) ();

  /** Function pointer type for */
	typedef int (*pcmOnUnloadType) ();

  /** Function pointer type for */
	typedef const char *(*pcmGetErrorMsgType) (const int);

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Plugin type.

	 @remarks
	 This is own attribute of this class.
	*/
	PluginType mPluginType;

	/** The plugin is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** Needed dynamic libraries.

	 @remarks
	 This is own attribute of this class.
	*/
	std::list < std::string > mNeededLibs;

	/** Error code of the last error occured by the plugin code. 0 must mean no error.

	 @remarks
	 This is own attribute of this class.
	*/
	s32 mLastError;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create a plugin.
	 @param[in] type Type of the plugin.
	 @param[in] name Name of the plugin.
	 @param[in] filename Plugin file name.
	 */
	Plugin(const PluginType & type, const std::string & name, const std::string & filename);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Plugin();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mPluginType. Returns value of mPluginType.

	 @return The value of mPluginType
	*/
	const Plugin::PluginType & getPluginType() const;

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mNeededLibs. Returns value of mNeededLibs.

	 @return The value of mNeededLibs
	*/
	const std::list < std::string > &getNeededLibs() const;

	/**
	 Getter of mLastError. Returns value of mLastError.

	 @return The value of mLastError
	*/
	const s32 & getLastError() const;

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   protected:

	/** 
	 Logger function with simple C style interface can be called by the plugin.
	 @param[in] handle Plugin handle
	 @param[in] message Message to log.
	 */
	static void _loggerFunction(const void *handle, const char *message);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initialize the plugin.
	 */
	virtual void initialize();

	/** 
	 Finalize the plugin.
	 */
	virtual void finalize();

   protected:

	/** 
	 Specific initialization code.
	 */
	virtual void _initializeSpecific() = 0;

	/** 
	 Specific finalization code.
	 */
	virtual void _finalizeSpecific() = 0;

	/** 
	 Get needed libraries from the plugin.
	 */
	virtual void _getNeededLibs();

	/** 
	 Set plugin handle to the plugin.
	 */
	virtual void _setPluginHandle();

	/** 
	 Set logger function to the plugin.
	 */
	virtual void _setLoggerFunction();

	/** 
	 Call pcmOnLoad event handler of the plugin.
	 */
	virtual void _onLoad();

	/** 
	 Call pcmOnUnload event handler of the plugin.
	 */
	virtual void _onUnload();

	/** 
	 Call pcmGetErrorMsg function of the plugin.
	 @return Description of the last error.
	 */
	virtual const char *_getErrorMsg();

	/** 
	 Check the error code of the last operation.
	 */
	virtual void _checkError();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMPlugin_impl.h"

}

#endif
