
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMPluginManager.h
/// Contains declaration of class PluginManager

#ifndef __PCM_PLUGINMANAGER_H__
#  define __PCM_PLUGINMANAGER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMSingleton.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMContainer.h"

#  include "PCMPointer.h"

#  include "PCMPlugin.h"

namespace ParCompMark
{

  /**
	Singleton plugin loader and handler class.
   */

  class PluginManager:public Singleton < PluginManager >
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestPluginManager;
#  endif

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Container of plugins.

	 @remarks
	 This is own attribute of this class.
	*/
	Container < Plugin, Mutex >::Pointer mPlugins;

	/** Plugin directory path, relative to application data directory (

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mPluginDirectory;

	/** Name of plugin ini file placed in mPluginDirectory.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mPluginIniFile;

	/** The plugin manager is initialized. Not used.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Creates plugin manager.
	 @param[in] pluginDirectory Plugin directory path, relative to application data directory
	 @param[in] iniFile Name of plugin ini file. Not used.
	 */
	PluginManager(const std::string & pluginDirectory = "plugins/", const std::string & iniFile =
			"plugins.ini");

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ PluginManager();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mPluginDirectory. Returns value of mPluginDirectory.

	 @return The value of mPluginDirectory
	*/
	const std::string & getPluginDirectory() const;

	/**
	 Getter of mPluginIniFile. Returns value of mPluginIniFile.

	 @return The value of mPluginIniFile
	*/
	const std::string & getPluginIniFile() const;

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initialize plugin manager.
	 */
	virtual void initialize();

	/** 
	 Finalize plugin manager.
	 */
	virtual void finalize();

	/** 
	 Load all plugins in described mPluginIniFile.
	 */
	virtual void loadPlugins();

	/** 
	 Load all plugins in described mPluginIniFile. Not used.
	 */
	virtual void loadPlugins_old();

	/** 
	 Unload all loaded plugins.
	 */
	virtual void unloadPlugins();

	/** 
	 Get plugin by name.
	 @param[in] name Name of the plugin.
	 @return Pointer to the plugin.
	 */
	virtual Plugin::Pointer getPlugin(const std::string & name);

   protected:

	/** 
	 Load a plugin for the given filename relative to mPluginDirectory.
	 @param[in] type Pointer to the plugin.
	 @param[in] filename Filename relative to mPluginDirectory.
	 @return Pointer to the plugin.
	 */
	virtual Plugin::Pointer _loadPlugin(const Plugin::PluginType & type,
					  const std::string & filename);

	/** 
	 Check needed libs for the specified plugin. Throw exception when a lib needed by the plugin is not linked to parcompmark.
	 @param[out] plugin Type of the plugin.
	 */
	virtual void _checkNeededLibs(Plugin::Pointer & plugin);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMPluginManager_impl.h"

}

#endif
