
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMPointer.h
/// Contains declaration of class Pointer

#ifndef __PCM_POINTER_H__
#  define __PCM_POINTER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMException.h"

#  include "PCMDummyLock.h"

#  include "PCMMutex.h"

namespace ParCompMark
{

  /**

		Template smart pointer class.

	 @remarks

			This class provides: exception safe, garbage colletion, thread safeness, and more efficiency

   */
  template < typename T, class Lock > class Pointer
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestPointer;
#  endif

	//
	// Typedefs
	//

   protected:

  /** Meta field type for the referenced object. */
	typedef struct

	{		 /** Pointer to the referenced object */

	 T *ptr;								 /** Indicates that the object is deleted */

	 bool dead;																 /** Usage counter */

	 u32 usage;																					  /** Indicates that the referenced memory should deallocated by the smart pointer */

	 bool ownMemory;																																		/** Lock for the object */
	 Lock lock;
	} Meta;

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** Null pointer constant.

	 @remarks
	 This is own attribute of this class.
	*/
	static const Pointer < T,
	Lock > *NULLPTR;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Meta field for the referenced object.

	 @remarks
	 This is own attribute of this class.
	*/
	Meta * mMeta;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create a NULL pointer.
	 */
	Pointer();

	/** 
	 Create smart pointer from a C style pointer.
	 @param[in] pointer C style pointer
	 @param[in] takeOwnership Takes ownership of the assigned object. It will be deleted when smart pointer dies.
	 */
	explicit Pointer(const T * pointer, const bool & takeOwnership = true);

	/** 
	 Create smart pointer from an another smart pointer. (Copy constructor)
	 @param[out] pointer Another smart pointer to assign.
	 */
	Pointer(Pointer < T, Lock > &pointer);

	/** 
	 Create smart pointer from an another const smart pointer. (Constantant copy constructor)
	 @param[in] pointer Another smart pointer to assign.
	 */
	Pointer(const Pointer < T, Lock > &pointer);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Pointer();

	//@}  

	//
	// Operators
	//

	/// @name Operators
	//@{  

   public:

	/** 
	 Assign a C style pointer.
	 @param[in] pointer C style pointer
	 @return Self reference.
	 */
	virtual const Pointer < T, Lock > &operator =(const T * pointer);

	/** 
	 Assign another smart pointer.
	 @param[out] pointer Another smart pointer to assign.
	 @return Self reference.
	 */
	virtual const Pointer < T,
	Lock > &operator =(Pointer < T, Lock > &pointer);

	/** 
	 Assign another smart pointer (const version).
	 @param[in] pointer Another smart pointer to assign.
	 @return Self reference.
	 */
	virtual const Pointer < T,
	Lock > &operator =(const Pointer < T, Lock > &pointer);

	/** 
	 Get referenced object.
	 @return Referenced object.
	 */
	virtual T *operator ->();

	/** 
	 Equality test operator on a C style pointer.
	 @param[in] pointer C style pointer
	 @return True if the referenced object is identical to the parameter.
	 */
	virtual bool operator ==(const T * pointer);

	/** 
	 Equality test operator.
	 @param[out] pointer Another smart pointer to test.
	 @return True if the referenced objects are identical.
	 */
	virtual bool operator ==(Pointer < T, Lock > &pointer);

	/** 
	 Equality test operator on a C style pointer.
	 @param[in] pointer C style pointer
	 @return True if the referenced object is not identical to the parameter.
	 */
	virtual bool operator !=(const T * pointer);

	/** 
	 Equality test operator.
	 @param[out] pointer Another smart pointer to test.
	 @return True if the referenced objects are not identical.
	 */
	virtual bool operator !=(Pointer < T, Lock > &pointer);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Return true if the pointer does not reference anything.
	 @return True if the pointer does not reference anything.
	 */
	virtual bool isNull() const;

	/** 
	 Return true if the pointer does reference something.
	 @return True if the pointer does reference something.
	 */
	virtual bool isNotNull() const;

	/** 
	 Assign another smart pointer with keeping it locked.
	 @param[out] pointer Another smart pointer to assign.
	 */
	virtual void assignWithLock(Pointer < T, Lock > &pointer);

	/** 
	 Take reference from a C style pointer (does not delete referenced object when smart pointer dies).
	 @param[in] pointer C style pointer
	 */
	virtual void reference(const T * pointer);

	/** 
	 Return a C style pointer to the referenced object.
	 @return Pointer to the referenced object.
	 */
	virtual T *getPtr();

	/** 
	 Force deallocating referenced object.
	 @param[in] force If false, the object will only be deallocated when one it has one reference.
	 */
	virtual void kill(const bool & force = false);

	/** 

			Set the referenced object to null without trying to deallocate the actual object.

	 @remarks

				The typical usage of this method the external deallocating a pointer,
				like fclose.

	 @param[in] force If false, the reference will only be set to null when one it has one reference.
	 */
	virtual void setNull(const bool & force = false);

	/** 
	 Lock the referenced object.
	 */
	virtual void lock();

	/** 
	 Try locking referenced object.
	 @return True if the locking was successful.
	 */
	virtual bool trylock();

	/** 
	 Unlock the referenced object.
	 */
	virtual void unlock();

	/** 
	 Return true, if the object is locked.
	 @return True if the referenced object is locked.
	 */
	virtual bool getLocked() const;

   protected:

	/** 
	 Delete the pointer. Decrease reference counter, relese lock, free memory if needed.
	 @param[in] force If false, the object will only be deallocated when one it has one reference.
	 @param[in] keepLock Keep the pointer locked.
	 */
	virtual void _deletePointer(const bool & force = false, const bool & keepLock = false);

	/** 
	 Assign a C style pointer.
	 @param[in] pointer C style pointer
	 @param[in] takeOwnership Takes ownership of the assigned object. It will be deleted when smart pointer dies.
	 */
	virtual void _assignCPointer(const T * pointer, const bool & takeOwnership = true);

	/** 
	 Assign another smart pointer.
	 @param[out] pointer Another smart pointer to assign.
	 @param[in] keepLock Keep the pointer locked.
	 */
	virtual void _assignPointer(Pointer < T, Lock > &pointer, const bool & keepLock = false);

	/** 
	 Switch to another smart pointer. Helps changing reference under continous lock.
	 @param[out] pointer Another smart pointer to assign.
	 @param[in] keepLock Keep the pointer locked.
	 */
	virtual void _switchPointer(Pointer < T, Lock > &pointer, const bool & keepLock = false);

	/** 
	 Test indentity.
	 @param[in] pointer C style pointer
	 @return True if the data in parameter is identical to the smart pointer.
	 */
	virtual bool _equalsCPointer(const T * pointer) const;

	/** 
	 Test indentity.
	 @param[out] pointer Another smart pointer to assign.
	 @return True if the data in parameter is identical to the smart pointer.
	 */
	virtual bool _equalsPointer(Pointer < T, Lock > &pointer) const;

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMPointer_impl.h"

}

#endif
