
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMRenderer.h
/// Contains declaration of class Renderer

#ifndef __PCM_RENDERER_H__
#  define __PCM_RENDERER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMPointer.h"

#  include "PCMRendererPlugin.h"

#  include "PCMOpenGLRenderingEngine.h"

#  include "PCMProcess.h"

//
// Scripting bind includes (for Squirrel)
//

#  include <squirrel/squirrel.h>
#  include <squirrel/sqstdio.h>
#  include <squirrel/sqstdaux.h>
#  include <squirrel/sqplus.h>

namespace ParCompMark
{

  /**
	Custom renderer object belongs to a renderer plugin.
   */

  class Renderer
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestRenderer;
#  endif

	//
	// Scripting binding (for Squirrel)
	//

	/// @name Scripting binding
	//@{  

  /**
   Static glue code method for Squirrel-C++ binding. 
   This method should be call from each Squirrel virtual machines 
   to generate the proper Squirrel glue code.
   To content of this method is fully autogenerated in the header implementation file, 
   you should not modify it.
   */
   public:static void squirrelGlue()
	{
	 ::SqPlus::SQClassDef < Renderer > (_T("Renderer")).func(&Renderer::render,
								 _T("render")).func(&Renderer::setMiscParam,
										   _T("setMiscParam")).
		func(&Renderer::setObjectSpaceBoundingBox, _T("setObjectSpaceBoundingBox")).func(&Renderer::setObjectId,
												 _T("setObjectId")).
		func(&Renderer::setScreenSpaceFramelet,
		  _T("setScreenSpaceFramelet")).func(&Renderer::registerObjectSpaceBoundingBox,
							_T("registerObjectSpaceBoundingBox")).
		func(&Renderer::getAutoRenderOrder, _T("getAutoRenderOrder")).func(&Renderer::setAutoRenderOrder,
										 _T("setAutoRenderOrder"));
	}

	//@}  

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef ParCompMark::Pointer < Renderer,
	Mutex > Pointer;

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** Constant for indication, that the renderer does not render automatically.

	 @remarks
	 This is own attribute of this class.
	*/
	static const s32 NOAUTORENDERING;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Pointer to the renderer instance in the plugin code.

	 @remarks
	 This attribute references an attribute.
	*/
	void *mRendererHandle;

	/** Corresponding renderer plugin.

	 @remarks
	 This attribute references an attribute.
	*/
	RendererPlugin *mRendererPlugin;

	/** Parent rendering engine of the renderer.

	 @remarks
	 This attribute references an attribute.
	*/
	OpenGLRenderingEngine *mParent;

	/** The renderer is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** If this attribute not equals Renderer::NOAUTORENDERING, it is the order in the automatic rendering queue on the correspondent rendering engine.

	 @remarks
	 This is own attribute of this class.
	*/
	s32 mAutoRenderOrder;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Default constructor for Squirrel compatibility. Calling this constructor always raises an exception.
	 */
	Renderer();

	/** 
	 Create a custom renderer.
	 @param[in] rendererPlugin Corresponding renderer plugin.
	 @param[in] rendererHandle Pointer to the renderer instance in the plugin code.
	 @param[in] parent Parent rendering engine of the renderer.
	 */
	Renderer(RendererPlugin * rendererPlugin, void *rendererHandle, OpenGLRenderingEngine * parent);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Renderer();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mRendererHandle. Returns value of mRendererHandle.

	 @return The value of mRendererHandle
	*/
	void *getRendererHandle() const;

	/**
	 Setter of mRendererHandle. Sets value of mRendererHandle.

	 @param[in] rendererhandle The value of mRendererHandle
	*/
	void setRendererHandle(const void *rendererhandle);

	/**
	 Getter of mParent. Returns value of mParent.

	 @return The value of mParent
	*/
	OpenGLRenderingEngine *getParent() const;

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mAutoRenderOrder. Returns value of mAutoRenderOrder.

	 @return The value of mAutoRenderOrder
	*/
	s32 & getAutoRenderOrder();

	/**
	 Setter of mAutoRenderOrder. Sets value of mAutoRenderOrder.

	 @param[in] autorenderorder The value of mAutoRenderOrder
	*/
	void setAutoRenderOrder(const s32 autorenderorder);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initialize the renderer.
	 */
	virtual void initialize();

	/** 
	 Finalize the renderer.
	 */
	virtual void finalize();

	/** 
	 Call the <tt>pcmOnResize</tt> event handler of the plugin.
	 @param[in] width New width of the window on which the renderer renders.
	 @param[in] height New height of the window on which the renderer renders.
	 */
	virtual void resize(const u32 width, const u32 height);

	/** 
	 Call the <tt>pcmSetSortOrder</tt> function of the plugin.
	 @return Sort order. -1 value indicates that the plugin has no sort order calculation routine or the sort order is unknown.
	 */
	virtual s32 getSortOrder();

	/** 
	 Call the <tt>pcmOnRender</tt> event handler of the plugin.
	 */
	virtual void render();

	/** 
	 Call the <tt>pcmSetMiscParam</tt> function of the plugin.
	 @param[in] name Name of the parameter.
	 @param[in] value Value of the parameter.
	 */
	virtual void setMiscParam(const char *name, const char *value);

	/** 
	 Call the <tt>pcmSetObjectSpaceBoundingBox</tt> function of the plugin.
	 @param[in] x0 Lowest corner of the bounding box.
	 @param[in] y0 Lowest corner of the bounding box.
	 @param[in] z0 Lowest corner of the bounding box.
	 @param[in] x1 Highest corner of the bounding box.
	 @param[in] y1 Highest corner of the bounding box.
	 @param[in] z1 Highest corner of the bounding box.
	 */
	virtual void setObjectSpaceBoundingBox(const double x0, const double y0, const double z0,
					   const double x1, const double y1, const double z1);

	/** 
	 Call the <tt>pcmSetObjectId</tt> function of the plugin.
	 @param[in] objectId Render object with the specified id.
	 */
	virtual void setObjectId(const unsigned objectId);

	/** 
	 Call the <tt>pcmSetScreenSpaceFramelet</tt> function of the plugin.
	 @param[in] u0 Top-left corner of the framelet.
	 @param[in] v0 Top-left corner of the framelet.
	 @param[in] u1 Bottom-right corner of the framelet.
	 @param[in] v1 Bottom-right corner of the framelet.
	 */
	virtual void setScreenSpaceFramelet(const double u0, const double v0, const double u1,
					 const double v1);

	/** 
	 Call the <tt>pcmRegisterObjectSpaceBoundingBox</tt> function of the plugin.
	 @param[in] x0 Lowest corner of the bounding box.
	 @param[in] y0 Lowest corner of the bounding box.
	 @param[in] z0 Lowest corner of the bounding box.
	 @param[in] x1 Highest corner of the bounding box.
	 @param[in] y1 Highest corner of the bounding box.
	 @param[in] z1 Highest corner of the bounding box.
	 */
	virtual void registerObjectSpaceBoundingBox(const double x0, const double y0, const double z0,
						 const double x1, const double y1,
						 const double z1);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMRenderer_impl.h"

}

// Declare Renderer class (and its enums) for SqPlus binding
/// \skip DECLARE_INSTANCE_TYPE_NAME
DECLARE_INSTANCE_TYPE_NAME(ParCompMark::Renderer, Renderer);

#endif
