
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMRendererPlugin.h
/// Contains declaration of class RendererPlugin

#ifndef __PCM_RENDERERPLUGIN_H__
#  define __PCM_RENDERERPLUGIN_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMPlugin.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

#  include "PCMPointer.h"

#  include "PCMGLXRenderWindow.h"

#  include "PCMStringConverter.h"

//
// Outer includes
//

#  include <X11/Xlib.h>

#  include <GL/glew.h>

#  include <GL/glx.h>

#  include <list>

namespace ParCompMark
{

  /**
	Custom renderer plugin.
   */

  class RendererPlugin:public Plugin
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestRendererPlugin;
#  endif

	//
	// Typedefs
	//

   public:

  /** Buffer object of the plugin. */
	typedef struct
	{
	 unsigned left;
	 unsigned top;
	 unsigned width;

	 unsigned height;									 /** Colour data */
	 unsigned char *colour;
	} PluginBuffer;

  /** Type for pointer on this class. */
	typedef ParCompMark::Pointer < RendererPlugin,
	Mutex > Pointer;

  /** Function pointer type for */
	typedef int (*pcmSetMiscParamType) (void *, const char *, const char *);

  /** Function pointer type for */
	typedef int (*pcmSetObjectSpaceBoundingBoxType) (void *, double, double, double, double, double, double);

  /** Function pointer type for */
	typedef int (*pcmregisterObjectSpaceBoundingBoxType) (void *, double, double, double, double, double, double);

  /** Function pointer type for */
	typedef int (*pcmSetObjectIdType) (void *, unsigned);

  /** Function pointer type for */
	typedef int (*pcmSetScreenSpaceFrameletType) (void *, double, double, double, double);

  /** Function pointer type for */
	typedef int (*pcmOnResizeType) (void *, unsigned, unsigned);

  /** Function pointer type for */
	typedef int (*pcmGetSortOrderType) (void *);

  /** Function pointer type for */
	typedef int (*pcmOnRenderType) (void *, double, unsigned);

   protected:

  /** Function pointer type for the argument of */
	typedef void (*bufferGetterType) (const void *, const char *, void *);

  /** Function pointer type for */
	typedef const char **(*pcmGetNeededOpenGLExtsType) ();

  /** Function pointer type for */
	typedef void *(*pcmOnCreateRendererType) (void *, Display *, Window, XVisualInfo *, GLXContext);

  /** Function pointer type for */
	typedef int (*pcmOnDestroyRendererType) (void *);

  /** Function pointer type for */
	typedef int (*pcmSetBufferGetterType) (bufferGetterType);

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Needed OpenGL extensions.

	 @remarks
	 This is own attribute of this class.
	*/
	std::list < std::string > mNeededOpenGLExts;

	/** Function pointer for 

	 @remarks
	 This is own attribute of this class.
	*/
	RendererPlugin::pcmOnResizeType mPcmOnResize;

	/** Function pointer for 

	 @remarks
	 This is own attribute of this class.
	*/
	RendererPlugin::pcmGetSortOrderType mPcmGetSortOrder;

	/** Function pointer for 

	 @remarks
	 This is own attribute of this class.
	*/
	RendererPlugin::pcmOnRenderType mPcmOnRender;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create a renderer plugin.
	 @param[in] type Type of the plugin.
	 @param[in] name Name of the renderer plugin.
	 @param[in] filename Plugin file name.
	 */
	RendererPlugin(const Plugin::PluginType & type, const std::string & name,
			 const std::string & filename);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ RendererPlugin();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mNeededOpenGLExts. Returns value of mNeededOpenGLExts.

	 @return The value of mNeededOpenGLExts
	*/
	const std::list < std::string > &getNeededOpenGLExts() const;

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   protected:

	/** 

	 @par
	Buffer getter function with simple C style interface can be called by the plugin.

	 @par
	If the <tt>pBufferData</tt> is not null, the specified buffer will be locked and its data will be retrieved.
				When the <tt>pBufferData</tt> is null, the locked buffer will be unlocked.

	 @param[in] pRenderer Pointer to the correspondent ParCompMark::Renderer object.
	 @param[in] bufferName Name of the buffer to get.
	 @param[in] pBufferData Pointer to the buffer description data.
	 */
	static void _bufferGetter(const void *pRenderer, const char *bufferName, void *pBufferData);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Create renderer object. Also call OpenGL intialization code for the plugin.
	 @param[in] window The window on which the renderer will render.
	 @param[in] parent Parent rendering engine of the renderer.
	 @return Created renderer.
	 */
	virtual Renderer * createRenderer(GLXRenderWindow::Pointer window, OpenGLRenderingEngine * parent);

	/** 
	 Destroy the specified renderer.
	 @param[in] renderer Renderer to destroy.
	 */
	virtual void destroyRenderer(Renderer * renderer);

	/** 
	 Call the <tt>pcmOnResize</tt> event handler of the plugin.
	 @param[in] rendererHandle Pointer to the renderer instance in the plugin code.
	 @param[in] width New width of the window on which the renderer renders.
	 @param[in] height New height of the window on which the renderer renders.
	 */
	virtual void resize(void *rendererHandle, const u32 & width, const u32 & height);

	/** 
	 Call the <tt>pcmGetSortOrder</tt> function of the plugin.
	 @param[in] rendererHandle Pointer to the renderer instance in the plugin code.
	 @return Sort order. -1 value indicates that the plugin has no sort order calculation routine or the sort order is unknown.
	 */
	virtual s32 getSortOrder(void *rendererHandle);

	/** 
	 Call the <tt>pcmOnRender</tt> event handler of the plugin.
	 @param[in] rendererHandle Pointer to the renderer instance in the plugin code.
	 @param[in] time Time since start of the rendering.
	 @param[in] frame Frame number.
	 */
	virtual void render(void *rendererHandle, const double &time, const unsigned &frame);

	/** 
	 Call the <tt>pcmSetMiscParam</tt> function of the plugin.
	 @param[in] rendererHandle Pointer to the renderer instance in the plugin code.
	 @param[in] name Name of the parameter.
	 @param[in] value Value of the parameter.
	 */
	virtual void setMiscParam(void *rendererHandle, const char *&name, const char *&value);

	/** 
	 Call the <tt>pcmSetObjectSpaceBoundingBox</tt> function of the plugin.
	 @param[in] rendererHandle Pointer to the renderer instance in the plugin code.
	 @param[in] x0 Lowest corner of the bounding box.
	 @param[in] y0 Lowest corner of the bounding box.
	 @param[in] z0 Lowest corner of the bounding box.
	 @param[in] x1 Highest corner of the bounding box.
	 @param[in] y1 Highest corner of the bounding box.
	 @param[in] z1 Highest corner of the bounding box.
	 */
	virtual void setObjectSpaceBoundingBox(void *rendererHandle, const double &x0, const double &y0,
					   const double &z0, const double &x1, const double &y1,
					   const double &z1);

	/** 
	 Call the <tt>pcmSetObjectId</tt> function of the plugin.
	 @param[in] rendererHandle Pointer to the renderer instance in the plugin code.
	 @param[in] objectId Render object with the specified id.
	 */
	virtual void setObjectId(void *rendererHandle, const unsigned &objectId);

	/** 
	 Call the <tt>pcmSetScreenSpaceFramelet</tt> function of the plugin.
	 @param[in] rendererHandle Pointer to the renderer instance in the plugin code.
	 @param[in] u0 Top-left corner of the framelet.
	 @param[in] v0 Top-left corner of the framelet.
	 @param[in] u1 Bottom-right corner of the framelet.
	 @param[in] v1 Bottom-right corner of the framelet.
	 */
	virtual void setScreenSpaceFramelet(void *rendererHandle, const double &u0, const double &v0,
					 const double &u1, const double &v1);

	/** 
	 Call the <tt>pcmRegisterObjectSpaceBoundingBox</tt> function of the plugin.
	 @param[in] rendererHandle Pointer to the renderer instance in the plugin code.
	 @param[in] x0 Lowest corner of the bounding box.
	 @param[in] y0 Lowest corner of the bounding box.
	 @param[in] z0 Lowest corner of the bounding box.
	 @param[in] x1 Highest corner of the bounding box.
	 @param[in] y1 Highest corner of the bounding box.
	 @param[in] z1 Highest corner of the bounding box.
	 */
	virtual void registerObjectSpaceBoundingBox(void *rendererHandle, const double &x0,
						 const double &y0, const double &z0,
						 const double &x1, const double &y1,
						 const double &z1);

   protected:

	/** 
	 Specific initialization code.
	 */
	virtual void _initializeSpecific();

	/** 
	 Specific finalization code.
	 */
	virtual void _finalizeSpecific();

	/** 
	 Set buffer getter function to the plugin.
	 */
	virtual void _setBufferGetter();

	/** 
	 Get needed OpenGL externsions from the plugin.
	 */
	virtual void _getNeededOpenGLExts();

	/** 
	 Initialize mPcmOnResize and mPcmOnRender function pointers. These function calls should be fast.
	 */
	virtual void _initFastFunctions();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMRendererPlugin_impl.h"

}

#endif
