
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMSqVM.h
/// Contains declaration of class SqVM

#ifndef __PCM_SQVM_H__
#  define __PCM_SQVM_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMName.h"

#  include "PCMLogger.h"

#  include "PCMFileSystemManager.h"

#  include "PCMException.h"

#  include "PCMPointer.h"

#  include "PCMContainer.h"

//
// Outer includes
//

#  include <squirrel/squirrel.h>

#  include <squirrel/sqstdio.h>

#  include <squirrel/sqstdaux.h>

#  include <squirrel/sqplus.h>

#  include <list>

namespace ParCompMark
{

  /**
	Squirrel virtual machine.
   */

  class SqVM:public Name
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestSqVM;
#  endif

	//
	// Enumerations
	//

   public:

  /** Definitions for output of scripts. */
	enum ScriptOutput
	{

	/** The output of the script is the ParCompMark logging system */
	 LOG,

	/** The output of the script is the standard output and the standard error stream */
	 STD
	};

   protected:

  /** Definitions of script states. */
	enum ScriptState
	{

	/** The script is not compiled */
	 UNCOMPILED,

	/** The script is compiled */
	 COMPILED,

	/** The script is executed (main method called) */
	 EXECUTED
	};

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < SqVM,
	Mutex > Pointer;

   protected:

  /** Struct for script attributes. */
	typedef struct Script

	{		  /** Indicate that the script is dynamic otherwise loaded from a file */

	 bool dynamic;												 /** Name of the script (filename or dynamic script name) */

	  std::string name;																						 /** Script containing the source code. This is only set at dynamic scripts */

	  std::string scriptString;																																			 /** Squirrel script object */

	 SquirrelObject scriptObject;																																										   /** The loaded script */

	 bool compiled;																																																		  /** Name of the entry method */

	  std::string mainMethod;																																																									 /** The script has a return value */

	 bool hasReturn;																																																																	   /** Return value of the last execution */

	  std::string returnValue;																																																																										 /** Script parameters */

	 const SQChar *parameters[10];																																																																																	   /** Number of parameters */

	 u32 parameterCount;																																																																																										  /** Smart pointer on this struct */
	 typedef ParCompMark::Pointer < Script,
	  DummyLock > Pointer;
	} Script;

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** Constant for indicating that no main method are defined for a script.

	 @remarks
	 This is own attribute of this class.
	*/
	static const std::string NOMAINMETHOD;

   protected:

	/** Size of mStringBuffer.

	 @remarks
	 This is own attribute of this class.
	*/
	static const u32 mStringBufferSize;

	//@}  

	//
	// Class variables
	//

	/// @name Class variables
	//@{  

   protected:

	/** Currently active Squirrel Virtual Machine. Only one VM can be active.

	 @remarks
	 This is own attribute of this class.
	*/
	static SqVM::Pointer mCurrentVM;

	/** 
			External mutex for mCurrentVM. 

	 @remarks
	 This is own attribute of this class.
	*/
	static Mutex mCurrentVMLock;

	/** Script output.

	 @remarks
	 This is own attribute of this class.
	*/
	static SqVM::ScriptOutput mScriptOutput;

	/** Common C style string buffer of printFunction.

	 @remarks
	 This is own attribute of this class.
	*/
	static char mStringBuffer[32768];

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Smart pointer on this object.

	 @remarks
	 This is own attribute of this class.
	*/
	SqVM::Pointer mThis;

	/** Squirrel virtual machine.

	 @remarks
	 This attribute references an attribute.
	*/
	SquirrelVMSys *mSquirrelVMSys;

	/** An error occured on this virtual machine.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mError;

	/** The virtual machine is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** Script container. Dynamic script can be also stored here when they have a proper name.

	 @remarks
	 This is own attribute of this class.
	*/
	Container < SqVM::Script, Mutex >::Pointer mScripts;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create Squirrel virtual machine.
	 @param[in] name Name of the virtual machine.
	 */
	SqVM(const std::string & name);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ SqVM();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mError. Returns value of mError.

	 @return The value of mError
	*/
	const bool & getError() const;

	/**
	 Setter of mError. Sets value of mError.

	 @param[in] error The value of mError
	*/
	void setError(const bool & error);

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mScriptOutput. Returns value of mScriptOutput.

	 @return The value of mScriptOutput
	*/
	static const SqVM::ScriptOutput & getScriptOutput();

	/**
	 Setter of mScriptOutput. Sets value of mScriptOutput.

	 @param[in] scriptoutput The value of mScriptOutput
	*/
	static void setScriptOutput(const SqVM::ScriptOutput & scriptoutput);

	/**
	 Getter of mStringBufferSize. Returns value of mStringBufferSize.

	 @return The value of mStringBufferSize
	*/
	static const u32 & getStringBufferSize();

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   protected:

	/** 
	 The print function of the virtual machine.
			This function is used by the built-in function '::print()' to output text.

	 @param[in] sqVM Squirrel VM
	 @param[in] s Format string
	 @param[in] ... Additional parameters
	 */
	static void printFunction(::HSQUIRRELVM sqVM, const SQChar * s, ...);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Execute script loaded from file. Giving SqVM::NOMAINMETHOD as mainMethod indicates that no main method.
	 @param[in] filename Script filename
	 @param[in] mainMethod Script entry method name. Giving SqVM::NOMAINMETHOD as mainMethod indicates that no main method.
	 @param[in] parameters List of passed string parameters
	 @param[in] hasReturn The script has a return value
	 @return The return value of the script.
	 */
	virtual std::string runScriptFromFile(const std::string & filename,
						const std::string & mainMethod =
						"main", const std::list < std::string > &parameters =
						std::list < std::string > (), const bool & hasReturn = false);

	/** 
	 Execute script from the given string. 
			If the scriptName is not empty the VM store this script into the class level store for better performance.
			Giving SqVM::NOMAINMETHOD as mainMethod indicates that no main method.
	 @param[in] scriptString Script string
	 @param[in] scriptName Name of the script. If it is not empty the VM store this script into the class level store for better performance.
	 @param[in] mainMethod Script entry method name. Giving SqVM::NOMAINMETHOD as mainMethod indicates that no main method.
	 @param[in] parameters List of passed string parameters
	 @param[in] hasReturn The script has a return value
	 @return The return value of the script.
	 */
	virtual std::string runScriptFromString(const std::string & scriptString,
						const std::string & scriptName =
						"", const std::string & mainMethod =
						"main", const std::list < std::string > &parameters =
						std::list < std::string > (), const bool & hasReturn = false);

	/** 
	 Execute a previously stored script.
	 @param[in] scriptName Name of a dynamic script name or filename.
	 @param[in] parameters List of passed string parameters
	 @return The return value of the script.
	 */
	virtual std::string runScriptByName(const std::string & scriptName,
					 const std::list < std::string > &parameters =
					 std::list < std::string > ());

	/** 
	 Finalize virtual machine.
	 */
	virtual void finalize();

   protected:

	/** 
	 Activate this VM. This virtual machine will be selected to operate.
	 */
	virtual void activate();

	/** 
	 Deactivate this VM. This virtual machine will go to sleep and let other VMs to be activated.
	 */
	virtual void deactivate();

	/** 
	 Initialize virtual machine.
	 */
	virtual void initialize();

	/** 

			Create script object with the specified name and entry point.
			The dynamic flag is also set, and the returned script object is locked by default.

	 @param[in] scriptName Name of the script.
	 @param[in] dynamic Dynamic flag.
	 @param[in] mainMethod Script entry method name. Giving SqVM::NOMAINMETHOD as mainMethod indicates that no main method.
	 @param[in] hasReturn The script has a return value
	 @return Pointer to the created script object.
	 */
	virtual SqVM::Script::Pointer createScript(const std::string & scriptName =
						 "", const bool & dynamic =
						 false, const std::string & mainMethod =
						 "main", const bool & hasReturn = false);

	/** 
	 Finds a prevously added script.
	 @param[in] scriptName Name of a dynamic script name or filename.
	 @return Pointer of the found script object.
	 */
	virtual SqVM::Script::Pointer findScript(const std::string & scriptName);

	/** 
	 If the script exists with the given name then returns, if not then adds it to the internal class level container and return it.
	 @param[in] scriptName Name of the searched script.
	 @param[in] dynamic Dynamic flag; if the script is have to be created this flag is set.
	 @param[in] mainMethod Script entry method name. Giving SqVM::NOMAINMETHOD as mainMethod indicates that no main method.
	 @param[in] hasReturn The script has a return value
	 @return Pointer of the found or the newly created script object.
	 */
	virtual SqVM::Script::Pointer findOrAddScript(const std::string & scriptName, const bool & dynamic =
						  false, const std::string & mainMethod =
						  "main", const bool & hasReturn = false);

	/** 
	 Compile and execute the script.
	 @param[out] script Script handle.
	 */
	virtual void compileAndExecuteScript(SqVM::Script::Pointer & script);

	/** 
	 Set parameters of a script.
	 @param[out] script Script handle.
	 @param[in] parameters List of passed string parameters
	 */
	virtual void setParameters(SqVM::Script::Pointer & script, const std::list < std::string > &parameters);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMSqVM_impl.h"

}

#endif
