
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMStringConverter.h
/// Contains declaration of class StringConverter

#ifndef __PCM_STRINGCONVERTER_H__
#  define __PCM_STRINGCONVERTER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMException.h"

#  include "PCMLogger.h"

//
// Outer includes
//

#  include <string>

#  include <sstream>

#  include <vector>

namespace ParCompMark
{

  /**
	Convert different types to std::string.
   */

  class StringConverter
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestStringConverter;
#  endif

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** Constant for default precision.

	 @remarks
	 This is own attribute of this class.
	*/
	static const s32 DEFAULTPRECISION;

	/** Constant for default field width.

	 @remarks
	 This is own attribute of this class.
	*/
	static const s32 DEFAULTFIELDWIDTH;

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Convert u8 to string.
	 @param[in] value Value to convert.
	 @return Converted string value
	 */
	static std::string toString(const u8 & value);

	/** 
	 Convert u16 to string.
	 @param[in] value Value to convert.
	 @return Converted string value
	 */
	static std::string toString(const u16 & value);

	/** 
	 Convert u32 to string.
	 @param[in] value Value to convert.
	 @return Converted string value
	 */
	static std::string toString(const u32 & value);

	/** 
	 Convert s8 to string.
	 @param[in] value Value to convert.
	 @return Converted string value
	 */
	static std::string toString(const s8 & value);

	/** 
	 Convert s16 to string.
	 @param[in] value Value to convert.
	 @return Converted string value
	 */
	static std::string toString(const s16 & value);

	/** 
	 Convert s32 to string.
	 @param[in] value Value to convert.
	 @return Converted string value
	 */
	static std::string toString(const s32 & value);

	/** 
	 Convert Real to string. If the precision is set, the real value is converted in fixed point notation scientific notation otherwise.
	 @param[in] value Value to convert.
	 @param[in] fieldWidth Field width.
	 @param[in] precision Number of fragment digits.
	 @return Converted string value
	 */
	static std::string toString(const Real & value, const s32 & fieldWidth =
				 StringConverter::DEFAULTFIELDWIDTH, const s32 & precision =
				 StringConverter::DEFAULTPRECISION);

	/** 
	 Convert bool to string.
	 @param[in] value Value to convert.
	 @return Converted string value
	 */
	static std::string toString(const bool value);

	/** 
	 Convert void * to string.
	 @param[in] value Value to convert.
	 @return Converted string value
	 */
	static std::string toString(const void *value);

	/** 
	 Convert string to u32.
	 @param[in] value Value to convert.
	 @return Converted u32 value
	 */
	static u32 toU32(const std::string value);

	/** 
	 Convert string to Real.
	 @param[in] value Value to convert.
	 @return Converted Real value
	 */
	static Real toReal(const std::string value);

	/** 
	 Convert string to void *.
	 @param[in] value Value to convert.
	 @return Converted pointer
	 */
	static void *parsePointer(const std::string value);

	/** 
	 Leave whitespaces from the beginning and the end of the string.
	 @param[out] str String to trim.
	 */
	static void trim(std::string & str);

	/** 
	 Tokenize the specified string. Original source: http://www.digitalpeer.com/id/simple
	 @param[in] str String to trim.
	 @param[in] delimiters String to trim.
	 @return Array of tokens.
	 */
	static std::vector < std::string > tokenize(const std::string & str, const char *delimiters);

   protected:

	/** 
	 Append empty spaces to fit to field width.
	 @param[in] str String to fill to fit to the field width.
	 @param[in] fieldWidth Field width.
	 @return Converted string value
	 */
	static std::string _fitFieldWidth(const std::string & str, const s32 & fieldWidth);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMStringConverter_impl.h"

}

#endif
