
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMThread.h
/// Contains declaration of class Thread

#ifndef __PCM_THREAD_H__
#  define __PCM_THREAD_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMLogger.h"

#  include "PCMException.h"

#  include "PCMTimer.h"

#  include "PCMPointer.h"

//
// Outer includes
//

#  include <pthread.h>

#  include <sys/time.h>

namespace ParCompMark
{

  /**
	This is a unique thread class.
   */

  class Thread
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestThread;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer to an int. */
	typedef ParCompMark::Pointer < bool,
	Mutex > BoolPointer;

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** Name of the thread.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mThreadName;

	/** Indicates that thread waits or not.

	 @remarks
	 This is own attribute of this class.
	*/
	BoolPointer mWait;

	/** The thread is requested to stop.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mStopRequested;

	/** The thread is created as joinable or not.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mJoinable;

	/** When class destructor call, wait thread or not.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mWaitThread;

	/** Thread handle.

	 @remarks
	 This is own attribute of this class.
	*/
	pthread_t mThread;

	/** Condition.

	 @remarks
	 This is own attribute of this class.
	*/
	pthread_cond_t mCondition;

	/** Condition.

	 @remarks
	 This is own attribute of this class.
	*/
	pthread_mutex_t mConditionMutex;

	/** The thread is running.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mRunning;

	/** The thread is running.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mCurrentFPS;

	/** The thread is running. Zero means no limit.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mExpectedFPS;

	/** Iteration number of task. Zero means nan.

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mIterationNumber;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Thread constructor.
	 @param[in] name Name of the thread.
	 */
	Thread(const std::string & name);

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ Thread();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mThreadName. Returns value of mThreadName.

	 @return The value of mThreadName
	*/
	const std::string & getThreadName() const;

	/**
	 Getter of mWait. Returns value of mWait.

	 @return The value of mWait
	*/
	const Thread::BoolPointer & getWait() const;

	/**
	 Getter of mStopRequested. Returns value of mStopRequested.

	 @return The value of mStopRequested
	*/
	const bool & getStopRequested() const;

	/**
	 Getter of mJoinable. Returns value of mJoinable.

	 @return The value of mJoinable
	*/
	const bool & getJoinable() const;

	/**
	 Getter of mWaitThread. Returns value of mWaitThread.

	 @return The value of mWaitThread
	*/
	const bool & getWaitThread() const;

	/**
	 Getter of mRunning. Returns value of mRunning.

	 @return The value of mRunning
	*/
	const bool & getRunning() const;

	/**
	 Getter of mCurrentFPS. Returns value of mCurrentFPS.

	 @return The value of mCurrentFPS
	*/
	const u32 & getCurrentFPS() const;

	/**
	 Getter of mExpectedFPS. Returns value of mExpectedFPS.

	 @return The value of mExpectedFPS
	*/
	const u32 & getExpectedFPS() const;

	/**
	 Getter of mIterationNumber. Returns value of mIterationNumber.

	 @return The value of mIterationNumber
	*/
	const u32 & getIterationNumber() const;

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Yield current thread. 
	 */
	static void yield();

	/** 
	 Get system time in us.
	 @return System time in us.
	 */
	static Real getUSTime();

   protected:

	/** 
	 Static entry point for the thread.
	 @param[in] arg This is only for thread.
	 @return This is only for thread.
	 */
	static void *entryPoint(void *arg);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Start thread.
	 @param[in] iterationNumber How much is the task running. Zero means nan.
	 @param[in] expectedFPS Expected FPS.
	 @param[in] joinable Joinable?
	 @param[in] waitThread Wait when destroy class?
	 */
	virtual void initThread(const u32 & iterationNumber = 0, const u32 & expectedFPS =
				0, const bool & joinable = false, const bool & waitThread = false);

	/** 
	 Some init step as thread.
	 */
	virtual void threadInitialize();

	/** 
	 Some finitiate step as thread.
	 */
	virtual void threadFinalize();

	/** 
	 Start thread.
	 */
	virtual void startThread();

	/** 
	 Join thread. Wait for its ending.

	 @remarks
	!!! You can join if the thread will stop (mIterationNumber != 0 or 
				mStopRequested = true).

	 */
	virtual void joinThread();

	/** 
	 Immediately stop the thread.
	 */
	virtual void shutDownThread();

	/** 
	 Request thread shut down.
	 */
	virtual void stopThread();

	/** 
	 Go forward from wait.
	 */
	virtual void go();

   protected:

	/** 
	 This is the thread method containing one or more (or inifinite) iterations.
	 */
	virtual void thread();

	/** 
	 One iteration step. This method calls the overridable task method.
	 @return If true, the thread is stopped.
	 */
	virtual bool iteration();

	/** 

			It is the task of the thread. It have to be overiden.

	 @remarks
	This method should be abstact. It is not, because of unit testing of this class.

	 */
	virtual void task();

	/** 
	 Wait for something.
	 */
	virtual void wait();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMThread_impl.h"

}

#endif
