
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMTimer.h
/// Contains declaration of class Timer

#ifndef __PCM_TIMER_H__
#  define __PCM_TIMER_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

//
// Outer includes
//

#  include <string>

#  include <sys/time.h>

//
// Scripting bind includes (for Squirrel)
//

#  include <squirrel/squirrel.h>
#  include <squirrel/sqstdio.h>
#  include <squirrel/sqstdaux.h>
#  include <squirrel/sqplus.h>

namespace ParCompMark
{

  /**
	Collection of time handling methods.
   */

  class Timer
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestTimer;
#  endif

	//
	// Scripting binding (for Squirrel)
	//

	/// @name Scripting binding
	//@{  

  /**
   Static glue code method for Squirrel-C++ binding. 
   This method should be call from each Squirrel virtual machines 
   to generate the proper Squirrel glue code.
   To content of this method is fully autogenerated in the header implementation file, 
   you should not modify it.
   */
   public:static void squirrelGlue()
	{
	 ::SqPlus::SQClassDef < Timer > (_T("Timer")).staticFunc(&Timer::getSystemTime,
								 _T("getSystemTime")).
		staticFunc(&Timer::getApplicationTime, _T("getApplicationTime")).staticFunc(&Timer::sleep, _T("sleep"));
	}

	//@}  

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** Constant for minimal time delay (1us).

	 @remarks
	 This is own attribute of this class.
	*/
	static const Real EPSILONDELAY;

	//@}  

	//
	// Class variables
	//

	/// @name Class variables
	//@{  

   protected:

	/** Time correction for this application according to the host of the commander mode application.

	 @remarks
	 This is own attribute of this class.
	*/
	static Real mTimeCorrection;

	/** (Uncorrected) time of the last write access to mTimeCorrection variable.

	 @remarks
	 This is own attribute of this class.
	*/
	static Real mStartTime;

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mTimeCorrection. Returns value of mTimeCorrection.

	 @return The value of mTimeCorrection
	*/
	static const Real & getTimeCorrection();

	/**
	 Setter of mTimeCorrection. Sets value of mTimeCorrection.

	 @param[in] timecorrection The value of mTimeCorrection
	*/
	static void setTimeCorrection(const Real & timecorrection);

	/**
	 Getter of mStartTime. Returns value of mStartTime.

	 @return The value of mStartTime
	*/
	static const Real & getStartTime();

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   public:

	/** 
	 Return corrected system time in seconds.
	 @return Corrected time in seconds
	 */
	static Real getSystemTime();

	/** 
	 Return (uncorrected) application time (getUncorrectedSystemTime() - mStartTime) in seconds.
	 @return Corrected application time in seconds
	 */
	static Real getApplicationTime();

	/** 
	 Return uncorrected system time in seconds.
	 @return Uncorrected time in seconds
	 */
	static Real getUncorrectedSystemTime();

	/** 
	 Return current time as a string.
	 @param[in] microseconds Flag indicates microseconds to display in output
	 @param[in] seconds Flag indicates seconds to display in output
	 @param[in] minutes Flag indicates minutes to display in output
	 @param[in] hours Flag indicates hours to display in output
	 @param[in] correction Time correction in seconds
	 @return Current time in string format
	 */
	static std::string getTimeString(const bool & microseconds = true, const bool & seconds =
					 true, const bool & minutes = true, const bool & hours =
					 true, const Real & correction = 0.0);

	/** 
	 Return current date as a string.
	 @param[in] days Flag indicates days to display in output
	 @param[in] months Flag indicates months to display in output
	 @param[in] years Flag indicates years to display in output
	 @param[in] correction Time correction in seconds
	 @return Current time in string format
	 */
	static std::string getDateString(const bool & days = true, const bool & months =
					 true, const bool & years = true, const Real & correction = 0.0);

	/** 
	 Return current time and date as a string.
	 @param[in] microseconds Flag indicates microseconds to display in output
	 @param[in] seconds Flag indicates seconds to display in output
	 @param[in] minutes Flag indicates minutes to display in output
	 @param[in] hours Flag indicates hours to display in output
	 @param[in] days Flag indicates days to display in output
	 @param[in] months Flag indicates months to display in output
	 @param[in] years Flag indicates years to display in output
	 @param[in] correction Time correction in seconds
	 @return Current time in string format
	 */
	static std::string getTimeDateString(const bool & microseconds = true, const bool & seconds =
					  true, const bool & minutes = true, const bool & hours =
					  true, const bool & days = true, const bool & months =
					  true, const bool & years = true, const Real & correction =
					  0.0);

	/** 
	 Sleep process for the specified seconds.
	 @param[in] time Time to sleep
	 */
	static void sleep(const Real time);

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMTimer_impl.h"

}

// Declare Timer class (and its enums) for SqPlus binding
/// \skip DECLARE_INSTANCE_TYPE_NAME
DECLARE_INSTANCE_TYPE_NAME(ParCompMark::Timer, Timer);

#endif
