
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file PCMXDisplay.h
/// Contains declaration of class XDisplay

#ifndef __PCM_XDISPLAY_H__
#  define __PCM_XDISPLAY_H__

//
// Inner includes
//

#  include "PCMPremeditations.h"

#  include "PCMContainer.h"

#  include "PCMPointer.h"

//
// Outer includes
//

#  include <X11/Xlib.h>

#  include <GL/glew.h>

#  include <GL/glx.h>

#  include <GL/glxext.h>

namespace ParCompMark
{

  /**
	Class that encapsulates an X Display.
   */

  class XDisplay
  {

	//
	// Unit test class
	//
#  ifdef PARCOMPMARK_TEST
	friend class ParCompMarkTest::TestXDisplay;
#  endif

	//
	// Typedefs
	//

   public:

  /** Type for pointer on this class. */
	typedef Pointer < XDisplay,
	Mutex > Pointer;

   protected:

  /** Struct for visual attributes. */
	typedef struct
	{			/* X visual attribs */
	 int id;
	 int klass;
	 int depth;
	 int redMask,
	  greenMask,
	  blueMask;
	 int colormapSize;
	 int bitsPerRGB;	/* GL visual attribs */
	 int supportsGL;
	 int transparentType;
	 int transparentRedValue;
	 int transparentGreenValue;
	 int transparentBlueValue;
	 int transparentAlphaValue;
	 int transparentIndexValue;
	 int bufferSize;
	 int level;
	 int rgba;
	 int doubleBuffer;
	 int stereo;
	 int auxBuffers;
	 int redSize,
	  greenSize,
	  blueSize,
	  alphaSize;
	 int depthSize;
	 int stencilSize;
	 int accumRedSize,
	  accumGreenSize,
	  accumBlueSize,
	  accumAlphaSize;
	 int numSamples,
	  numMultisample;
	 int visualCaveat;
	} VisualAttribs;

	//
	// Class constants
	//

	/// @name Class constants
	//@{  

   public:

	/** Constant for ignoring multisample.

	 @remarks
	 This is own attribute of this class.
	*/
	static const s32 IGNOREMULTISAMPLE;

	/** Constant for unknown display dimension.

	 @remarks
	 This is own attribute of this class.
	*/
	static const s32 UNKNOWNDIMENSION;

	//@}  

	//
	// Class variables
	//

	/// @name Class variables
	//@{  

   protected:

	/** X multithreading is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	static bool mXMTInitialized;

	/** X multithreading is supported.

	 @remarks
	 This is own attribute of this class.
	*/
	static bool mXMTSupported;

	/** Mutex for synchronization the class level error handling.

	 @remarks
	 This is own attribute of this class.
	*/
	static Mutex mErrorHandlerMutex;

	/** Skip Xlib errors. Do not use it.

	 @remarks
	 This is own attribute of this class.
	*/
	static bool mTolerateErrors;

	//@}  

	//
	// Variables
	//

	/// @name Variables
	//@{  

   protected:

	/** X display name.

	 @remarks
	 This is own attribute of this class.
	*/
	std::string mDisplayName;

	/** Wrapped X Display.

	 @remarks
	 This attribute references an attribute.
	*/
	::Display * mDisplay;

	/** The display is initialized.

	 @remarks
	 This is own attribute of this class.
	*/
	bool mInitialized;

	/** Width of the display in pixels (of the default screen).

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mWidth;

	/** Height of the display in pixels (of the default screen).

	 @remarks
	 This is own attribute of this class.
	*/
	u32 mHeight;

	//@}  

	//
	// Constructors & destructor
	//

	/// @name Constructors & destructor
	//@{  

   public:

	/** 
	 Create an X display.
	 @param[in] displayName X display name
	 */
	XDisplay(const std::string & displayName = "");

	/** 
	 The destructor. This class has virtual destructor. 
	 */
	virtual ~ XDisplay();

	//@}  

	//
	// Getters & setters
	//

	/// @name Getters & setters
	//@{  

   public:

	/**
	 Getter of mDisplayName. Returns value of mDisplayName.

	 @return The value of mDisplayName
	*/
	const std::string & getDisplayName() const;

	/**
	 Getter of mDisplay. Returns value of mDisplay.

	 @return The value of mDisplay
	*/
	::Display * getDisplay() const;

	/**
	 Getter of mInitialized. Returns value of mInitialized.

	 @return The value of mInitialized
	*/
	const bool & getInitialized() const;

	/**
	 Getter of mXMTInitialized. Returns value of mXMTInitialized.

	 @return The value of mXMTInitialized
	*/
	static const bool & getXMTInitialized();

	/**
	 Getter of mXMTSupported. Returns value of mXMTSupported.

	 @return The value of mXMTSupported
	*/
	static const bool & getXMTSupported();

	/**
	 Getter of mWidth. Returns value of mWidth.

	 @return The value of mWidth
	*/
	const u32 & getWidth() const;

	/**
	 Getter of mHeight. Returns value of mHeight.

	 @return The value of mHeight
	*/
	const u32 & getHeight() const;

	/**
	 Getter of mErrorHandlerMutex. Returns value of mErrorHandlerMutex.

	 @return The value of mErrorHandlerMutex
	*/
	static const Mutex & getErrorHandlerMutex();

	/**
	 Getter of mTolerateErrors. Returns value of mTolerateErrors.

	 @return The value of mTolerateErrors
	*/
	static const bool & getTolerateErrors();

	/**
	 Setter of mTolerateErrors. Sets value of mTolerateErrors.

	 @param[in] tolerateerrors The value of mTolerateErrors
	*/
	static void setTolerateErrors(const bool & tolerateerrors);

	//@}  

	//
	// Class methods
	//

	/// @name Class methods
	//@{  

   protected:

	/** 
	 X error handler.
	 @param[in] display X display
	 @param[in] errorEvent Error event
	 @return Return code of the handler.
	 */
	static int errorHandler(Display * display, XErrorEvent * errorEvent);

	//@}  

	//
	// Methods
	//

	/// @name Methods
	//@{  

   public:

	/** 
	 Initialize X display.
	 */
	virtual void initialize();

	/** 
	 Finalize X display.
	 */
	virtual void finalize();

	/** 
	 Flush the output buffer and then waits until all requests have been received and processed by the X server.
	 @param[in] discard Indicates whether synchronize() discards all events on the event queue
	 */
	virtual void synchronize(const bool & discard = false);

	/** 
	 Examine all visuals to find the so-called best one. We prefer deepest RGBA buffer with depth, 
			stencil and accum that has no caveats. This will only choose formats with a multisample that equals multisample.

	 @param[in] screenNumber Screen number to test
	 @param[in] multiSample Select specific multisample value
	 @return -1 in case of failure, otherwise a valid visual ID
	 */
	virtual s32 findBestVisual(const s32 & screenNumber, const s32 & multiSample =
				 XDisplay::IGNOREMULTISAMPLE);

	/** 
	 Get visual attributes for the specified visual info descriptor
	 @param[in] vInfo Visual info descriptor
	 @param[out] attribs Variable to hold the restrieved attributes
	 */
	virtual void getVisualAttribs(XVisualInfo * vInfo, VisualAttribs & attribs);

   protected:

	/** 
	 Initialize X multithreading, enable Xlib support for 
			concurrent threads. This function must be the first Xlib function a 
			multi-threaded program calls, and it must complete before any other 
			Xlib call is made. (Internally called at the first XDisplay initialization.)
	 */
	virtual void initializeMT();

	//@}  

  };

  //
  // Include header implementation
  //

#  include "PCMXDisplay_impl.h"

}

#endif
