//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Inner includes
//

#include "../include/PCMApplication.h"

#include "../include/PCMStringConverter.h"

//
// Outer includes
//

#include <getopt.h>
#include <iostream>
#include <fstream>
#include <signal.h>

namespace ParCompMark
{

  // Inherited static attribute initialization
  template <> Application * Singleton < Application >::mInstance = 0;

  //
  // Class constants
  //

  const std::string Application::PCAPINUT = "scripts/framework/pcapi.nut";
  const std::string Application::PARCOMPMARKNUT = "scripts/framework/parcompmark.nut";
  const std::string Application::UTILSNUT = "scripts/framework/utils.nut";
  const std::string Application::ABBREVIATIONSNUT = "scripts/framework/abbreviations.nut";
  const std::string Application::DYNAMICINITNUT = "scripts/framework/dynamic-init.nut";
  const std::string Application::UIINITNUT = "scripts/framework/ui-init.nut";
  const std::string Application::COMMANDDIRECTORY = "scripts/commands/";
  const std::string Application::HELPNUT = "scripts/commands/help.nut";
  const std::string Application::PCIMPLEMENTATIONS[] = {
	"libpcapi.so", "libparcomp.so"
  };

  const Application::CommandLineOption Application::mCommandLineOptions[] = {
	{'h', "help", " ........... : show help", false, Application::showHelp}
	,
	{'v', "version", " ........ : show version", false, Application::showVersion}
	,
	{'c', "commander", " ...... : enable commander mode", false, Application::setCommanderOn}
	,
	//{'g', "enable-gui", " ..... : enable GUI", false, Application::setGUIOn},
	//{'C', "cluster", " ........ : set cluster description", true, Application::setCluster},
	//{'l', "enable-lowlevel", ". : indicate the input script is a low-level script", false, Application::setLowLevelOn},
	//{'p', "parameters", " ..... : set input parameters from file", true, Application::setParameters},
	{'l', "low-level", " ...... : set low-level script file", true, Application::setLowLevelScriptFile}
	,
	{'d', "dynamic", " ........ : set cluster dynamic script file", true, Application::setDynamicScriptFile}
	,
	{'s', "scenario", " ....... : set scenario script file", true, Application::setScenarioScriptFile}
	,
	{'i', "input", " .......... : set intput script file", true, Application::setInput}
	,
	{'o', "output", " ......... : set output filename", true, Application::setOutput}
	,
	{'H', "hostcount", " ...... : set expected number of rendering hosts", true, Application::setExpectedHostCount}
	,
	{'u', "ui", " ............. : set user interface (none|console)", true, Application::setUserInterface}
  };

  const u32 Application::mCommandLineOptionCount = sizeof(mCommandLineOptions) / sizeof(mCommandLineOptions[0]);
  const std::string Application::mUsageString = "ParCompMark [ options ]";

  //
  // Class variables
  //

  std::string Application::mLibrarySearchPath = "/usr/lib";
  bool Application::mCommanderMode = false;
  bool Application::mGUIMode = false;
  bool Application::mManualClusterDescription = false;

  std::string Application::mClusterDescription;
  bool Application::mLowLevelMode = false;
  bool Application::mInteractiveParameters = true;

  std::string Application::mParameters;
  // '-' means stdin
  std::string Application::mInput = "-";
  std::string Application::mOutput = "-";
  int Application::mExpectedHostCount = 0;

  Application::UserInterface Application::mUserInterface = CONSOLE;

  //
  // Constructors & destructor
  //

  Application::Application()
	// You have to initialize the following attributes:
	// - mLibrarySearchPath
	// - mHostInfo
	// - mInitialized
	// - mPostCommand
	// - mQuiting
	// - mOutputIsDone
	// - mOutputDocument
	// - mCurrentExecutionOutputDocument
	// - mExecutionIndex
	// - mCollectClusterDescription
	// - mConfigOptions
	// - mLowLevelScript
	// - mDynamicScript
	// - mScenarioScript
	// - mSquirrelCommandList
	// - mDynamicScriptCompiled
	// - mServer
	// - mClient
	// - mCommanderMode
	// - mGUIMode
	// - mManualClusterDescription
	// - mClusterDescription
	// - mLowLevelMode
	// - mInteractiveParameters
	// - mParameters
	// - mInput
	// - mOutput
	// - mExpectedHostCount
	// - mUserInterface
	// - mCompRun
	// - mDynamicScriptSqVM
	// - mUserInterfaceSqVM
	// - mDynamicScriptParameters
  {
	mInitialized = false;
	mPostCommand = NOTHING;
	mQuiting = false;
	mOutputIsDone = true;

	mServer = 0;
	mClient = 0;

	mCompRun = false;

	// Initialize the application
	initialize();

	Logger::getInstance()->log(Logger::NOTICE, "PCM Application has been created.");
  }

 /*----------------------------------------------------------------------*/

  Application::~Application()
  {
	// Finalize the application
	if(mInitialized)
	 finalize();
  }

 /*----------------------------------------------------------------------*/

  //
  // Class methods
  //

  int Application::complement(int first, int second)
  {
	//printf("Not implemented yet:)\n");

	return 0;

  }

 /*----------------------------------------------------------------------*/

  void Application::parseCommandLine(const u32 & argc, const char **&argv)
  {
	// Create getopts structure
	struct option long_options[mCommandLineOptionCount + 1];

	for(u32 i = 0; i < mCommandLineOptionCount; i++)
	{
	 long_options[i].name = mCommandLineOptions[i].longName.c_str();
	 long_options[i].has_arg = mCommandLineOptions[i].hasArgument ? required_argument : no_argument;
	 long_options[i].flag = 0;
	 long_options[i].val = (int) mCommandLineOptions[i].shortName;
	}
	long_options[mCommandLineOptionCount].name = 0;
	long_options[mCommandLineOptionCount].has_arg = 0;
	long_options[mCommandLineOptionCount].flag = 0;
	long_options[mCommandLineOptionCount].val = 0;

	// Create getopts string
	char *optionString = new char[2 * mCommandLineOptionCount + 1],
	 *_optionString = optionString;

	for(u32 i = 0; i < mCommandLineOptionCount; i++)
	{
	 *_optionString = mCommandLineOptions[i].shortName;
	 _optionString++;
	 if(mCommandLineOptions[i].hasArgument)
	 {
		*_optionString = ':';
		_optionString++;
	 }
	}

	// Parse aruments
	std::string _dummy;
	for(;;)
	{

	 // getopt_long stores the option index here.
	 int option_index = 0;

	 int c = getopt_long(argc, (char *const *) argv, optionString, long_options, &option_index);

	 // Parse argument
	 if(c == -1)
		break;
	 for(u32 i = 0; i < mCommandLineOptionCount; i++)
		if(c == mCommandLineOptions[i].shortName && mCommandLineOptions[i].handler)
		 mCommandLineOptions[i].handler(mCommandLineOptions[i].hasArgument ? std::string(optarg) : _dummy);
	}

	delete[]optionString;
  }

 /*----------------------------------------------------------------------*/

  void Application::showHelp(const std::string & strarg)
  {
	// Show program information
	std::cout << getInstance()->getProgramInfo() << std::endl;

	// Show usage
	std::cout << "usage: " << mUsageString << std::endl;

	// Show options
	std::cout << std::endl << "Options:" << std::endl;
	for(u32 i = 0; i < mCommandLineOptionCount; i++)
	 std::cout << "\t-" << mCommandLineOptions[i].shortName << "\t--" << mCommandLineOptions[i].
		longName << mCommandLineOptions[i].description << std::endl;

	// Bug report information
	std::cout << std::endl << "Report bugs to <" << PACKAGE_BUGREPORT << ">" << std::endl;

	// Quit from application
	Application::getInstance()->_quit();
	exit(0);

  }

 /*----------------------------------------------------------------------*/

  void Application::showVersion(const std::string & strarg)
  {
	std::cout << PACKAGE_VERSION << std::endl;

	Application::getInstance()->_quit();
	exit(0);
  }

 /*----------------------------------------------------------------------*/

  void Application::setCommanderOn(const std::string & strarg)
  {
	mCommanderMode = true;
  }

 /*----------------------------------------------------------------------*/

  void Application::setGUIOn(const std::string & strarg)
  {
	mGUIMode = true;
  }

 /*----------------------------------------------------------------------*/

  void Application::setLowLevelOn(const std::string & strarg)
  {
	mLowLevelMode = true;
  }

 /*----------------------------------------------------------------------*/

  void Application::setLowLevelScriptFile(const std::string & strarg)
  {
	Application::getInstance()->getConfigOptions()->setString("lowLevelScriptFile", "scripts/low-level/" + strarg);
  }

 /*----------------------------------------------------------------------*/

  void Application::setDynamicScriptFile(const std::string & strarg)
  {
	Application::getInstance()->getConfigOptions()->setString("dynamicScriptFile", "scripts/dynamic/" + strarg);
  }

 /*----------------------------------------------------------------------*/

  void Application::setScenarioScriptFile(const std::string & strarg)
  {
	Application::getInstance()->getConfigOptions()->setString("scenarioScriptFile", "scripts/scenario/" + strarg);
  }

 /*----------------------------------------------------------------------*/

  void Application::setCluster(const std::string & strarg)
  {
	mManualClusterDescription = true;
	mClusterDescription = strarg;
  }

 /*----------------------------------------------------------------------*/

  void Application::setParameters(const std::string & strarg)
  {
	mInteractiveParameters = false;
	mParameters = strarg;
  }

 /*----------------------------------------------------------------------*/

  void Application::setInput(const std::string & strarg)
  {
	mInput = strarg;
  }

 /*----------------------------------------------------------------------*/

  void Application::setOutput(const std::string & strarg)
  {
	mOutput = strarg;
  }

 /*----------------------------------------------------------------------*/

  void Application::setExpectedHostCount(const std::string & hostCount)
  {
	if(hostCount.length())
	 mExpectedHostCount = StringConverter::toU32(hostCount);
	else
	 mExpectedHostCount = 0;
  }

 /*----------------------------------------------------------------------*/

  void Application::setUserInterface(const std::string & strUI)
  {
	if(strUI == "none")
	{
	 mUserInterface = NONE;
	} else if(strUI == "console")
	{
	 mUserInterface = CONSOLE;
	} else
	 Except(INVALID_ENUM_ERROR, "Application::setUserInterface()", strUI + " is not valid user interface.");
  }

 /*----------------------------------------------------------------------*/

  bool Application::hasEnvironmentVariable(const std::string & name)
  {
	char *buffer;

	buffer = getenv(name.c_str());
	return buffer;
  }

 /*----------------------------------------------------------------------*/

  std::string Application::getEnvironmentVariable(const std::string & name)
  {
	char *buffer;

	buffer = getenv(name.c_str());
	Assert(buffer, INVALID_NAME_ERROR, "Application::getEnvironmentVariable()");
	return buffer;
  }

 /*----------------------------------------------------------------------*/

  void Application::terminateHandler()
  {
	// Set NOP termination hander to avoid infinite loops
	std::set_terminate(Application::terminateHandlerNOP);

	if(Logger::getInstance())
	{
	 Logger::getInstance()->log(Logger::FATAL, "Uncaught exception !!!");
	 Logger::getInstance()->log(*Exception::getLastException());
	} else
	{
	 std::cerr << "Uncaught exception !!!" << std::endl;
	 std::cerr << Logger::translateException(*Exception::getLastException()) << std::endl;
	}

	// Try to finalize the application
	Application::getInstance()->finalize();
  }

 /*----------------------------------------------------------------------*/

  void Application::terminateHandlerNOP()
  {
	// Do nothing, this method is called when an exception is thrown during the terminateHandler()
  }

 /*----------------------------------------------------------------------*/

  void Application::unexpectedHandler()
  {
	Application::terminateHandler();
  }

 /*----------------------------------------------------------------------*/

  void Application::segfaultHandler(int value)
  {
	//asm volatile ("incl 0x44(%ebp)");
	//Timer::sleep(0.5);
  }

 /*----------------------------------------------------------------------*/

  void Application::interruptHandler(int value)
  {
	if(Logger::getInstance())
	{
	 Logger::getInstance()->log(Logger::NOTICE, "Interrupt signal caught.");
	} else
	{
	 std::cerr << "Interrupt signal caught." << std::endl;
	}

	// Quit from application
	Application::getInstance()->_quit();
	exit(0);
  }

 /*----------------------------------------------------------------------*/

  Application *Application::getInstance()
  {
	return Singleton < Application >::getInstance();
  }

 /*----------------------------------------------------------------------*/

  //
  // Methods
  //

  void Application::setupHandlers() const
  {
	std::set_terminate(Application::terminateHandler);
	std::set_unexpected(Application::unexpectedHandler);

	::signal(SIGINT, Application::interruptHandler);

	// TODO: later will be implemented
	//::signal(SIGSEGV, Application::segfaultHandler);
  }

 /*----------------------------------------------------------------------*/

  void Application::initialize()
  {
	Assert(!mInitialized, INVALID_OPERATION_ERROR, "Application::initialize()");

	// Reset timer correction
	Timer::setTimeCorrection(0.0);

	// Setup handlers
	setupHandlers();

	// Initialize file system manager
	FileSystemManager::createInstance();
	FileSystemManager::getInstance()->initialize();

	// Initialize logger to be able to log in constructors
	Logger::createInstance();
	Logger::getInstance()->initialize();

	// Create root of output document with namespace definitions
	mOutputDocument = new OutputNode("results", OutputNode::INFORMATION);
	mOutputDocument->setAttribute("xmlns:def", "http://www.it2.bme.hu");
	mOutputDocument->setAttribute("xmlns:info", "http://www.it2.bme.hu");
	mOutputDocument->setAttribute("xmlns:ref", "http://www.it2.bme.hu");
	mOutputDocument->setAttribute("xmlns:stat", "http://www.it2.bme.hu");
	mCurrentExecutionOutputDocument = 0;
	mExecutionIndex = 0;

	mCollectClusterDescription = new OutputNode("cluster", OutputNode::INFORMATION);

	// Create configuration options
	mConfigOptions = new ConfigOptions();
	mConfigOptions->initialize();

	// Create ini file if needed
	if(!FileSystemManager::getInstance()->existsAppFile(FileSystemManager::getInstance()->getIniFile()))
	{
	 // Default config options
	 mConfigOptions->setString("dataDirectory", FileSystemManager::getInstance()->findDataDirectory(),
				  "Path of application data directory");

	 mConfigOptions->setString("frameletIcon", "true", "Disable or enable framelet icon on rendering windows");

	 mConfigOptions->setString("broadcastPort", "1235", "Broadcast port used by parcompmark");

	 mConfigOptions->setString("communicationPort", "1234", "Communication port used by parcompmark");

	 // Search for PC library
	 std::vector < std::string > additionalPaths =
		Application::getInstance()->
		hasEnvironmentVariable("PC_LIBRARY_PATH") ? StringConverter::tokenize(Application::getInstance()->
										  getEnvironmentVariable
										  ("PC_LIBRARY_PATH"),
										  ":") : std::vector < std::string >
		();

	 // Place exotic library search paths
	 additionalPaths.push_back("/opt/paracomp/lib/");
	 additionalPaths.push_back("/opt/paracomp/lib64/");

	 u32 pcImplCount = sizeof(PCIMPLEMENTATIONS) / sizeof(PCIMPLEMENTATIONS[0]);

	 for(u32 i = 0; i < pcImplCount; i++)
		if(FileSystemManager::getInstance()->existsLibrary(PCIMPLEMENTATIONS[i], additionalPaths))
		 mConfigOptions->setString("pcLibraryPath",
					  FileSystemManager::getInstance()->findLibraryPath(PCIMPLEMENTATIONS[i],
												additionalPaths),
					  "PC library path");

	 if(!mConfigOptions->hasOption("pcLibraryPath"))
		Logger::getInstance()->log(Logger::WARNING,
					 "PC library path not found. You should specify it, probably you will get an error.");

	 mConfigOptions->saveToIniFile(FileSystemManager::getInstance()->getIniFile());
	}
	// Load ini file
	else
	{
	 mConfigOptions->loadFromIniFile(FileSystemManager::getInstance()->getIniFile());
	}

	// Call application-wide setters
	setFromConfigOptions();

	// Create plugin manager
	PluginManager::createInstance();
	PluginManager::getInstance()->initialize();

	// Initialize PC
	if(PCerr err = pcSystemInitialize((const PCstring) mLibrarySearchPath.c_str()) != PC_NO_ERROR)
	 Except(INTERNAL_ERROR, "Application::initialize()", pcGetErrorString(err));

	if(PCerr err = pcSessionCreate(0) != PC_NO_ERROR)
	 Except(INTERNAL_ERROR, "Application::initialize()", pcGetErrorString(err));

	rl_command_func_t *func;

	func = complement;

	rl_bind_key('\t', func);

	mInitialized = true;

	Logger::getInstance()->log(Logger::DEBUG, "PCM Application has been initialized.");

  }

 /*----------------------------------------------------------------------*/

  void Application::finalize()
  {
	Assert(mInitialized, INVALID_OPERATION_ERROR, "Application::finalize()");

	// Specific finalization code
	// (Finalize soldier first)

	finalizeSoldier();
	if(mCommanderMode)
	 finalizeCommander();

	if(PCerr err = pcSessionDestroy() != PC_NO_ERROR)
	 Except(INTERNAL_ERROR, "Host::finalize()", pcGetErrorString(err));

	if(mDynamicScriptSqVM.isNotNull())
	 mDynamicScriptSqVM.kill();

	if(mUserInterfaceSqVM.isNotNull())
	 mUserInterfaceSqVM.kill();

	if(PluginManager::getInstance())
	 PluginManager::destroyInstance();

	if(FileSystemManager::getInstance())
	 FileSystemManager::destroyInstance();

	if(mConfigOptions.isNotNull())
	 mConfigOptions.kill();

	if(mOutputDocument.isNotNull())
	 mOutputDocument.kill();

	if(Logger::getInstance())
	{
	 Logger::getInstance()->log(Logger::DEBUG, "PCM Application has been finalized.");
	 Logger::destroyInstance();
	}

	mInitialized = false;
  }

 /*----------------------------------------------------------------------*/

  void Application::finalizeCommander()
  {
	Cluster::destroyInstance();
	delete mServer;
  }

 /*----------------------------------------------------------------------*/

  void Application::finalizeSoldier()
  {
	// Kill the soldier host
	if(Host::getInstance())
	{
	 Host::getInstance()->closeXDisplays();
	 Host::destroyInstance();
	}

	if(mClient)
	 delete mClient;
  }

 /*----------------------------------------------------------------------*/

  void Application::setFromConfigOptions()
  {
	FileSystemManager::getInstance()->setDataDirectory(mConfigOptions->getString("dataDirectory"));
	if(mConfigOptions->hasOption("communicationPort"))
	{
	 Network::setCommunicationPort((unsigned short) StringConverter::
					toU32(mConfigOptions->getString("communicationPort")));
	}

	if(mConfigOptions->hasOption("broadcastPort"))
	{
	 Network::setBroadcastPort((unsigned short) StringConverter::
				  toU32(mConfigOptions->getString("broadcastPort")));
	}

	if(mConfigOptions->hasOption("pcLibraryPath"))
	{
	 setLibrarySearchPath(mConfigOptions->getString("pcLibraryPath"));
	}
	Logger::getInstance()->log(Logger::DEBUG,
				 "Application-wide setters has been called on configuration options (parcompmark.ini).");

  }

 /*----------------------------------------------------------------------*/

  void Application::createVirtualMachines()
  {
	// Create virtual machine for dynamic script exection
	mUserInterfaceSqVM = new SqVM("UserInterface-VM");

	// Load PC API, ParCompMark API, and utils
	mUserInterfaceSqVM.lock();
	mUserInterfaceSqVM->runScriptFromFile(FileSystemManager::getInstance()->getPathDataFile(PCAPINUT),
					  SqVM::NOMAINMETHOD);
	mUserInterfaceSqVM->runScriptFromFile(FileSystemManager::getInstance()->getPathDataFile(PARCOMPMARKNUT),
					  SqVM::NOMAINMETHOD);
	mUserInterfaceSqVM->runScriptFromFile(FileSystemManager::getInstance()->getPathDataFile(UTILSNUT),
					  SqVM::NOMAINMETHOD);
	mUserInterfaceSqVM->runScriptFromFile(FileSystemManager::getInstance()->getPathDataFile(ABBREVIATIONSNUT),
					  SqVM::NOMAINMETHOD);
	mUserInterfaceSqVM.unlock();

	// Load user commands
	std::list < std::string > files = FileSystemManager::getInstance()->listDataDirectory(COMMANDDIRECTORY);
	mUserInterfaceSqVM.lock();

	// Save current script output, and switch to console output 
	SqVM::ScriptOutput lastOutput = SqVM::getScriptOutput();
	SqVM::setScriptOutput(SqVM::STD);

	// Load help.nut first
	mUserInterfaceSqVM->runScriptFromFile(FileSystemManager::getInstance()->getPathDataFile(HELPNUT),
					  SqVM::NOMAINMETHOD);

	// Load commands
	for(std::list < std::string >::iterator i = files.begin(); i != files.end(); i++)
	 // Take only the .nut files
	 if((*i).substr((*i).length() - 4) == ".nut")
		mUserInterfaceSqVM->runScriptFromFile(FileSystemManager::getInstance()->getPathDataFile(*i),
						  SqVM::NOMAINMETHOD);

	SqVM::setScriptOutput(lastOutput);
	mUserInterfaceSqVM.unlock();

	// Load dynamic script initialization code
	mDynamicScriptSqVM = mUserInterfaceSqVM;
	mDynamicScriptSqVM.lock();
	mDynamicScriptSqVM->runScriptFromFile(FileSystemManager::getInstance()->getPathDataFile(DYNAMICINITNUT),
					  SqVM::NOMAINMETHOD);
	mDynamicScriptSqVM.unlock();
  }

 /*----------------------------------------------------------------------*/

  bool Application::startOperation()
  {
	Logger::getInstance()->log(Logger::NOTICE, "PCM Application starts its operation.");

	// Return codes
	bool retC = false,
	 retS = false,
	 retUI = false;

	// Commander mode (= commander + soldier + user interface)
	if(mCommanderMode)
	{

	 // Start a commander
	 retS = soldierOperation();

	 // And a soldier too
	 if(!retS)
		retC = commanderOperation();

	 // Start user interface
	 if(!retS && !retC)
	 {
		switch (mUserInterface)
		{
		 case CONSOLE:
			retUI = textUserInterface();
			break;
		 default:
			retUI = noUserInterface();
		}
	 }
	 // Wait for output document
	 waitForOutput();

	}
	// Soldier mode (= soldier only)
	else
	{
	 // Start only a soldier
	 retS = soldierOperation();

	 // Wait for thread exit
	 mClient->joinThread();
	}

	Logger::getInstance()->log(Logger::NOTICE, "PCM Application has finished its operation.");

	// Return true is either operation has error
	return retC || retS || retUI;
  }

 /*----------------------------------------------------------------------*/

  void Application::writeOutput()
  {
	mOutputDocument.lock();

	// Put the cluster description into the output document
	mCollectClusterDescription.lock();
	mOutputDocument->createChildNode(mCollectClusterDescription->serialize2XML());
	mCollectClusterDescription.unlock();

	// Compose xml document
	std::ostringstream xmlSourceStream;
	//std::ostringstream ccd;
	xmlSourceStream << "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" << std::endl;
	xmlSourceStream << mOutputDocument->serialize2XML() << std::endl;

	// Write output to std out
	if(mOutput == "-")
	{
	 std::cout << xmlSourceStream.str() << std::endl;

	 Logger::getInstance()->log(Logger::NOTICE, "Benchmark output has been written to standard output.");
	}
	// Write output to file
	else
	{
	 std::fstream file_op(mOutput.c_str(), std::ios::out);
	 file_op << xmlSourceStream.str();
	 //file_op << ccd.str();
	 file_op.close();

	 Logger::getInstance()->log(Logger::NOTICE, "Benchmark output has been written to `" + mOutput + "\'");
	}

	mOutputDocument.unlock();

	// The output document is written now. It is safe to quit
	mOutputIsDone = true;
  }

 /*----------------------------------------------------------------------*/

  void Application::waitForOutput()
  {
	while(!mOutputIsDone)
	{
	 Timer::sleep(0.5);
	 Thread::yield();
	}
  }

 /*----------------------------------------------------------------------*/

  void Application::quit()
  {
	mPostCommand = (mPostCommand == STOP || mPostCommand == STOP_QUIT ? STOP_QUIT : QUIT);
  }

 /*----------------------------------------------------------------------*/

  void Application::compileDynamic()
  {
	mPostCommand = (mPostCommand == START || mPostCommand == COMPILE_START ? COMPILE_START : COMPILE);
  }

 /*----------------------------------------------------------------------*/

  void Application::start()
  {
	mPostCommand = (mPostCommand == COMPILE || mPostCommand == COMPILE_START ? COMPILE_START : START);
  }

 /*----------------------------------------------------------------------*/

  void Application::stop()
  {
	mPostCommand = (mPostCommand == QUIT || mPostCommand == STOP_QUIT ? STOP_QUIT : STOP);
  }

 /*----------------------------------------------------------------------*/

  void Application::autoDetection()
  {
	Assert(!mCompRun, INVALID_NAME_ERROR, "Application::autoDetection()");
	mServer->buildCluster();

	// New cluster, new low-level script
	mDynamicScriptCompiled = false;

	// Refresh cluster description
	// TODO: parsing not works

	/**
	if(mOutputDocument->getChildCount())
	 mOutputDocument->getFirstChildNode() = new OutputNode(Cluster::getInstance()->serialize2XML());
	else
	 mOutputDocument->createChildNode(Cluster::getInstance()->serialize2XML());
	*/

	//calculateMessageSendTime();
  }

 /*----------------------------------------------------------------------*/

  void Application::cleanup()
  {
	mOutputDocument->clean();
	mCurrentExecutionOutputDocument = 0;
  }

 /*----------------------------------------------------------------------*/

  char *Application::getHostListForSquirrel()
  {
	std::ostringstream osstr;
	u32 _i = 0,
	 _size = Cluster::getInstance()->getHosts()->getSize();

	osstr << '[';
	for(Container < HostInfo, Mutex >::Iterator i = Cluster::getInstance()->getHosts()->begin();
	 i != Cluster::getInstance()->getHosts()->end(); i++, _i++)
	 osstr << '\"' << i->first << (_i + 1 != _size ? "\"," : "\"");
	osstr << ']';

	// TODO: is it deallcated by Squirrel?
	return strdup(osstr.str().c_str());
  }

 /*----------------------------------------------------------------------*/

  char *Application::getLowLevelScriptForSquirrel()
  {
	// TODO: is it deallcated by Squirrel?
	return (char *) mLowLevelScript.c_str();
  }

 /*----------------------------------------------------------------------*/

  char *Application::getDynamicScriptForSquirrel()
  {
	// TODO: is it deallcated by Squirrel?
	return (char *) mDynamicScript.c_str();
  }

 /*----------------------------------------------------------------------*/

  bool Application::hasDynamicScript()
  {
	return mDynamicScript.size() > 0;
  }

 /*----------------------------------------------------------------------*/

  bool Application::hasScenarioScript()
  {
	return mScenarioScript.size() > 0;
  }

 /*----------------------------------------------------------------------*/

  char *Application::getDynamicScriptParametersForSquirrel()
  {
	std::ostringstream osstr;

	osstr << '[' << std::endl;
	for(std::list < DynamicScriptParameter * >::iterator i = mDynamicScriptParameters.begin();
	 i != mDynamicScriptParameters.end(); i++)
	 osstr << "{name=\"" << (*i)->name << "\"}" << std::endl;
	osstr << ']';

	// TODO: is it deallcated by Squirrel?
	return strdup(osstr.str().c_str());
  }

 /*----------------------------------------------------------------------*/

  char *Application::getProgramInfo()
  {
	std::ostringstream osstr;
	osstr << PACKAGE_STRING << ' ' << PACKAGE_COPYRIGHT;
	return strdup(osstr.str().c_str());
  }

 /*----------------------------------------------------------------------*/

  void Application::calculateMessageSendTime()
  {
	mServer->sendBroadcastMessage("TIME", "NONE");
  }

 /*----------------------------------------------------------------------*/

  void Application::_pseudoStop()
  {

	Assert(mCompRun, INVALID_VALUE_ERROR, "Application::_stop()");

	mServer->sendBroadcastMessage("COLLECTDATA", "NONE");

	//writeOutput();

	mCompRun = false;

  }

 /*----------------------------------------------------------------------*/

  bool Application::commanderOperation()
  {
	Assert(mCommanderMode, INVALID_OPERATION_ERROR, "Application::commanderOperation()");
	Logger::getInstance()->log(Logger::NOTICE, "Starting commander-mode code.");

	try
	{
	 // Create cluster
	 Cluster::createInstance();

	 // Create and initialize virtual machines for dynamic script execution and for user interaction handling
	 createVirtualMachines();

	 // Load low level script if specified
	 if(mConfigOptions->hasOption("lowLevelScriptFile"))
		loadLowLevelScript(mConfigOptions->getString("lowLevelScriptFile").c_str());

	 // Load dynamic script if specified
	 else if(mConfigOptions->hasOption("dynamicScriptFile"))
	 {
		loadDynamicScript(mConfigOptions->getString("dynamicScriptFile").c_str());
		doPostCommand();
	 }
	 // Load scenario script if specified
	 else if(mConfigOptions->hasOption("scenarioScriptFile"))
	 {
		loadScenarioScript(mConfigOptions->getString("scenarioScriptFile").c_str());
		doPostCommand();
	 }
	 // Start netserver
	 mServer = new NetServer("server");
	 mServer->startThread();

	 // Initialize user interface
	 mUserInterfaceSqVM.lock();
	 SqVM::ScriptOutput lastOutput = SqVM::getScriptOutput();
	 SqVM::setScriptOutput(SqVM::STD);
	 mUserInterfaceSqVM->runScriptFromFile(FileSystemManager::getInstance()->getPathDataFile(UIINITNUT),
						SqVM::NOMAINMETHOD);
	 SqVM::setScriptOutput(lastOutput);
	 mUserInterfaceSqVM.unlock();

	}
	catch(Exception & e)
	{
	 Logger::getInstance()->log(Logger::FATAL,
				   "An exception has been raised during the commander-mode operation!");
	 Logger::getInstance()->log(e);
	 return true;
	}

	Logger::getInstance()->log(Logger::NOTICE, "Commander-mode code has been finished.");

	return false;
  }

 /*----------------------------------------------------------------------*/

  bool Application::soldierOperation()
  {
	Logger::getInstance()->log(Logger::NOTICE, "Starting soldier-mode code.");

	try
	{
	 mHostInfo = new HostInfo(Network::getHostName(), OutputNode::INFORMATION);

	 mHostInfo->refreshData();

	 // Create host

	 Host::createInstance();
	 Host::getInstance()->openXDisplays();
	 mClient = new NetClient("client");
	 Host::getInstance()->setNetClient(mClient);
	 mClient->startThread();
	}
	catch(Exception & e)
	{
	 Logger::getInstance()->log(Logger::FATAL,
				   "An exception has been raised during the soldier-mode operation!");
	 Logger::getInstance()->log(e);
	 return true;
	}

	Logger::getInstance()->log(Logger::NOTICE, "Soldier-mode code has been finished.");

	return false;
  }

 /*----------------------------------------------------------------------*/

  bool Application::textUserInterface()
  {

	for(;;)
	{
#if defined(HAVE_READLINE_HISTORY_H) && defined(HAVE_READLINE_READLINE_H)
	 char *s = 0;

	 s = readline("\n: ");

	 if(s && *s)
		add_history(s);
#else
	 char s[1024];

	 printf("\n: ");
	 gets(s);
#endif

	 if(executeUserCommand(s))
		return true;

#if defined(HAVE_READLINE_HISTORY_H) && defined(HAVE_READLINE_READLINE_H)
	 free(s);
#endif

	 // Process post commands
	 doPostCommand();

	 // Process command list
	 processSquirrelCommandList();

	 if(mQuiting)
		break;
	}

	return false;
  }

 /*----------------------------------------------------------------------*/

  bool Application::noUserInterface()
  {
	// Wait for a while
	Timer::sleep(0.5);

	// Start compositing 
	executeUserCommand("start");
	doPostCommand();

	// Process command list
	processSquirrelCommandList();

	if(!mCompRun)
	 return true;

	for(;;)
	{
	 Timer::sleep(0.5);

	 // Quit when the compositing is done
	 if(!mCompRun)
		executeUserCommand("quit");

	 doPostCommand();

	 if(mQuiting)
		break;
	}

	// Wait for a while
	Timer::sleep(1.0);

	return false;
  }

 /*----------------------------------------------------------------------*/

  bool Application::executeUserCommand(const std::string & command)
  {
	try
	{
	 mUserInterfaceSqVM.lock();

	 // Save current script output, and switch to console output 
	 SqVM::ScriptOutput lastOutput = SqVM::getScriptOutput();
	 SqVM::setScriptOutput(SqVM::STD);

	 // Resolve abbreviation
	 std::list < std::string > params;
	 params.push_back(command);
	 std::string resolved_command =
		mUserInterfaceSqVM->runScriptFromString("", "", "resolve_abbreviation", params, true);

	 // Execute user command
	 mUserInterfaceSqVM->runScriptFromString(resolved_command, "", SqVM::NOMAINMETHOD);

	 // Restore original output
	 SqVM::setScriptOutput(lastOutput);
	 mUserInterfaceSqVM.unlock();
	}
	catch(const Exception & e)
	{
	 // Unlock VM
	 if(mUserInterfaceSqVM.getLocked())
		mUserInterfaceSqVM.unlock();

	 // Clear error flag
	 if(e.getType() == Exception::SCRIPT_ERROR)
		mUserInterfaceSqVM->setError(false);

	 // If not script error, throw forward
	 else
		throw Exception(e);
	}

	return false;
  }

 /*----------------------------------------------------------------------*/

  bool Application::processSquirrelCommandList()
  {
	while(mSquirrelCommandList.size())
	{

	 // Wait until compositing is running
	 while(mCompRun)
		Timer::sleep(0.5);

	 executeUserCommand(mSquirrelCommandList.front());
	 doPostCommand();

	 mSquirrelCommandList.pop_front();
	 std::cout << "There are " << mSquirrelCommandList.size() << " commands in the queue." << std::endl;
	}

	return false;
  }

 /*----------------------------------------------------------------------*/

  void Application::doPostCommand()
  {
	switch (mPostCommand)
	{
	 case LOADSCENARIO:
		_loadScenario();
	 case LOADDYNAMIC:
		_loadDynamic();
		break;
	 case COMPILE:
		_compileDynamic();
		break;
	 case START:
		_start();
		break;
	 case COMPILE_START:
		_compileDynamic();
		_start();
		break;
	 case STOP:
		_stop();
		break;
	 case STOP_QUIT:
		_stop();
		_quit();
		break;
	 case QUIT:
		_quit();
		break;
	 default:
		return;
	}

	mPostCommand = NOTHING;
  }

 /*----------------------------------------------------------------------*/

  void Application::loadLowLevelScript(const char *filename)
  {
	mLowLevelScript = FileSystemManager::getInstance()->readDataTextFile(filename);
  }

 /*----------------------------------------------------------------------*/

  void Application::loadDynamicScript(const char *filename)
  {
	// New script
	mDynamicScript = FileSystemManager::getInstance()->readDataTextFile(filename);

	// Not compiled
	mDynamicScriptCompiled = false;

	mPostCommand = LOADDYNAMIC;
  }

 /*----------------------------------------------------------------------*/

  void Application::loadScenarioScript(const char *filename)
  {
	// New script
	mScenarioScript = FileSystemManager::getInstance()->readDataTextFile(filename);

	mPostCommand = LOADSCENARIO;
  }

 /*----------------------------------------------------------------------*/

  void Application::pushSquirrelCommand(const char *command)
  {
	mSquirrelCommandList.push_back(command);
  }

 /*----------------------------------------------------------------------*/

  void Application::createLowLevelScript(const std::string & clusterDescription)
  {
	try
	{
	 // Create low level script
	 mDynamicScriptSqVM.lock();
	 std::list < std::string > params;
	 params.push_back(clusterDescription);
	 mLowLevelScript =
		mDynamicScriptSqVM->runScriptFromString(mDynamicScript, "", "createLowLevelScript", params, true);
	 mDynamicScriptSqVM.unlock();

	 mDynamicScriptCompiled = true;
	}
	catch(const Exception & e)
	{
	 // Unlock VM
	 if(mDynamicScriptSqVM.getLocked())
		mDynamicScriptSqVM.unlock();

	 // Clear error flag
	 if(e.getType() == Exception::SCRIPT_ERROR)
		mDynamicScriptSqVM->setError(false);

	 // If not script error, throw forward
	 else
		throw Exception(e);
	}
  }

 /*----------------------------------------------------------------------*/

  void Application::processScenarioScript()
  {
	try
	{
	 // Create low level script
	 mDynamicScriptSqVM.lock();
	 mDynamicScriptSqVM->runScriptFromString(mScenarioScript, "", "prepareScenario");
	 mDynamicScriptSqVM.unlock();

	 mDynamicScriptCompiled = true;
	}
	catch(const Exception & e)
	{
	 // Unlock VM
	 if(mDynamicScriptSqVM.getLocked())
		mDynamicScriptSqVM.unlock();

	 // Clear error flag
	 if(e.getType() == Exception::SCRIPT_ERROR)
		mDynamicScriptSqVM->setError(false);

	 // If not script error, throw forward
	 else
		throw Exception(e);
	}
  }

 /*----------------------------------------------------------------------*/

  void Application::setEnvironmentVariablesToSquirrel()
  {
	XDisplay::Pointer commanderDisplay =
	 Host::getInstance()? Host::getInstance()->getFirstXDisplay() : XDisplay::Pointer();

	// Create environment variables
	std::map < std::string, std::string > envVars;
	envVars["commanderHost"] = Network::getHostName();
	if(commanderDisplay.getPtr())
	 envVars["screenWidth"] = StringConverter::toString(commanderDisplay->getWidth());
	if(commanderDisplay.getPtr())
	 envVars["screenHeight"] = StringConverter::toString(commanderDisplay->getHeight());
	std::ostringstream environmentVariables;
	environmentVariables << "{";
	for(std::map < std::string, std::string >::iterator i = envVars.begin(); i != envVars.end(); i++)
	 environmentVariables << i->first << "=\"" << i->second << "\" ";
	environmentVariables << "}";

	try
	{
	 // Transmit these parameters to the squirrel
	 mDynamicScriptSqVM.lock();
	 std::list < std::string > params;
	 params.push_back(environmentVariables.str());
	 mDynamicScriptSqVM->runScriptFromString(mDynamicScript, "", "setEnvironmentVariables", params);
	 mDynamicScriptSqVM.unlock();
	}
	catch(const Exception & e)
	{
	 // Unlock VM
	 if(mDynamicScriptSqVM.getLocked())
		mDynamicScriptSqVM.unlock();

	 // Clear error flag
	 if(e.getType() == Exception::SCRIPT_ERROR)
		mDynamicScriptSqVM->setError(false);

	 // If not script error, throw forward
	 else
		throw Exception(e);
	}
  }

 /*----------------------------------------------------------------------*/

  void Application::initializeDynamicScriptParameters(const bool & reset)
  {
	mDynamicScriptSqVM.lock();
	std::list < std::string > params;
	params.push_back(reset ? "true" : "false");
	mDynamicScriptSqVM->runScriptFromString(mDynamicScript, "", "initializeDynamicScriptParameters", params);
	mDynamicScriptSqVM.unlock();
  }

 /*----------------------------------------------------------------------*/

  void Application::retrieveDynamicScriptParameters()
  {
	Assert(mDynamicScript.length() > 0, INVALID_OPERATION_ERROR, "Application::retrieveDynamicScriptParameters()");

	try
	{
	 // Get script parameters from dynamic script as XML
	 mDynamicScriptSqVM.lock();
	 std::string dScriptParamsXML =
		mDynamicScriptSqVM->runScriptFromString(mDynamicScript, "dynamicScriptParameters", "getParameters",
							std::list < std::string > (), true);
	 mDynamicScriptSqVM.unlock();

	 //std::cerr << dScriptParamsXML << std::endl;

	 // Drop current parameters
	 mDynamicScriptParameters.clear();

	 // Parse XML (TODO: move this to SqVM) 
	 TiXmlDocument tiDocument;

	 tiDocument.Parse(dScriptParamsXML.c_str());

	 TiXmlElement *tiArray = tiDocument.RootElement();

	 for(TiXmlElement * tiElement = tiArray->FirstChildElement(); tiElement != 0;
		tiElement = tiElement->NextSiblingElement())
	 {
		TiXmlElement *tiTable = tiElement->FirstChildElement();

		if(!tiTable)
		 continue;

		DynamicScriptParameter *dsparam = new DynamicScriptParameter();

		for(TiXmlElement * tiSlot = tiTable->FirstChildElement(); tiSlot != 0;
		 tiSlot = tiSlot->NextSiblingElement())
		{
		 const char *name = tiSlot->Attribute("name");
		 const char *text = tiSlot->NoChildren()? 0 : tiSlot->GetText();

		 if(!strcmp(name, "name"))
		 {
			dsparam->name = text ? text : "";
		 } else if(!strcmp(name, "type"))
		 {
			Assert(text, NULL_POINTER_ERROR, "Application::retrieveDynamicScriptParameters()");
			if(!strcmp(text, "string"))
			 dsparam->type = STRING;
			else if(!strcmp(text, "integer") || !strcmp(text, "int"))
			 dsparam->type = INTEGER;
			else if(!strcmp(text, "real") || !strcmp(text, "float"))
			 dsparam->type = REAL;
			else if(!strcmp(text, "boolean") || !strcmp(text, "bool"))
			 dsparam->type = BOOL;
			else
			 Except(INVALID_ENUM_ERROR, "Application::retrieveDynamicScriptParameters()",
				 std::string("Invalid dynamic script type parameter `") + text + "\'");
		 } else if(!strcmp(name, "description"))
			dsparam->description = text ? text : "";
		 else if(!strcmp(name, "possibleValues"))
		 {

			/**TiXmlElement *tiArray2 = tiSlot->FirstChildElement();

			for(TiXmlElement * tiElement2 = tiArray2->FirstChildElement(); tiElement2 != 0;
			 tiElement2 = tiElement2->NextSiblingElement())
			{
			 if(!tiElement2->NoChildren())
			 {
				std::string text2 = tiElement2->GetText();
				StringConverter::trim(text2);
				dsparam->possibleValues.insert(text2);
			 }
			}*/
		 } else if(!strcmp(name, "defaultValue"))
			dsparam->defaultValue = text ? text : "";
		 else
			Except(INVALID_ENUM_ERROR, "Application::retrieveDynamicScriptParameters()",
			   std::string("Invalid dynamic script parameter `") + name + "\'");
		}
		mDynamicScriptParameters.push_back(dsparam);
		//std::cerr << dsparam->name << "|" << dsparam->description << "|" << dsparam->defaultValue << "|" << std::endl;
	 }
	}
	catch(const Exception & e)
	{
	 // Unlock VM
	 if(mDynamicScriptSqVM.getLocked())
		mDynamicScriptSqVM.unlock();

	 // Clear error flag
	 if(e.getType() == Exception::SCRIPT_ERROR)
		mDynamicScriptSqVM->setError(false);

	 // If not script error, throw forward
	 else
		throw Exception(e);
	}
  }

 /*----------------------------------------------------------------------*/

  void Application::_loadScenario()
  {
	processScenarioScript();
  }

 /*----------------------------------------------------------------------*/

  void Application::_loadDynamic()
  {
	// Set environment variables to Squirrel
	setEnvironmentVariablesToSquirrel();

	// Compile dynamic script and initialize dynamic script parameters with reset
	initializeDynamicScriptParameters(true);
  }

 /*----------------------------------------------------------------------*/

  void Application::_compileDynamic()
  {
	// If a dynamic script is given, generate low level script
	if(mDynamicScript.length())
	{
	 // Get squirrel cluster description
	 std::string clusterDescription = Cluster::getInstance()->serialize2Squirrel();

	 // Create low-level script from dynamic script and cluster description 
	 createLowLevelScript(clusterDescription);
	}
  }

 /*----------------------------------------------------------------------*/

  void Application::_start()
  {
	Assert(!mCompRun, INVALID_NAME_ERROR, "Application::_start()");

	// Verify low level script existence
	if(!mLowLevelScript.length())
	 Except(INVALID_OPERATION_ERROR, "Application::start()",
		 "Unable to send low-level scripts to the soldier nodes. You have to specify an explicit low-level script or a dynamic script.");

	// Create new execution node
	mCurrentExecutionOutputDocument = mOutputDocument->createChildNode("execution", OutputNode::INFORMATION);
	mCurrentExecutionOutputDocument->setAttribute("index", StringConverter::toString(mExecutionIndex++));

	// Get dynamic script parameters from squirrel as XML
	std::string dynamicParameters = "<info:parameters/>";

	// If we have dynamic script, get the parameters from squirrel
	if(hasDynamicScript())
	{

	 try
	 {
		// Get dynamic script parameters from squirrel
		mDynamicScriptSqVM.lock();
		dynamicParameters =
		 mDynamicScriptSqVM->
		 runScriptFromString
		 ("function main() { return \"<info:parameters>\" + objectToXML(DYNAMICSCRIPTPARAMETERS) + \"</info:parameters>\" }",
		  "", "main", std::list < std::string > (), true);
		mDynamicScriptSqVM.unlock();
	 }
	 catch(const Exception & e)
	 {
		// Unlock VM
		if(mDynamicScriptSqVM.getLocked())
		 mDynamicScriptSqVM.unlock();

		// Clear error flag
		if(e.getType() == Exception::SCRIPT_ERROR)
		 mDynamicScriptSqVM->setError(false);

		// If not script error, throw forward
		else
		 throw Exception(e);
	 }

	}

	mCurrentExecutionOutputDocument->createChildNode(dynamicParameters);
	mCollectClusterDescription->clean();

	mServer->sendBroadcastMessage("INIT",
				  StringConverter::toString(Timer::getUncorrectedSystemTime(),
								StringConverter::DEFAULTFIELDWIDTH, 6));

	// Compositing is running and the output is not written yet
	mCompRun = true;
	mOutputIsDone = false;
  }

 /*----------------------------------------------------------------------*/

  void Application::_stop()
  {
	Assert(mCompRun, INVALID_NAME_ERROR, "Application::_stop()");

	NetServer::IntPointer frameID;
	frameID = mServer->getStopFrameID();
	frameID.lock();
	*frameID.getPtr() = 0;
	frameID.unlock();

	mServer->sendBroadcastMessage("STOP", "NONE");

	mServer->sendBroadcastMessage("FRAMEID", StringConverter::toString(*frameID.getPtr() + 5));

	//writeOutput();  

	mCompRun = false;
  }

 /*----------------------------------------------------------------------*/

  void Application::_quit()
  {

	if(!mQuiting)
	{

	 // Do not assert, quiting is quiting
	 if(mCompRun)
		_stop();

	 writeOutput();

	 mQuiting = true;
	 mServer->sendBroadcastMessage("QUIT", "NONE");
	}

  }

 /*----------------------------------------------------------------------*/

}
