
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Inner includes
//

#include "../include/PCMBuffer.h"

#include "../include/PCMStringConverter.h"

#include "../include/PCMNode.h"

//
// Outer includes
//

#include <IL/il.h>
#include <IL/ilu.h>

namespace ParCompMark
{

  //
  // Constructors & destructor
  //

  Buffer::Buffer()
	// You have to initialize the following attributes:
	// - mParent
	// - mLeft
	// - mTop
	// - mWidth
	// - mHeight
	// - mOwnPointers
	// - mColour
	// - mDepth
	// - mDepthFormat
	// - mOutputRowPixel
	// - mInitialized
  {
	Except(INVALID_OPERATION_ERROR, "Buffer::Buffer()",
	   "Default constructor for Squirrel compatibility. Calling this constructor always raises an exception.");
  }

 /*----------------------------------------------------------------------*/

  Buffer::Buffer(const PCuint & left, const PCuint & top, const PCuint & width,
		 const PCuint & height, const PCuint & depthFormat, Node * parent):
	// Initialize parent node 
   mParent(parent		/* Parent node */
	)
	// You have to initialize the following attributes:
	// - mParent
	// - mLeft
	// - mTop
	// - mWidth
	// - mHeight
	// - mOwnPointers
	// - mColour
	// - mDepth
	// - mDepthFormat
	// - mOutputRowPixel
	// - mInitialized
  {
	mLeft = left;
	mTop = top;
	mWidth = width;
	mHeight = height;
	mOwnPointers = true;
	mColour = 0;
	mDepth = 0;
	mDepthFormat = depthFormat;
	mOutputRowPixel = 0;
	mInitialized = false;

	Logger::getInstance()->log(Logger::NOTICE, "Buffer on node `" + mParent->getName() + "\' has been created.");
  }

 /*----------------------------------------------------------------------*/

  Buffer::~Buffer()
  {
	if(mInitialized)
	 finalize();

	Logger::getInstance()->log(Logger::NOTICE, "Buffer on node `" + mParent->getName() + "\' has been destroyed.");
  }

 /*----------------------------------------------------------------------*/

  //
  // Methods
  //

  void Buffer::initialize()
  {
	mColour = new PCuint[mWidth * mHeight];

	switch (mDepthFormat)
	{
	 case PC_PF_Z32I:
		mDepth = new PCuint[mWidth * mHeight];
		break;
	 case PC_PF_Z32F:
		mDepth = new float[mWidth * mHeight];

		break;
	 default:
		Except(INVALID_ENUM_ERROR, "Buffer::initialize()",
		   "Depth format `" + StringConverter::toString(mDepthFormat) + "\' on node `" +
		   mParent->getName() + "\' is invalid.");
	}

	mInitialized = true;

	Logger::getInstance()->log(Logger::DEBUG, "Buffer on node `" + mParent->getName() + "\' has been initialized.");
  }

 /*----------------------------------------------------------------------*/

  void Buffer::finalize()
  {
	Assert(mInitialized, INVALID_OPERATION_ERROR, "Buffer::finalize()");

	freeBuffers();

	mInitialized = false;

	Logger::getInstance()->log(Logger::DEBUG, "Buffer on node `" + mParent->getName() + "\' has been finalized.");
  }

 /*----------------------------------------------------------------------*/

  void Buffer::saveToFile(const std::string & filename)
  {
	ilInit();
	iluInit();
	ilEnable(IL_FILE_OVERWRITE);

	// Create img
	ILuint img;

	ilGenImages(1, &img);
	ilBindImage(img);

	// Set image
	ilTexImage(mWidth,	// Width
		 mHeight,	// Height
		 1,		// Depth
		 4,		// Number of channels
		 IL_BGRA,	// Format
		 IL_UNSIGNED_BYTE,	// Type
		 (ILvoid *) mColour	// Data
	 );

	// Save image
	ilSaveImage((char *) filename.c_str());

	// Delete img
	ilDeleteImages(1, &img);
  }

 /*----------------------------------------------------------------------*/

  void Buffer::freeBuffers()
  {
	if(mOwnPointers && mColour)
	{
	 delete[]mColour;

	 mColour = 0;
	}

	if(mOwnPointers && mDepth)
	{
	 switch (mDepthFormat)
	 {
		case PC_PF_Z32I:
		 delete[](PCuint *) mDepth;
		 break;
		case PC_PF_Z32F:
		 delete[](float *) mDepth;

		 break;
		default:
		 Except(INTERNAL_ERROR, "Buffer::finalize()",
			 "Something has changed depth format on node `" + mParent->getName() +
			 "\' to invalid value: `" + StringConverter::toString(mDepthFormat) + "\' .");
	 }
	 mDepth = 0;
	}
  }

 /*----------------------------------------------------------------------*/

}
