
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// Inner includes
//

#include "../include/PCMGLXGLContext.h"

#include "../include/PCMGLXRenderWindow.h"

#include "../include/PCMStringConverter.h"

namespace ParCompMark
{

  //
  // Constructors & destructor
  //

  GLXGLContext::GLXGLContext(XDisplay::Pointer & display, GLXRenderWindow * glxWindow,::XVisualInfo * visualInfo):
	// X Display pointer initializer 
  mDisplay(display		/* Corresponding X Display */
  ),
	// Renderwindow initializer 
  mGLXWindow(glxWindow	/* Corresponding GLX rendering window. */
	)
	// You have to initialize the following attributes:
	// - mInitialized
	// - mDisplay
	// - mGLXWindow
	// - mVisualInfo
	// - mGLXContext
	// - mTainted
  {
	Assert(display.isNotNull() && glxWindow && visualInfo, NULL_POINTER_ERROR, "GLXGLContext::GLXGLContext()");

	mInitialized = false;
	mGLXContext = 0;
	mTainted = false;

	// Copy visual info
	mVisualInfo = new::XVisualInfo();
	memcpy(mVisualInfo, visualInfo, sizeof(::XVisualInfo));

	Logger::getInstance()->log(Logger::NOTICE,
				 "GLX Context for window `" + mGLXWindow->getCaption() + "\' has been created");
  }

 /*----------------------------------------------------------------------*/

  GLXGLContext::~GLXGLContext()
  {
	if(mInitialized)
	 finalize();

	// Delete visual info
	Assert(mVisualInfo, NULL_POINTER_ERROR, "GLXGLContext::finalize()");
	delete mVisualInfo;

	Logger::getInstance()->log(Logger::NOTICE,
				 "GLX Context for window `" + mGLXWindow->getCaption() + "\' has been destroyed");
  }

 /*----------------------------------------------------------------------*/

  //
  // Methods
  //

  void GLXGLContext::initialize()
  {
	Assert(!mInitialized, INVALID_OPERATION_ERROR, "GLXGLContext::initialize()");

	// Lock and synchronize
	mDisplay.lock();
	mDisplay->synchronize();

	// Create GLX context
	mGLXContext = glXCreateContext(mDisplay->getDisplay(), mVisualInfo, NULL, True);
	Assert(mGLXContext, INVALID_OPERATION_ERROR, "GLXGLContext::initialize()");

	mDisplay.unlock();

	mInitialized = true;
	Logger::getInstance()->log(Logger::DEBUG,
				 "GLX Context for window `" + mGLXWindow->getCaption() + "\' has been initialized: " +
				 StringConverter::toString((void *) mGLXContext));
  }

 /*----------------------------------------------------------------------*/

  void GLXGLContext::finalize()
  {
	Assert(mInitialized, INVALID_OPERATION_ERROR, "GLXGLContext::finalize()");

	// Lock and synchronize
	mDisplay.lock();
	mDisplay->synchronize();

	// Destroy GLX context if it is not tainted
	Assert(mGLXContext, NULL_POINTER_ERROR, "GLXGLContext::finalize()");
	if(!mTainted)
	 glXDestroyContext(mDisplay->getDisplay(), mGLXContext);

	mDisplay.unlock();

	mInitialized = false;
	Logger::getInstance()->log(Logger::DEBUG,
				 "GLX Context for window `" + mGLXWindow->getCaption() + "\' has been finalized");
  }

 /*----------------------------------------------------------------------*/

  void GLXGLContext::setCurrent()
  {
	Assert(mInitialized, INVALID_OPERATION_ERROR, "GLXGLContext::setCurrent()");

	mDisplay.lock();
	if(glXMakeCurrent(mDisplay->getDisplay(), mGLXWindow->getWindow(), mGLXContext) != True)
	{
	 Except(INTERNAL_ERROR, "GLXGLContext::setCurrent()", "calling glXMakeCurrent");
	}
	mDisplay.unlock();
  }

 /*----------------------------------------------------------------------*/

  void GLXGLContext::releaseCurrent()
  {
	Assert(mInitialized, INVALID_OPERATION_ERROR, "GLXGLContext::releaseCurrent()");

	mDisplay.lock();
	if(glXMakeCurrent(mDisplay->getDisplay(), None, 0) != True)
	{
	 Except(INTERNAL_ERROR, "GLXGLContext::releaseCurrent()", "calling glXMakeCurrent");
	}
	mDisplay.unlock();
  }

 /*----------------------------------------------------------------------*/

}
