#!/bin/bash 
#-----------------------------------------------------------------------------
#
# 	This file is a part of ParCompMark
# 	Parallel Compositing Benchmark Framework
#
# 	for latest info see http://parcompmark.sourceforge.net
#
# 	Copyright (C) 2006 IT2 ParCompMark Dev. Team
#
# 	Original template (sva_startx.sh) license:
#
#	© Copyright 2005 Hewlett-Packard Development Company, L.P.
#	
#	Confidential computer software. Valid license from HP required for
#	possession, use or copying. Consistent with FAR 12.211 and 12.212,
#	Commercial Computer Software, Computer Software Documentation, and 
#	Technical Data for Commercial Items are licensed to the U.S. Government
#	under vendor's standard commercial license.
#-----------------------------------------------------------------------------
#------------------------------------------------------------------------------
# MODULE: parcompmark.sh
# FACILITY: Scalable Visualization Cluster
# ABSTRACT:
#
#   Script to run the 'parcompmark' program on a SVA cluster
#
#   Starting template : sva_startx.sh
#
# VERSION:
#------------------------------------------------------------------------------
export theProgramName=${0##*/}
export theUpperProgramName=`echo $theProgramName|tr a-z A-Z`
export theProgramVersion="0.4"

# Find parcompmark binary (developer version first in the same directory)
if [ -f ./parcompmark ]; then
    PARCOMPMARK_BINARY="./parcompmark"
else
    PARCOMPMARK_BINARY="parcompmark"
fi

#------------------------------------------------------------------------------
# Module header and display function for help
DisplayHelp() {
more <<EOF
${theUpperProgramName}(1)           SVA samples          ${theUpperProgramName}(1)

NAME
    parcompmark.sh - script to run 'parcompmark' Parallel Compositing Benchmark and 
	Execution Framework on a SVA cluster

SYNOPSIS
    parcompmark.sh [-p partition] [-g geomtery] [-l|--local] [-w nodes] 
	[-a parcompmark-parameters]

DESCRIPTION
    Allocate resources, launch services and run the 'parcompmark' program.

    Options

    -p, --partition=partition_name
        Request resources from the SLURM partition "partition_name". 
        The default partition is used if no partition is named.
        1
    -w nodes
        Pass a list on nodes to run the sample on. e.g., devc[10-15], svawhite[8-15]

    -l --local
        Specify this option if you want the master to run on this node

    Help Options

    -h, -help 
        Show this help message.
   
    -u, --usage
        Display a brief usage message.

    Other Options

    -v, --version
        Display the version of this script.

    -V, --verbose
        Display debugging information as the script runs.
     
    -D, --debug
        Turn on script debugging.

    Allocate

    Launch

    Run

    Terminiate

EXAMPLES

    To run on nodes devc[5-1], use

         $ ./parcompmark.sh -w devc[5,1]

           The display will be shown on devc5

    If you are already in an X desktop, you could use -l to make the current node the
    master (results are shown on this node)

         $ ./parcompmark.sh -w svaopt[8-10] -l

BUGS

SEE ALSO
    svadisplaysurface, sinfo(1)

${theProgramName} ${theProgramVersion}       January 2005         ${theUpperProgramName}(1)
EOF
}
#------------------------------------------------------------------------------
# Display function for module usage
svaDisplayUsage() {
    echo "Usage: ${theProgramName} [-p partition] [-g geomtery] -w nodes [-l|--local] [-a parcompmark-parameters]"
}

#------------------------------------------------------------------------------
# FUNCTION: svaAllocateJob
# PURPOSE:
#   Call this function to allocate resources based on the settings in the 
#   following environment variables and to start a SLURM job with those
#   resources.
#       theProgramName
#       SVA_Compute_Count
#       SVA_Display_Count
#       SVA_Render_Count
#       SVA_Display_Surface
#   There are important side effects of making this call.
#   o The JOB data file is created and populated with the tags for the 
#     JOB section and a section for each host allocated to the job.
#   o JOB variables are set for the running job.
#   o A SLURM job is started. The calling script is loaded in the context
#     of the job.
#------------------------------------------------------------------------------
function svaAllocateJobRawNodes() {
    svaDisplayVerbose "Allocate job resources"
    # Set the render and compute args and default to 0 if they aren't 
    # specified.
    render_arg="-r ${SVA_Render_Count:-0}"
    compute_arg="-c ${SVA_Compute_Count:-0}"

    # Add options for processing remote graphics (remote 3d) jobs
    if [[ $SVA_USE_REMOTE3D -eq 1 ]] ; then
        remote_arg="-R"
        if [[ ${SVA_Remote_Render_Only:-0} -eq 1 ]] ; then
            remote_arg="$remote_arg --onlyrender"
        fi
    else
        remote_arg=""
    fi

    #

    # Initialize global variables with settings from the allocation.
    # These values are set by the svaselect call.
    SVA_Execution_Host=`svagetdata SVA_JOB SVA_EXECUTION_HOST`
    SVA_Display_Surface=""
    SVA_Display_Hosts=""
    SVA_Display_Count=0

    SVA_Render_Hosts=$SVA_Desired_Hosts
    SVA_Render_Count=`expandnodes $SVA_Desired_Hosts | wc -w`

    SVA_Total_Count=`expandnodes $SVA_Desired_Hosts | wc -w`
    SVA_Allocate_Hosts=$SVA_Desired_Hosts

    svaDisplayVerbose "  Total allocated hosts: ${SVA_Total_Count} ${SVA_Allocate_Hosts}"


    if [ -z "$SVA_NUM_TILES" ] ; then
	SVA_NUM_TILES=`svagetdata SVA_CLUSTER SVA_NUM_TILES`
	SVA_TILE_ORIENTATION=`svagetdata SVA_CLUSTER SVA_TILE_ORIENTATION`
    fi

    if [[ $((SVA_Display_Count+SVA_Render_Count)) -gt 0 ]] ; then
        # License check 
        (svalmclient -c $((SVA_Display_Count+SVA_Render_Count)) -p $$ & sleep 1)
        if [[ `svagetdata SVA_JOB SVA_LIC_STATUS` != "OK" ]]; then 
            svaDisplayError "Unable to acquire $((SVA_Display_Count+SVA_Render_Count)) licenses."
            exit ${status}
        fi
        svaDisplayVerbose "License status is" `svagetdata SVA_JOB SVA_LIC_STATUS`
    fi
    
    # Now allocate the resources using SLURM and reload the job script in the 
    # context of the SLURM job
    export JOB_VARIABLE=`srun -N${SVA_Total_Count} \
        -w${SVA_Allocate_Hosts} \
        -J${theProgramName:-parcompmark} \
        -AI --no-shell `

    status=$?

    if [[ ${JOB_VARIABLE} == "" ]]; then
	echo "Couldn't Start job!"
	exit 1
    fi

    export ${JOB_VARIABLE}

    if [[ ${status} -ne 0 ]]; then
        kill -TERM `svagetdata SVA_JOB SVA_LIC_PID`
    fi

    return ${status}
}




# Initialize the SVA data. This sets the temporary data file name
# which is used until the job is allocated.
. svainit
if [[ $? -ne 0 ]]; then
    svaDisplayError "Unable to perform SVA initialization."
    exit
fi

#------------------------------------------------------------------------------
# If the job id is not defined, then process command-line options and 
# allocate resources for the job.

if [[ -z "${SLURM_JOBID}" ]]; then

    # Setup job default from user configuration files if they exist
    svaGetJobDefaults
    export DMX_INPUT_SERVER=$DISPLAY
    SVA_Render_Count=0
fi

_Use_Local_Host=0

_ParCompMark_Parameters=""

# Process Command-Line options
while [[ -n "$1" ]]; do
    case "$1" in
	"--help" | "-h" | "-?") DisplayHelp;exit;;
        "--usage" | "-u") svaDisplayUsage;exit;;
	"--version" | "-V") svaDisplayVersion;exit;;
	"--verbose" | "-v") SVA_Verbose_Flag=1; shift;continue;;
	"--debug" | "-D") set -x;SVA_Debug_Flag=1; shift;continue;;
	"--where" | "-w") shift;SVA_Desired_Hosts=$1;shift;continue;;
	"--partition" | "-p") shift;SVA_Partition_Name=$1;shift;continue;;
	"--tile_geometry" | "-g") shift;SVA_Tile_Geometry=$1;shift;continue;;
	"--local" | "-l") shift;_Use_Local_Host=1;continue;;
	"--args" | "-a") shift; 
	    while [[ -n "$1" ]]; do
		_ParCompMark_Parameters="${_ParCompMark_Parameters} $1"
		shift
	    done
	continue;;
	esac
       # The value in $1 isn't a local option, so break out of the local
       # option process
	break
	done

if [ -z $SVA_Tile_Geometry ] ; then
    SVA_Tile_Geometry="800x600"
fi

# Test to see if the requested resolution is one of the supported
# resolutions.
SVA_Supported_Resolutions="`svagetdata SVA_CLUSTER SVA_SUPPORTED_RESOLUTIONS`"
if [ -z "$SVA_Supported_Resolutions" ] ; then
    SVA_Supported_Resolutions="1280x1024,1400x1050,1600x1200"
fi

SVA_Supported_Resolutions=`echo ${SVA_Supported_Resolutions} | tr ',' ' '`
resolution_is_supported=0
for resolution in $SVA_Supported_Resolutions ; do
    if [[ "$SVA_Tile_Geometry" == "$resolution" ]] ; then
	resolution_is_supported=1
    fi
done

if  [[ $resolution_is_supported -eq 0 ]] ; then
    echo "${SVA_Tile_Geometry} is not one of the supported SVA tile geometries."
    echo "Please choose from ${SVA_Supported_Resolutions}."
    exit 1
fi

SVA_UNCLUTTER=0

if [[ ${SVA_Usage_Error} -ne 0 ]]; then svaDisplayUsage;exit;fi

if [ -z "$SVA_Desired_Hosts" ] ; then

    ${PARCOMPMARK_BINARY} -c ${_ParCompMark_Parameters} --hostcount 1

else

if [[ -z "${SLURM_JOBID}" ]]; then
    # ALLOCATE JOB
    svaAllocateJobRawNodes
fi

# Start X Servers on allocated hosts
svaDisplayVerbose "Start X servers"
export SVA_X_Options="-ac `svaChooseXConfig` ${SVA_X_Options}"
export SVA_XServer_Delay=0

svaRunRenderXServers -nowait
svaRunRenderHosts sva_wait_for_X.sh :0

if [ $? -ne 0 ]; then
    echo "Error Starting X servers on Render Nodes.";
    scancel $SLURM_JOBID
    # Wait for everthing to terminate
    wait
    exit 1;
fi 

echo 
echo "================================================================================"
echo "SVA: Started the X server(s). Now running sample app."
echo "================================================================================"
SESSIONID=10
ALLNODES=`expandnodes ${SVA_Allocate_Hosts}`
echo "Allocated hosts = $ALLNODES"
if [ ${_Use_Local_Host} -eq 1 ] ; then
    ALLNODES="$HOSTNAME $ALLNODES"
    MASTER=$HOSTNAME
else
    MASTER=`echo $ALLNODES | sed -e "s/\([a-z0-9]*\).*/\1/"`
fi

SLAVES=`echo $ALLNODES | sed -e "s/[a-z0-9]*\(.*\)/\1/"`
export DISPLAY=:0
echo "Master = $MASTER"
if [ ${_Use_Local_Host} -eq 0 ] ; then
    echo "Running window manager on $MASTER"
    srun -N1 -w $MASTER /usr/bin/metacity &
fi
for n in $SLAVES; do
    echo "Running slave piece on $n"
    srun -N1 -w $n ${PARCOMPMARK_BINARY} &
done
echo "Running master piece on $MASTER"
if [ ${_Use_Local_Host} -eq 1 ] ; then
   ${PARCOMPMARK_BINARY} -c ${_ParCompMark_Parameters} -H $((1+SVA_Render_Count))
else
   srun -N1 -w $MASTER ${PARCOMPMARK_BINARY} -c ${_ParCompMark_Parameters} -H $((SVA_Render_Count))
fi

#Kill the rest of job
scancel $SLURM_JOBID

# Wait for everything to terminate
wait

fi


#------------------------------------------------------------------------------
# MODULE END: parcompmark.sh
#------------------------------------------------------------------------------
