
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestContainer.cpp
/// Test case implementation for class Container.

#include "../include/TestContainer.h"
#include "../../include/PCMFileSystemManager.h"

namespace ParCompMarkTest
{

 /** Test class (for container test) */
  struct MyClass
  {
	int i;
	typedef ParCompMark::Pointer < MyClass,
	Mutex > Pointer;
  };

 /** Test container */
  typedef Container < MyClass,
   Mutex > MyContainer;

  void TestContainer::setUp()
  {
	std::string currentLogFile = "logs/TestContainer.log";

	if(Logger::getInstance() && Logger::getInstance()->getLogFileName() != currentLogFile)
	 Logger::destroyInstance();

	if(!FileSystemManager::getInstance())
	{
	 FileSystemManager::createInstance();
	 FileSystemManager::getInstance()->setAppDirectory("./");
	 FileSystemManager::getInstance()->initialize();
	}

	if(!Logger::getInstance())
	{
	 Logger::createInstance();
	 Logger::getInstance()->setLogFileName(currentLogFile);
	 Logger::getInstance()->initialize();
	}
  }

 /*----------------------------------------------------------------------*/

  void TestContainer::tearDown()
  {
  }

 /*----------------------------------------------------------------------*/

  //
  // Constructor & destructor tests
  //

  void TestContainer::test_constructor()
  {
	CPPUNIT_ASSERT_NO_THROW(MyContainer::Pointer container(new MyContainer()));
  }

 /*----------------------------------------------------------------------*/

  void TestContainer::test_destructor()
  {
	// Tested in contstructor
  }

 /*----------------------------------------------------------------------*/

  //
  // Method tests
  //

  void TestContainer::test_add_std__string_ElementPointer()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Add an element.
	 */

	// Create container
	MyContainer::Pointer container(new MyContainer());

	// Create element
	MyClass::Pointer element(new MyClass);

	// Add element
	CPPUNIT_ASSERT(!container->has("element"));
	CPPUNIT_ASSERT_NO_THROW(container->add("element", element));
	CPPUNIT_ASSERT(container->has("element"));

	// Add again
	CPPUNIT_ASSERT_THROW(container->add("element", element), ParCompMark::Exception);
	CPPUNIT_ASSERT(container->has("element"));
  }

 /*----------------------------------------------------------------------*/

  void TestContainer::test_get_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Get an element by name.
	 */

	// Create container
	MyContainer::Pointer container(new MyContainer());
	CPPUNIT_ASSERT_THROW(container->get("element"), ParCompMark::Exception);

	// Create element
	MyClass::Pointer element(new MyClass);

	// Add element
	CPPUNIT_ASSERT_NO_THROW(container->add("element", element));
	CPPUNIT_ASSERT(container->get("element") == element);
  }

 /*----------------------------------------------------------------------*/

  void TestContainer::test_has_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Search for an element by name.
	 */

	// Tested in add and get test methods
  }

 /*----------------------------------------------------------------------*/

  void TestContainer::test_remove_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Remove an element by name.
	 */

	// Create container
	MyContainer::Pointer container(new MyContainer());
	CPPUNIT_ASSERT_THROW(container->remove("element"), ParCompMark::Exception);

	// Create element
	MyClass::Pointer element(new MyClass);

	// Add element
	CPPUNIT_ASSERT_NO_THROW(container->add("element", element));
	CPPUNIT_ASSERT_NO_THROW(container->remove("element"));
	CPPUNIT_ASSERT(!container->has("element"));
  }

 /*----------------------------------------------------------------------*/

  void TestContainer::test_getSize()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Return the number of elements.
	 */

	// Create container
	MyContainer::Pointer container(new MyContainer());
	CPPUNIT_ASSERT(container->getSize() == 0);

	// Create elements
	MyClass::Pointer element1(new MyClass), element2(new MyClass);

	// Add element
	CPPUNIT_ASSERT_NO_THROW(container->add("element1", element1));
	CPPUNIT_ASSERT(container->getSize() == 1);

	// Add another element
	CPPUNIT_ASSERT_NO_THROW(container->add("element2", element2));
	CPPUNIT_ASSERT(container->getSize() == 2);

	// Remove elements
	CPPUNIT_ASSERT_NO_THROW(container->remove("element1"));
	CPPUNIT_ASSERT(container->getSize() == 1);
	CPPUNIT_ASSERT_NO_THROW(container->remove("element2"));
	CPPUNIT_ASSERT(container->getSize() == 0);
  }

 /*----------------------------------------------------------------------*/

  void TestContainer::test_isEmpty()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Return true if the container is empty.
	 */

	// Tested in getSize method
  }

 /*----------------------------------------------------------------------*/

  void TestContainer::test_begin()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Begin iterator on the container.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  void TestContainer::test_end()
  {

	/*
	 * You have to verify the following:
	 * 
	 * End iterator on the container.
	 */

	CPPUNIT_FAIL("Implement this test!");
  }

 /*----------------------------------------------------------------------*/

  // To register the suite we add:
  /// \skip CPPUNIT_TEST_SUITE_REGISTRATION
  CPPUNIT_TEST_SUITE_REGISTRATION(TestContainer);
}
