
//
// This source file is a part of ParCompMark
// Parallel Compositing Benchmark Framework
//
// for latest info see http://parcompmark.sourceforge.net

//
// Copyright (C) 2006 IT2 ParCompMark Dev. Team
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

/// @file TestXDisplay.cpp
/// Test case implementation for class XDisplay.

#include "../include/TestXDisplay.h"
#include "../../include/PCMFileSystemManager.h"
#include "../../include/PCMStringConverter.h"

namespace ParCompMarkTest
{

  void TestXDisplay::setUp()
  {
	std::string currentLogFile = "logs/TestXDisplay.log";

	if(Logger::getInstance() && Logger::getInstance()->getLogFileName() != currentLogFile)
	 Logger::destroyInstance();

	if(!FileSystemManager::getInstance())
	{
	 FileSystemManager::createInstance();
	 FileSystemManager::getInstance()->setAppDirectory("./");
	 FileSystemManager::getInstance()->initialize();
	}

	if(!Logger::getInstance())
	{
	 Logger::createInstance();
	 Logger::getInstance()->setLogFileName(currentLogFile);
	 Logger::getInstance()->initialize();
	}
  }

 /*----------------------------------------------------------------------*/

  void TestXDisplay::tearDown()
  {
  }

 /*----------------------------------------------------------------------*/

  //
  // Constructor & destructor tests
  //

  void TestXDisplay::test_constructor_cstd__string()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Create an X display.
	 */

	// Create display with default string
	XDisplay::Pointer d1(new XDisplay());
	d1.kill();

	// Create display with specified string
	XDisplay::Pointer d2(new XDisplay(":0.0"));
	d2.kill();
  }

 /*----------------------------------------------------------------------*/

  void TestXDisplay::test_destructor()
  {
	// Tested at constructor
  }

 /*----------------------------------------------------------------------*/

  //
  // Class method tests
  //

  void TestXDisplay::test_errorHandler_Display_p_XErrorEvent_p()
  {

	/*
	 * You have to verify the following:
	 * 
	 * X error handler.
	 */

	XDisplay::Pointer d(new XDisplay());
	CPPUNIT_ASSERT_NO_THROW(d->initialize());

	// Generate fake error message
	XErrorEvent errorEvent;

	errorEvent.type = 0;
	errorEvent.display = d->getDisplay();
	errorEvent.resourceid = 0;
	errorEvent.serial = 0;
	errorEvent.error_code = 0;
	errorEvent.request_code = 0;
	errorEvent.minor_code = 0;

	CPPUNIT_ASSERT_THROW(XDisplay::errorHandler(d->getDisplay(), &errorEvent), ParCompMark::Exception);
  }

 /*----------------------------------------------------------------------*/

  //
  // Method tests
  //

  void TestXDisplay::test_initialize()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Initialize X display.
	 */

	// Create display with default string
	XDisplay::Pointer d(new XDisplay());
	CPPUNIT_ASSERT_NO_THROW(d->initialize());

	// Create display with uncorrect string
	XDisplay::Pointer derr(new XDisplay(":0.1234"));
	CPPUNIT_ASSERT_THROW(derr->initialize(), ParCompMark::Exception);
  }

 /*----------------------------------------------------------------------*/

  void TestXDisplay::test_finalize()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Finalize X display.
	 */

	// Tested at initialize
  }

 /*----------------------------------------------------------------------*/

  void TestXDisplay::test_synchronize_cbool()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Flush the output buffer and then waits until all requests have been received and processed by the X server.
	 */

	// Test flush
	{
	 XDisplay::Pointer d(new XDisplay());
	 CPPUNIT_ASSERT_NO_THROW(d->initialize());
	 CPPUNIT_ASSERT_NO_THROW(d->synchronize());
	}

	// Test discard
	{
	 XDisplay::Pointer d(new XDisplay());
	 CPPUNIT_ASSERT_NO_THROW(d->initialize());
	 CPPUNIT_ASSERT_NO_THROW(d->synchronize(true));
	}
  }

 /*----------------------------------------------------------------------*/

  void TestXDisplay::test_findBestVisual_cs32_cs32()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Examine all visuals to find the so-called best one. We prefer deepest RGBA buffer with depth, 
	 * stencil and accum that has no caveats. This will only choose formats with a multisample that equals multisample.
	 * 
	 */

	XDisplay::Pointer d(new XDisplay());
	CPPUNIT_ASSERT_NO_THROW(d->initialize());

	int screen = DefaultScreen(d->getDisplay());
	int best_visual = -1;

	for(int fsaa = 8; fsaa > -1; fsaa--)
	{
	 CPPUNIT_ASSERT_NO_THROW(best_visual = d->findBestVisual(screen, fsaa));
	 if(best_visual == -1)
		Logger::getInstance()->log(Logger::DEBUG,
					 "FSAA " + StringConverter::toString(fsaa) + " is not supported.");
	 else
		Logger::getInstance()->log(Logger::DEBUG, "FSAA " + StringConverter::toString(fsaa) + " is supported.");
	}
  }

 /*----------------------------------------------------------------------*/

  void TestXDisplay::test_getVisualAttribs_XVisualInfo_p_VisualAttribs()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Get visual attributes for the specified visual info descriptor
	 */

	XDisplay::Pointer d(new XDisplay());
	CPPUNIT_ASSERT_NO_THROW(d->initialize());

	XDisplay::VisualAttribs attribs;
	XVisualInfo theTemplate;
	XVisualInfo *visuals;
	int numVisuals;
	long mask;

	// get list of all visuals on this screen 
	theTemplate.screen = DefaultScreen(d->getDisplay());
	mask = VisualScreenMask;
	visuals = XGetVisualInfo(d->getDisplay(), mask, &theTemplate, &numVisuals);
	if(numVisuals == 0 || visuals == 0)
	{
	 // something went wrong 
	 if(visuals)
		XFree(visuals);
	 CPPUNIT_ASSERT_MESSAGE(false, "numVisuals == 0 || visuals == 0");
	}

	CPPUNIT_ASSERT_NO_THROW(d->getVisualAttribs(&visuals[0], attribs));
  }

 /*----------------------------------------------------------------------*/

  void TestXDisplay::test_initializeMT()
  {

	/*
	 * You have to verify the following:
	 * 
	 * Initialize X multithreading, enable Xlib support for 
	 * concurrent threads. This function must be the first Xlib function a 
	 * multi-threaded program calls, and it must complete before any other 
	 * Xlib call is made. (Internally called at the first XDisplay initialization.)
	 */

	XDisplay::Pointer d(new XDisplay());
	CPPUNIT_ASSERT_NO_THROW(d->initialize());

	CPPUNIT_ASSERT_NO_THROW(d->initializeMT());
	CPPUNIT_ASSERT(XDisplay::mXMTInitialized);
  }

 /*----------------------------------------------------------------------*/

  // To register the suite we add:
  /// \skip CPPUNIT_TEST_SUITE_REGISTRATION
  CPPUNIT_TEST_SUITE_REGISTRATION(TestXDisplay);
}
